@@ -2,7 +2,7 @@
 ###
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
-# Copyright 2005-2012 The R Core team
+### Copyright 2005-2016  The R Core team
 
 #
 #  This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,12 @@
 #  http://www.r-project.org/Licenses/
 #
 
+### FIXME: For larger problems, the approach of corFactor() / corMatrix() of
+### ------ of size sumLenSq := sum(len^2), len := table(groups) sucks !
+### Conceptually, one could consider using something like  Matrix::sparseVector(.)
+### but would have to deal with corresponding C code as well !
+
+
 ##*## Generics that should be implemented for any corStruct class
 
 corFactor <-
@@ -34,15 +40,24 @@ corMatrix <-
 ### There is no constructor function for this class (i.e. no function
 ### called corStruct) because the class is virtual.
 
+## --- We now check 'sumLenSq' when it is first computed!
+## .chkLenSq <- function(nn) {
+##   if(nn > .Machine$integer.max)
+##     stop(gettextf("'sumLenSq' = %g is too large (larger than maximal integer)",
+## 		  nn), domain = NA)
+##   else
+##     nn
+## }
+
 ###*# Methods for local generics
 
-corFactor.corStruct <-
-  function(object, ...)
+corFactor.corStruct <- function(object, ...)
 {
   if (!is.null(aux <- attr(object, "factor"))) {
     return(aux)
   }
   corD <- Dim(object)
+  ## .chkLenSq(corD[["sumLenSq"]])
   val <- .C(corStruct_factList,
 	    as.double(unlist(corMatrix(object))),
 	    as.integer(unlist(corD)),
@@ -63,15 +78,13 @@ corMatrix.corStruct <-
                     dQuote(class(object)[1])), domain = NA)
   } else {
     ## transpose inverse square root
-    if (data.class(covariate) == "list") {
-      if (is.null(names(covariate))) {
-	names(covariate) <- 1:length(covariate)
-      }
-      corD <- Dim(object, rep(names(covariate),
-			      unlist(lapply(covariate, length))))
-    } else {
-      corD <- Dim(object, rep(1, length(covariate)))
-    }
+    corD <- Dim(object,
+		if (data.class(covariate) == "list") {
+		  if (is.null(names(covariate)))
+		    names(covariate) <- seq_along(covariate)
+		  rep(names(covariate), lengths(covariate))
+		} else
+		  rep(1, length(covariate)))
     val <- .C(corStruct_factList,
 	      as.double(unlist(corMatrix(object, covariate))),
 	      as.integer(unlist(corD)),
@@ -97,8 +110,7 @@ corMatrix.corStruct <-
 
 ###*# Methods for standard generics
 
-as.matrix.corStruct <-
-  function(x, ...) corMatrix(x)
+as.matrix.corStruct <- function(x, ...) corMatrix(x)
 
 coef.corStruct <-
   ## Accessor for constrained or unconstrained parameters of
@@ -120,8 +132,7 @@ coef.corStruct <-
   }
 }
 
-"coef<-.corStruct" <-
-  function(object, ..., value)
+`coef<-.corStruct` <- function(object, ..., value)
 {
   ## Assignment of the unconstrained parameter of corStruct objects
   value <- as.numeric(value)
@@ -137,19 +148,24 @@ coef.corStruct <-
   object
 }
 
-Dim.corStruct <-
-  function(object, groups, ...)
+Dim.corStruct <- function(object, groups, ...)
 {
   if (missing(groups)) return(attr(object, "Dim"))
   ugrp <- unique(groups)
   groups <- factor(groups, levels = ugrp)
   len <- table(groups)
+  suml2 <- sum(len^2)
+  if(suml2 > .Machine$integer.max)
+    stop(gettextf(
+      "'sumLenSq := sum(table(groups)^2)' = %g is too large.
+ Too large or no groups in your correlation structure?",
+		  suml2), call. = FALSE, domain = NA)
   list(N = length(groups),
        M = length(len),
        maxLen = max(len),
-       sumLenSq = sum(len^2),
+       sumLenSq = suml2,
        len = len,
-       start = match(ugrp, groups) - 1)
+       start = match(ugrp, groups) - 1L)
 }
 
 formula.corStruct <-
@@ -168,11 +184,8 @@ getCovariate.corStruct <-
       stop("need data to calculate covariate of \"corStruct\" object")
     }
     covForm <- getCovariateFormula(form)
-    if (!is.null(getGroupsFormula(form))) {
-      grps <- getGroups(object, data = data)
-    } else {
-      grps <- NULL
-    }
+    grps <- if(!is.null(getGroupsFormula(form)))
+                getGroups(object, data = data) ## else NULL
     if (length(all.vars(covForm)) > 0) { # primary covariate present
       if (is.null(grps)) {
         covar <- getCovariate(data, covForm)
@@ -188,7 +201,7 @@ getCovariate.corStruct <-
       if (is.null(grps)) {
         covar <- 1:nrow(data)
       } else {
-	covar <- lapply(split(grps, grps), function(x) 1:length(x))
+	covar <- lapply(split(grps, grps), function(x) seq_along(x))
       }
     }
     if (!is.null(grps)) {
@@ -249,9 +262,9 @@ logDet.corStruct <-
     ## checking for logDet attribute; if not present, get corr matrix
     aux <- corMatrix(object, covariate)
     if (data.class(aux) == "list") {    # by group
-      sum(log(abs(unlist(lapply(aux, function(el) svd(el)$d)))))/2
+      sum(log(abs(unlist(lapply(aux, svd.d)))))/2
     } else {
-      sum(log(abs(svd(aux)$d)))/2
+      sum(log(abs(svd.d(aux))))/2
     }
   } else {
     -aux1
@@ -259,14 +272,11 @@ logDet.corStruct <-
 }
 
 ## NB, no "nobs"
-logLik.corStruct <-
-  function(object, data, ...) -logDet(object)
+logLik.corStruct <- function(object, data, ...) -logDet(object)
 
-needUpdate.corStruct <-
-  function(object) FALSE
+needUpdate.corStruct <- function(object) FALSE
 
-print.corStruct <-
-  function(x, ...)
+print.corStruct <- function(x, ...)
 {
   if (length(aux <- coef(x, unconstrained = FALSE)) > 0) {
     cat("Correlation structure of class", class(x)[1], "representing\n")
@@ -277,14 +287,13 @@ print.corStruct <-
   invisible(x)
 }
 
-print.summary.corStruct <-
-  function(x, ...)
+print.summary.corStruct <- function(x, ...)
 {
   class(x) <- attr(x, "oClass")
-  cat(paste("Correlation Structure: ", attr(x, "structName"), "\n", sep = ""))
+  cat(paste0("Correlation Structure: ", attr(x, "structName"), "\n"))
   cat(paste(" Formula:", deparse(formula(x)),"\n"))
   cat(" Parameter estimate(s):\n")
-  print(coef(x, unconstrained = FALSE))
+  print(coef(x, unconstrained = FALSE), ...)
   invisible(x)
 }
 
@@ -335,8 +344,7 @@ corSymm <-
 
 ###*# Methods for local generics
 
-corFactor.corSymm <-
-  function(object, ...)
+corFactor.corSymm <- function(object, ...)
 {
   corD <- Dim(object)
   val <- .C(symm_factList,
@@ -355,15 +363,13 @@ corFactor.corSymm <-
 corMatrix.corSymm <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate, length))))
-  } else {
-    corD <- Dim(object, rep(1, length(covariate)))
-  }
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), lengths(covariate))
+	      } else
+		rep(1, length(covariate)))
   if (corr) {
     val <- .C(symm_matList,
 	      as.double(as.vector(object)),
@@ -400,8 +406,7 @@ corMatrix.corSymm <-
 
 ###*# Methods for standard generics
 
-coef.corSymm <-
-  function(object, unconstrained = TRUE, ...)
+coef.corSymm <- function(object, unconstrained = TRUE, ...)
 {
   if (unconstrained) {
     if (attr(object, "fixed")) {
@@ -415,8 +420,7 @@ coef.corSymm <-
      as.integer(mC), corr = double(round(mC * (mC - 1) / 2)))[["corr"]]
 }
 
-"coef<-.corSymm" <-
-  function(object, ..., value)
+`coef<-.corSymm` <- function(object, ..., value)
 {
   if (length(value) != length(object)) {
     stop("cannot change the length of the parameter of a \"corSymm\" object")
@@ -436,8 +440,7 @@ coef.corSymm <-
   object
 }
 
-Initialize.corSymm <-
-  function(object, data, ...)
+Initialize.corSymm <- function(object, data, ...)
 {
   if (!is.null(attr(object, "maxCov"))) {# initialized - nothing to do
     return(object)
@@ -473,9 +476,9 @@ Initialize.corSymm <-
     }
     natMat <- diag(maxCov)/2
     natMat[lower.tri(natMat)] <- natPar
-    natMat <- (t(natMat) + natMat)
+    natMat <- t(natMat) + natMat
     ## checking if positive-definite
-    if (any(eigen(natMat)$values <= 0)) {
+    if (any(eigen(natMat, symmetric=TRUE, only.values=TRUE)$values <= 0)) {
       stop("initial values for \"corSymm\" do not define a positive-definite correlation structure")
     }
     natMat <- chol(natMat)
@@ -495,11 +498,9 @@ Initialize.corSymm <-
   object
 }
 
-print.corSymm <-
-  function(x, ...)
+print.corSymm <- function(x, ...)
 {
-  if (length(as.vector(x)) > 0 &&
-      !is.null(mC <- attr(x, "maxCov"))) {
+  if (length(as.vector(x)) > 0 && !is.null(mC <- attr(x, "maxCov"))) {
     aux <- coef.corSymm(x, unconstrained = FALSE)
     val <- diag(mC)
     dimnames(val) <- list(1:mC, 1:mC)
@@ -512,11 +513,9 @@ print.corSymm <-
   invisible(x)
 }
 
-print.summary.corSymm <-
-  function(x, ...)
+print.summary.corSymm <- function(x, ...)
 {
-  if (length(as.vector(x)) > 0 &&
-      !is.null(mC <- attr(x, "maxCov"))) {
+  if (length(as.vector(x)) > 0 && !is.null(mC <- attr(x, "maxCov"))) {
     cat("Correlation Structure: General\n")
     cat(paste(" Formula:", deparse(formula(x)),"\n"))
     cat(" Parameter estimate(s):\n")
@@ -529,8 +528,7 @@ print.summary.corSymm <-
   invisible(x)
 }
 
-recalc.corSymm <-
-  function(object, conLin, ...)
+recalc.corSymm <- function(object, conLin, ...)
 {
   val <-
     .C(symm_recalc,
@@ -546,8 +544,7 @@ recalc.corSymm <-
   conLin
 }
 
-summary.corSymm <-
-  function(object, structName = "General correlation", ...)
+summary.corSymm <- function(object, structName = "General correlation", ...)
 {
   attr(object, "structName") <- structName
   class(object) <- "summary.corSymm"
@@ -558,9 +555,7 @@ summary.corSymm <-
 
 ####* Constructor
 
-corNatural <-
-  ## Constructor for the corSymm class
-  function(value = numeric(0), form = ~ 1, fixed = FALSE)
+corNatural <- function(value = numeric(0), form = ~ 1, fixed = FALSE)
 {
   attr(value, "formula") <- form
   attr(value, "fixed") <- fixed
@@ -570,8 +565,7 @@ corNatural <-
 
 ###*# Methods for local generics
 
-corFactor.corNatural <-
-  function(object, ...)
+corFactor.corNatural <- function(object, ...)
 {
   corD <- Dim(object)
   val <- .C(nat_factList,
@@ -590,15 +584,13 @@ corFactor.corNatural <-
 corMatrix.corNatural <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate, length))))
-  } else {
-    corD <- Dim(object, rep(1, length(covariate)))
-  }
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), lengths(covariate))
+	      } else
+		rep(1, length(covariate)))
   if (corr) {
     val <- .C(nat_matList,
 	      as.double(as.vector(object)),
@@ -621,8 +613,8 @@ corMatrix.corNatural <-
   if (corD[["M"]] > 1) {
     val <- split(val, rep(1:corD[["M"]], (corD[["len"]])^2))
     val <- lapply(val, function(el) {
-      nel <- round(sqrt(length(el)))
-      array(el, c(nel, nel))
+        nel <- round(sqrt(length(el)))
+        array(el, c(nel, nel))
     })
     names(val) <- names(corD[["len"]])
     val <- as.list(val)
@@ -648,15 +640,12 @@ coef.corNatural <-
   mC <- attr(object, "maxCov")
   val <- .C(nat_fullCorr, as.double(object),
             as.integer(mC), corr = double(round(mC * (mC - 1) / 2)))[["corr"]]
-  names(val) <- outer(1:mC, 1:mC,
-                      function(x,y) {
-                        paste("cor(",y,",",x,")",sep="")
-                      })[lower.tri(diag(mC))]
+  names(val) <- outer(1:mC, 1:mC, function(x,y) paste0("cor(",y,",",x,")"))[
+      lower.tri(diag(mC))]
   val
 }
 
-"coef<-.corNatural" <-
-  function(object, ..., value)
+`coef<-.corNatural` <- function(object, ..., value)
 {
   if (length(value) != length(object)) {
     stop("cannot change the length of the parameter of a \"corNatural\" object")
@@ -676,8 +665,7 @@ coef.corNatural <-
   object
 }
 
-Initialize.corNatural <-
-  function(object, data, ...)
+Initialize.corNatural <- function(object, data, ...)
 {
   if (!is.null(attr(object, "maxCov"))) {# initialized - nothing to do
     return(object)
@@ -713,9 +701,9 @@ Initialize.corNatural <-
     }
     natMat <- diag(maxCov)/2
     natMat[lower.tri(natMat)] <- natPar
-    natMat <- (t(natMat) + natMat)
+    natMat <- t(natMat) + natMat
     ## checking if positive-definite
-    if (any(eigen(natMat)$values <= 0)) {
+    if (any(eigen(natMat, symmetric=TRUE, only.values=TRUE)$values <= 0)) {
       stop("initial values for \"corNatural\" do not define a positive-definite correlation structure")
     }
     coef(object) <- log((natPar + 1)/(1 - natPar))
@@ -729,8 +717,7 @@ Initialize.corNatural <-
   object
 }
 
-print.corNatural <-
-  function(x, ...)
+print.corNatural <- function(x, ...)
 {
   if (length(as.vector(x)) > 0 &&
       !is.null(mC <- attr(x, "maxCov"))) {
@@ -823,8 +810,7 @@ corMatrix.corIdent <-
 coef.corIdent <-
   function(object, unconstrained = TRUE, ...) numeric(0)
 
-"coef<-.corIdent" <-
-  function(object, ..., value) object
+`coef<-.corIdent` <- function(object, ..., value) object
 
 Initialize.corIdent <-
   function(object, data, ...)
@@ -833,14 +819,9 @@ Initialize.corIdent <-
   object
 }
 
-logDet.corIdent <-
-  function(object, covariate, ...) 0
+logDet.corIdent <- function(object, covariate, ...) 0
 
-recalc.corIdent <-
-  function(object, conLin, ...)
-{
-  conLin
-}
+recalc.corIdent <- function(object, conLin, ...) conLin
 
 summary.corIdent <-
   function(object, structName = "Independent", ...)
@@ -868,14 +849,16 @@ corAR1 <-
 
 ###*# Methods for local generics
 
-corFactor.corAR1 <-
-  function(object, ...)
+corFactor.corAR1 <- function(object, ...)
 {
   corD <- Dim(object)
+  if(corD[["sumLenSq"]] > .Machine$integer.max)
+    stop(gettextf("'sumLenSq' = %g is too large (larger than maximal integer)",
+		  corD[["sumLenSq"]]), domain = NA)
   val <- .C(AR1_factList,
 	    as.double(as.vector(object)),
 	    as.integer(unlist(corD)),
-	    factor = double(corD[["sumLenSq"]]),
+	    factor = double(corD[["sumLenSq"]]), ## of size n^2 -- too large !!
 	    logDet = double(1))[c("factor", "logDet")]
   lD <- val[["logDet"]]
   val <- val[["factor"]]
@@ -886,15 +869,13 @@ corFactor.corAR1 <-
 corMatrix.corAR1 <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate, length))))
-  } else {
-    corD <- Dim(object, rep(1, length(covariate)))
-  }
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), lengths(covariate))
+	      } else
+		rep(1, length(covariate)))
   if (corr) {
     val <- .C(AR1_matList,
 	      as.double(as.vector(object)),
@@ -943,8 +924,7 @@ coef.corAR1 <-
   aux
 }
 
-"coef<-.corAR1" <-
-  function(object, ..., value)
+`coef<-.corAR1` <- function(object, ..., value)
 {
   if (length(value) != length(object)) {
     stop("cannot change the length of the parameter of a \"corAR1\" object")
@@ -1050,15 +1030,13 @@ corFactor.corCAR1 <-
 corMatrix.corCAR1 <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate, length))))
-  } else {
-    corD <- Dim(object, rep(1, length(covariate)))
-  }
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), lengths(covariate))
+	      } else
+		rep(1, length(covariate)))
   if (corr) {
     val <- .C(CAR1_matList,
 	      as.double(as.vector(object)),
@@ -1109,8 +1087,7 @@ coef.corCAR1 <-
   aux
 }
 
-"coef<-.corCAR1" <-
-  function(object, ..., value)
+`coef<-.corCAR1` <- function(object, ..., value)
 {
   if (length(value) != length(object)) {
     stop("cannot change the length of the parameter of a \"corCAR1\" object")
@@ -1230,15 +1207,13 @@ corFactor.corARMA <-
 corMatrix.corARMA <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate, length))))
-  } else {
-    corD <- Dim(object, rep(1, length(covariate)))
-  }
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), lengths(covariate))
+	      } else
+		rep(1, length(covariate)))
   p <- attr(object, "p")
   q <- attr(object, "q")
   maxLag <- attr(object, "maxLag")
@@ -1308,8 +1283,7 @@ coef.corARMA <-
   val
 }
 
-"coef<-.corARMA" <-
-  function(object, ..., value)
+`coef<-.corARMA` <- function(object, ..., value)
 {
   maxLag <- attr(object, "maxLag")
   if(is.null(maxLag)) stop("'object' has not been Initialize()d")
@@ -1427,15 +1401,13 @@ corFactor.compSymm <-
 corMatrix.corCompSymm <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate, length))))
-  } else {
-    corD <- Dim(object, rep(1, length(covariate)))
-  }
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), lengths(covariate))
+	      } else
+		rep(1, length(covariate)))
   if (corr) {
     val <- .C(compSymm_matList,
 	      as.double(as.vector(object)),
@@ -1486,8 +1458,7 @@ coef.corCompSymm <-
   val
 }
 
-"coef<-.corCompSymm" <-
-  function(object, ..., value)
+`coef<-.corCompSymm` <- function(object, ..., value)
 {
   if (length(value) != length(object)) {
     stop("cannot change the length of the parameter of a \"corCompSymm\" object")
@@ -1581,15 +1552,13 @@ summary.corCompSymm <-
 #corMatrix.corHF <-
 #  function(object, covariate = getCovariate(object), corr = TRUE)
 #{
-#  if (data.class(covariate) == "list") {
-#    if (is.null(names(covariate))) {
-#      names(covariate) <- 1:length(covariate)
-#    }
-#    corD <- Dim(object, rep(names(covariate),
-#			    unlist(lapply(covariate, length))))
-#  } else {
-#    corD <- Dim(object, rep(1, length(covariate)))
-#  }
+#   corD <- Dim(object,
+# 	      if (data.class(covariate) == "list") {
+# 		if (is.null(names(covariate)))
+# 		  names(covariate) <- seq_along(covariate)
+# 		rep(names(covariate), lengths(covariate))
+# 	      } else
+# 		rep(1, length(covariate)))
 #  if (corr) {
 #    val <- .C("HF_matList",
 #	      as.double(as.vector(object)),
@@ -1744,25 +1713,22 @@ corSpatial <-
              "rational"),
 	   metric = c("euclidean", "maximum", "manhattan"), fixed = FALSE)
 {
-  type <- match.arg(type)
-  spClass <- switch(type,
-		    spherical = "corSpher",
-		    exponential = "corExp",
-		    gaussian = "corGaus",
-		    linear = "corLin",
-                    rational = "corRatio")
-  attr(value, "formula") <- form
-  attr(value, "nugget") <- nugget
-  attr(value, "metric") <- match.arg(metric)
-  attr(value, "fixed") <- fixed
-  class(value) <- c(spClass, "corSpatial", "corStruct")
-  value
+  spClass <- c(spherical = "corSpher",
+	       exponential = "corExp",
+	       gaussian = "corGaus",
+	       linear = "corLin",
+	       rational = "corRatio")[match.arg(type)]
+  structure(value,
+	    "formula" = form,
+	    "nugget" = nugget,
+	    "metric" = match.arg(metric),
+	    "fixed" = fixed,
+	    class = c(spClass, "corSpatial", "corStruct"))
 }
 
 ###*# Methods for local generics
 
-corFactor.corSpatial <-
-  function(object, ...)
+corFactor.corSpatial <- function(object, ...)
 {
   corD <- Dim(object)
   val <- .C(spatial_factList,
@@ -1773,25 +1739,21 @@ corFactor.corSpatial <-
 	    as.double(attr(object, "minD")),
 	    factor = double(corD[["sumLenSq"]]),
 	    logDet = double(1))[c("factor", "logDet")]
-  lD <- val[["logDet"]]
-  val <- val[["factor"]]
-  attr(val, "logDet") <- lD
-  val
+  structure(val[["factor"]], logDet = val[["logDet"]])
 }
 
 corMatrix.corSpatial <-
   function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-  if (data.class(covariate) == "list") {
-    if (is.null(names(covariate))) {
-      names(covariate) <- 1:length(covariate)
-    }
-    corD <- Dim(object, rep(names(covariate),
-			    unlist(lapply(covariate,
-		  function(el) round((1 + sqrt(1 + 8 * length(el)))/2)))))
-  } else {
-    corD <- Dim(object, rep(1, round((1 + sqrt(1 + 8* length(covariate)))/2)))
-  }
+  nRt <- function(vec) round((1 + sqrt(1 + 8 * length(vec))) / 2)
+  corD <- Dim(object,
+	      if (data.class(covariate) == "list") {
+		if (is.null(names(covariate)))
+		  names(covariate) <- seq_along(covariate)
+		rep(names(covariate), vapply(covariate, nRt, numeric(1)))
+	      }
+	      else
+		rep(1, nRt(covariate)))
   if (corr) {
     val <- .C(spatial_matList,
 	      as.double(as.vector(object)),
@@ -1844,13 +1806,11 @@ coef.corSpatial <-
     val <- exp(val)
     if (attr(object, "nugget")) val[2] <- val[2]/(1+val[2])
   }
-  if (attr(object, "nugget")) names(val) <- c("range", "nugget")
-  else names(val) <- "range"
+  names(val) <- if(attr(object, "nugget")) c("range", "nugget") else "range"
   val
 }
 
-"coef<-.corSpatial" <-
-  function(object, ..., value)
+`coef<-.corSpatial` <- function(object, ..., value)
 {
   if (length(value) != length(object)) {
     stop("cannot change the length of the parameter after initialization")
@@ -1871,8 +1831,7 @@ coef.corSpatial <-
   object
 }
 
-Dim.corSpatial <-
-  function(object, groups, ...)
+Dim.corSpatial <- function(object, groups, ...)
 {
   if (missing(groups)) return(attr(object, "Dim"))
   val <- Dim.corStruct(object, groups)
@@ -1894,44 +1853,38 @@ getCovariate.corSpatial <-
       stop("need data to calculate covariate")
     }
     covForm <- getCovariateFormula(form)
-    if (length(all.vars(covForm)) > 0) { # covariate present
-      if (attr(terms(covForm), "intercept") == 1) {
-	covForm <-
-          eval(parse(text = paste("~", deparse(covForm[[2]]),"-1",sep="")))
-      }
-      covar <-
-          as.data.frame(unclass(model.matrix(covForm,
-                                             model.frame(covForm, data,
-                                                         drop.unused.levels = TRUE))))
-    } else {
-      covar <- NULL
-    }
+    covar <-
+        if (length(all.vars(covForm)) > 0) { # covariate present
+            if (attr(terms(covForm), "intercept") == 1) {
+                covForm <- eval(substitute(~ CV - 1, list(CV = covForm[[2]])))
+            }
+            as.data.frame(unclass(
+              model.matrix(covForm,
+                           model.frame(covForm, data,
+                                       drop.unused.levels = TRUE))))
+        } ## else NULL
 
-    if (!is.null(getGroupsFormula(form))) { # by groups
-      grps <- getGroups(object, data = data)
-      if (is.null(covar)) {
-	covar <- lapply(split(grps, grps),
-                        function(x) as.vector(dist(1:length(x))))
-      } else {
-	covar <- lapply(split(covar, grps),
-			function(el, metric) {
-                          el <- as.matrix(el)
-                          if (nrow(el) > 1) {
-                            as.vector(dist(el, metric))
-                          } else {
-                            numeric(0)
-                          }
-			}, metric = attr(object, "metric"))
+    covar <-
+      if (!is.null(getGroupsFormula(form))) { # by groups
+        grps <- getGroups(object, data = data)
+        grps <-
+          if (is.null(covar)) {
+            lapply(split(grps, grps), function(x) as.vector(dist(seq_along(x))))
+          } else {
+            lapply(split(covar, grps),
+                   function(el, metric) {
+                     el <- as.matrix(el)
+                     if (nrow(el) > 1) as.vector(dist(el, metric)) else numeric(0)
+                   }, metric = attr(object, "metric"))
+          }
+        grps[lengths(grps) > 0]# no 1-obs groups
+      } else { # no groups
+        as.vector(
+          if (is.null(covar))
+            dist(1:nrow(data))
+          else
+            dist(as.matrix(covar), method = attr(object, "metric")))
       }
-      covar <- covar[sapply(covar, length) > 0]  # no 1-obs groups
-    } else {				# no groups
-      if (is.null(covar)) {
-	covar <- as.vector(dist(1:nrow(data)))
-      } else {
-	covar <- as.vector(dist(as.matrix(covar),
-                                method = attr(object, "metric")))
-      }
-    }
     if (any(unlist(covar) == 0)) {
       stop("cannot have zero distances in \"corSpatial\"")
     }
@@ -2330,6 +2283,5 @@ Variogram.corSpher <-
 ### Local variables:
 ### mode: outline-minor
 ### outline-regexp: "\^L\\|\\`#\\|##\\*\\|###\\*\\|[a-zA-Z]\\|\\\"[a-zA-Z]\\|####\\*"
+### ess-indent-offset: 2
 ### End:
-
-
