@@ -1,9 +1,9 @@
 ###  Fit a general nonlinear regression model with correlated and/or
 ###  heteroscedastic errors
 ###
+### Copyright 2007-2017  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
-# Copyright 2007-2014 The R Core team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -142,9 +142,7 @@ gnls <-
 ##  }
 
   ## check if correlation is present and has groups
-  if (!is.null(correlation)) {
-    groups <- getGroupsFormula(correlation)
-  } else groups <- NULL
+  groups <- if (!is.null(correlation)) getGroupsFormula(correlation) # else NULL
 #  if (!is.null(correlation)) {
 #    groups <- getGroupsFormula(correlation, asList = TRUE)
 #    if (!is.null(groups)) {
@@ -174,7 +172,7 @@ gnls <-
   ## extract a data frame with enough information to evaluate
   ## form, params, random, groups, correlation, and weights
   mfArgs <- list(formula = asOneFormula(formula(gnlsSt), form, params,
-                   groups, omit = c(pnames, "pi")),
+                                        groups, omit = c(pnames, "pi")),
 		 data = data, na.action = na.action)
   if (!missing(subset)) {
     mfArgs[["subset"]] <- asOneSidedFormula(Call[["subset"]])[[2]]
@@ -204,8 +202,8 @@ gnls <-
   ##
   ## evaluating the naPattern expression, if any
   ##
-  if (missing(naPattern)) naPat <- rep(TRUE, N)
-  else naPat <- as.logical(eval(asOneSidedFormula(naPattern)[[2]], dataMod))
+  naPat <- if (missing(naPattern)) rep(TRUE, N)
+	   else as.logical(eval(asOneSidedFormula(naPattern)[[2]], dataMod))
   origOrderShrunk <- origOrder[naPat]
 
   dataModShrunk <- dataMod[naPat, , drop=FALSE]
@@ -225,14 +223,13 @@ gnls <-
   plist <- vector("list", length(pnames))
   names(plist) <- pnames
   for (nm in pnames) {
-    plist[[nm]] <- TRUE
     if (deparse(params[[nm]][[3]]) != "1") {
-      plist[[nm]] <-
-        model.matrix(asOneSidedFormula(params[[nm]][[3]]),
-                 model.frame(asOneSidedFormula(params[[nm]][[3]]), dataModShrunk))
+      form1s <- asOneSidedFormula(params[[nm]][[3]])
+      plist[[nm]] <- model.matrix(form1s, model.frame(form1s, dataModShrunk))
       auxContr <- attr(plist[[nm]], "contrasts")
       contr <- c(contr, auxContr[is.na(match(names(auxContr), names(contr)))])
-    }
+    } else
+      plist[[nm]] <- TRUE
   }
   ##
   ## Params effects names
@@ -287,25 +284,21 @@ gnls <-
   }
 
   ##
-  ## defining the nlFrame
+  ## defining the nlFrame, i.e., nlEnv, an environment in R :
   ##
-  nlEnv <- new.env()
-  nlList <-
-                            list(model = gnlsModel,
-			    data = dataMod,
-			    plist = plist,
-			    beta = as.vector(spar),
-			    X = array(0, c(NReal, pLen), list(NULL, pn)),
-			    pmap = pmap,
-                            N = NReal,
-                            naPat = naPat,
-			    .parameters = c("beta"),
-                            finiteDiffGrad = finiteDiffGrad)
+  nlEnv <- list2env(
+      list(model = gnlsModel,
+           data = dataMod,
+           plist = plist,
+           beta = as.vector(spar),
+           X = array(0, c(NReal, pLen), list(NULL, pn)),
+           pmap = pmap,
+           N = NReal,
+           naPat = naPat,
+           .parameters = c("beta"),
+           finiteDiffGrad = finiteDiffGrad))
 
-  lapply(names(nlList), function(x, y, env) assign(x, y[[x]], envir = env),
-         nlList, env = nlEnv)
-
-  modelExpression <- ~{
+  modelExpression <- ~ {
     pars <- getParsGnls(plist, pmap, beta, N)
     res <- eval(model, data.frame(data, pars))
     if (!length(grad <- attr(res, "gradient"))) {
@@ -316,11 +309,8 @@ gnls <-
     res <- res[naPat]
     for (nm in names(plist)) {
       gradnm <- grad[, nm]
-      if (is.logical(p <- plist[[nm]])) {
-        X[, pmap[[nm]]] <- gradnm
-      } else {
-        X[, pmap[[nm]]] <- gradnm * p
-      }
+      X[, pmap[[nm]]] <-
+        if(is.logical(p <- plist[[nm]])) gradnm else gradnm * p
     }
     result <- c(X, res)
     result[is.na(result)] <- 0
@@ -331,11 +321,15 @@ gnls <-
       getParsGnls(plist, pmap, beta, N)))[naPat]
   w <- eval(modelResid[[2]], envir = nlEnv)
   ## creating the condensed linear model
+  ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+  fixedSigma <- controlvals$sigma > 0
   Dims <- list(p = pLen, N = NReal, REML = FALSE)
   attr(gnlsSt, "conLin") <-
     list(Xy = array(w, c(NReal, 1),
-           list(row.names(dataModShrunk), deparse(form[[2]]))), dims = Dims,
-         logLik = 0)
+		    list(row.names(dataModShrunk), deparse(form[[2]]))),
+	 dims = Dims, logLik = 0,
+	 ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+	 sigma=controlvals$sigma, auxSigma=0, fixedSigma=fixedSigma)
 
   ## additional attributes of gnlsSt
   attr(gnlsSt, "resp") <- yShrunk
@@ -348,7 +342,7 @@ gnls <-
 
   numIter <- 0				# number of iterations
   nlsSettings <- c(controlvals$nlsMaxIter, controlvals$minScale,
-                    controlvals$nlsTol, 0, 0, 0)
+                   controlvals$nlsTol, 0, 0, 0)
   nlModel <- nonlinModel(modelExpression, nlEnv)
   repeat {
   ## alternating algorithm
@@ -409,27 +403,26 @@ gnls <-
 	       as.integer(!is.null(correlation)),
 	       as.integer(!is.null(weights)),
                nlModel,
-	       NAOK = TRUE)
+	       NAOK = TRUE)[c("thetaNLS", "settings", "additional")]
     if (work$settings[4] == 1) {
 ##      convResult <- 2
+      msg <- "step halving factor reduced below minimum in NLS step"
       if (controlvals$returnObject) {
-        warning("step halving factor reduced below minimum in NLS step")
-      } else {
-        stop("step halving factor reduced below minimum in NLS step")
-      }
-      break
+        warning(msg)
+        break
+      } else stop(msg)
     }
     oldPars <- c(spar, oldPars)
     spar[] <- work$thetaNLS
-    if (length(coef(gnlsSt)) == 0 && work$set[5] < controlvals$nlsMaxIter) {
+    if (length(coef(gnlsSt)) == 0 && work$settings[5] < controlvals$nlsMaxIter) {
       break
     }
     attr(gnlsSt, "conLin")$Xy[] <- work$additional
     attr(gnlsSt, "conLin")$logLik <- 0
     if (verbose) {
-      cat("\nNLS step: RSS = ", format(work$set[6]), "\n model parameters:")
+      cat("\nNLS step: RSS = ", format(work$settings[6]), "\n model parameters:")
       for (i in 1:pLen) cat(format(signif(spar[i]))," ")
-      cat("\n iterations:",work$set[5],"\n")
+      cat("\n iterations:",work$settings[5],"\n")
     }
     aConv <- c(spar, aConv)
 
@@ -454,38 +447,45 @@ gnls <-
 
     if ((max(aConv) <= controlvals$tolerance) ||
         (aConv["params"] <= controlvals$tolerance && convIter == 1)) {
-##      convResult <- 0
+      ##      convResult <- 0
       break
     }
     if (numIter >= controlvals$maxIter) {
-##      convResult <- 1
+      ##      convResult <- 1
+      msg <- "maximum number of iterations reached without convergence"
       if (controlvals$returnObject) {
-	warning("maximum number of iterations reached without convergence")
+	warning(msg)
 	break
-      } else {
-	stop("maximum number of iterations reached without convergence")
-      }
+      } else stop(msg)
     }
-  }
+  } ## end{ repeat } --------------
 
   ## wraping up
   ww <- eval(modelExpression[[2]], envir = nlEnv)
   auxRes <- ww[NReal * pLen + (1:NReal)]
   attr(gnlsSt, "conLin")$Xy <- array(ww, c(NReal, pLen + 1))
-  attr(gnlsSt, "conLin") <- recalc(gnlsSt)
-  sigma <-
-    sqrt(sum((attr(gnlsSt,"conLin")$Xy[,pLen+1])^2)/(NReal - pLen))
-  varBeta <- qr(attr(gnlsSt, "conLin")$Xy[ , 1:pLen, drop = FALSE])
+  attr(gnlsSt, "conLin") <- c.L <- recalc(gnlsSt)
+  ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+  if((sigma <- controlvals$sigma) == 0) {
+    sigma <- sqrt(sum((c.L$Xy[, pLen + 1])^2)/(NReal - pLen))
+    lsig <- log(sigma) + 0.5 * log(1 - pLen/NReal)
+    loglik <- ( - NReal * (1 + log(2 * pi) + 2 * lsig))/2 + c.L$logLik
+  } else {
+    loglik <- - (NReal * (log(2 * pi)/2 + log(sigma)) +
+                 sum((c.L$Xy[, pLen + 1])^2) / (2 * sigma^2)) + c.L$logLik
+    lsig <- log(sigma)
+  }
+  ## ####
+  varBeta <- qr(c.L$Xy[ , 1:pLen, drop = FALSE])
   if (varBeta$rank < pLen) {
-    stop("approximate covariance matrix for parameter estimates not of full rank")
+    ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+    print("approximate covariance matrix for parameter estimates not of full rank")
+	return()
   }
-  lsig <- log(sigma) + 0.5 * log(1 - pLen/NReal)
   attr(parAssign, "varBetaFact") <- varBeta <-
     sigma * t(backsolve(qr.R(varBeta), diag(pLen)))
   varBeta <- crossprod(varBeta)
   dimnames(varBeta) <- list(pn, pn)
-  loglik <- - NReal * (1 + log(2 * pi) + 2 * lsig) / 2 +
-    attr(gnlsSt, "conLin")$logLik
   ##
   ## fitted.values and residuals (in original order)
   ##
@@ -502,27 +502,32 @@ gnls <-
   ## getting the approximate var-cov of the parameters
   ## first making Xy into single column array again
   attr(gnlsSt, "conLin")$Xy <- array(auxRes, c(NReal, 1))
-  if (controlvals$apVar) {
-    apVar <- gnlsApVar(gnlsSt, lsig, .relStep = controlvals[[".relStep"]],
-                       minAbsPar = controlvals[["minAbsParApVar"]])
-  } else {
-    apVar <- "Approximate variance-covariance matrix not available"
-  }
+  ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+  attr(gnlsSt, "fixedSigma") <- (controlvals$sigma > 0)
+  apVar <-
+    if (controlvals$apVar)
+      gnlsApVar(gnlsSt, lsig, .relStep = controlvals[[".relStep"]],
+		minAbsPar = controlvals[["minAbsParApVar"]])
+    else "Approximate variance-covariance matrix not available"
   ## getting rid of condensed linear model and fit
   oClass <- class(gnlsSt)
   attributes(gnlsSt) <-
     attributes(gnlsSt)[!is.na(match(names(attributes(gnlsSt)),
-                                    c("names","pmap")))]
+    ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+                                    c("names","pmap","fixedSigma")))]
   class(gnlsSt) <- oClass
+  grpDta <- inherits(data, "groupedData")
   ##
   ## creating the  gnls object
   ##
-  estOut <- list(modelStruct = gnlsSt,
+  structure(class = c("gnls", "gls"),
+            list(modelStruct = gnlsSt,
 		 dims = Dims,
                  contrasts = contr,
 		 coefficients = spar,
 		 varBeta = varBeta,
-		 sigma = sigma,
+         ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+		 sigma = if(controlvals$sigma) controlvals$sigma else sigma,
 		 apVar = apVar,
 		 logLik = loglik,
 		 numIter = numIter,
@@ -534,14 +539,10 @@ gnls <-
 		 plist = plist,
                  pmap = pmap,
                  parAssign = parAssign,
-                 na.action = attr(dataMod, "na.action"))
-  if (inherits(data, "groupedData")) {
-    ## saving labels and units for plots
-    attr(estOut, "units") <- attr(data, "units")
-    attr(estOut, "labels") <- attr(data, "labels")
-  }
-  class(estOut) <- c("gnls", "gls")
-  estOut
+                 na.action = attr(dataMod, "na.action")),
+	    ## saving labels and units for plots
+	    units = if(grpDta) attr(data, "units"),
+	    labels= if(grpDta) attr(data, "labels"))
 }
 
 ### Auxiliary functions used internally in gls and its methods
@@ -556,13 +557,22 @@ gnlsApVar <-
   fullGnlsLogLik <-
     function(Pars, object, conLin, N) {
       ## logLik as a function of sigma and coef(glsSt)
+      ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+      fixedSigma <- attr(object,"fixedSigma")
       npar <- length(Pars)
-      lsigma <- Pars[npar]              # within-group std. dev.
-      Pars <- Pars[-npar]
+      if (!fixedSigma) {
+         lsigma <- Pars[npar]
+         Pars <- Pars[-npar]
+      } else {
+         lsigma <- log(conLin$sigma)
+      }
+      #######
       coef(object) <- Pars
       conLin <- recalc(object, conLin)
       conLin[["logLik"]] - N * lsigma - sum(conLin$Xy^2)/(2*exp(2*lsigma))
     }
+
+  fixedSigma <- attr(gnlsSt,"fixedSigma")
   if (length(gnlsCoef <- coef(gnlsSt)) > 0) {
     cSt <- gnlsSt[["corStruct"]]
     if (!is.null(cSt) && inherits(cSt, "corSymm") && natural) {
@@ -575,19 +585,28 @@ gnlsApVar <-
     dims <- conLin$dims
     N <- dims$N
     conLin[["logLik"]] <- 0               # making sure
-    Pars <- c(gnlsCoef, lSigma = lsigma)
+    ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+    Pars <- if(fixedSigma) gnlsCoef else c(gnlsCoef, lSigma = lsigma)
+                                        #   log(sigma) is used as input in contrast to gls
     val <- fdHess(Pars, fullGnlsLogLik, gnlsSt, conLin, N,
 		  .relStep = .relStep, minAbsPar = minAbsPar)[["Hessian"]]
-    if (all(eigen(val)$values < 0)) {
+    if (all(eigen(val, only.values=TRUE)$values < 0)) {
       ## negative definite - OK
       val <- solve(-val)
+      ## ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+      ## if(fixedSigma && !is.null(dim(val))){
+      ##     Pars <- c(gnlsCoef, lSigma = lsigma)
+      ##     npars<-length(Pars)
+      ##     val<-cbind(val,rep(0,npars-1))
+      ##     val<-rbind(val,rep(0,npars))
+      ## }
       nP <- names(Pars)
       dimnames(val) <- list(nP, nP)
       attr(val, "Pars") <- Pars
       attr(val, "natural") <- natural
       val
     } else {
-      ## problem - solution is not a maximum
+    	## problem - solution is not maximum
       "Non-positive definite approximate variance-covariance"
     }
   } else {
@@ -600,16 +619,15 @@ gnlsApVar <-
 ### the params and random effects
 ###
 
-getParsGnls <-
-  function(plist, pmap, beta, N)
+getParsGnls <- function(plist, pmap, beta, N)
 {
   pars <- array(0, c(N, length(plist)), list(NULL, names(plist)))
   for (nm in names(plist)) {
-    if (is.logical(p <- plist[[nm]])) {
-      pars[, nm] <- beta[pmap[[nm]]]
-    } else {
-      pars[, nm] <- p %*% beta[pmap[[nm]]]
-    }
+    pars[, nm] <-
+      if (is.logical(p <- plist[[nm]]))
+        beta[pmap[[nm]]]
+      else
+        p %*% beta[pmap[[nm]]]
   }
   pars
 }
@@ -641,25 +659,31 @@ getData.gnls <-
     subset <- eval(asOneSidedFormula(subset)[[2]], data)
     data <- data[subset, ]
   }
-  return(data)
+  data
 }
 
 
 logLik.gnls <-
+    ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
   function(object, REML = FALSE, ...)
 {
   if (REML) {
     stop("cannot calculate REML log-likelihood for \"gnls\" objects")
   }
+  ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+  fixSig <- attr(object[["modelStruct"]], "fixedSigma")
+  fixSig <- !is.null(fixSig) && fixSig
+
   p <- object$dims$p
   N <- object$dims$N
   val <- object[["logLik"]]
   attr(val, "nobs") <- attr(val, "nall") <- N
-  attr(val, "df") <- p + length(coef(object[["modelStruct"]])) + 1
+
+  ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+  attr(val, "df") <- p + length(coef(object[["modelStruct"]])) + as.integer(!fixSig)
   class(val) <- "logLik"
   val
 }
-
 nobs.gnls <- function(object, ...) object$dims$N
 
 predict.gnls <-
@@ -674,12 +698,13 @@ predict.gnls <-
   newdata <- data.frame(newdata, check.names = FALSE)
   mCall <- object$call
 
-  mfArgs <- list(formula = asOneFormula(formula(object),
-                   mCall$params, naPattern,
-                   omit = c(names(object$plist), "pi",
-                     deparse(getResponseFormula(object)[[2]]))),
-                 data = newdata, na.action = na.action)
-  mfArgs$drop.unused.levels <- TRUE
+  mfArgs <- list(formula =
+                   asOneFormula(formula(object),
+                                mCall$params, naPattern,
+                                omit = c(names(object$plist), "pi",
+                                         deparse(getResponseFormula(object)[[2]]))),
+                 data = newdata, na.action = na.action,
+                 drop.unused.levels = TRUE)
   dataMod <- do.call("model.frame", mfArgs)
 
   ## making sure factor levels are the same as in contrasts
@@ -719,12 +744,10 @@ predict.gnls <-
   plist <- object$plist
   pnames <- names(plist)
   if (is.null(params <- eval(object$call$params))) {
-    params <- eval(parse(text = paste(paste(pnames, collapse = "+"), "1",
-                           sep = "~")))
-  }
-  if (!is.list(params)) {
-    params <- list(params)
+    params <- eval(parse(text = paste0(paste(pnames, collapse = "+"), "~ 1")))
   }
+  if (!is.list(params)) params <- list(params)
+### FIXME: 2 x {very ugly}: 1) adding to val incrementally //  2) eval(parse(text = .))
   val <- NULL
   for(i in seq_along(params)) {
     if (is.name(params[[i]][[2]])) {
@@ -742,13 +765,13 @@ predict.gnls <-
 ##  pn <- names(prs)
   for(nm in pnames) {
     if (!is.logical(plist[[nm]])) {
-      plist[[nm]] <- model.matrix(asOneSidedFormula(params[[nm]][[3]]),
-                model.frame(asOneSidedFormula(params[[nm]][[3]]), dataMod))
+      form1s <- asOneSidedFormula(params[[nm]][[3]])
+      plist[[nm]] <- model.matrix(form1s, model.frame(form1s, dataMod))
     }
   }
   modForm <- getCovariateFormula(object)[[2]]
   val <- eval(modForm, data.frame(dataMod,
-              getParsGnls(plist, object$pmap, prs, N)))[naPat]
+		getParsGnls(plist, object$pmap, prs, N)))[naPat]
   names(val) <- row.names(newdata)
   lab <- "Predicted values"
   if (!is.null(aux <- attr(object, "units")$y)) {
@@ -772,10 +795,8 @@ update.gnls <-
 	existing <- !is.na(match(names(extras), names(call)))
 	## do these individually to allow NULL to remove entries.
 	for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
-	if(any(!existing)) {
-	    call <- c(as.list(call), extras[!existing])
-	    call <- as.call(call)
-	}
+	if(any(!existing))
+	    call <- as.call(c(as.list(call), extras[!existing]))
     }
     if(evaluate) eval(call, parent.frame())
     else call
@@ -828,30 +849,24 @@ gnlsStruct <-
 
 ##*## gnlsStruct methods for standard generics
 
-fitted.gnlsStruct <-
-  function(object, ...)
-{
-  attr(object, "resp") - resid(object)
-}
+fitted.gnlsStruct <- function(object, ...) attr(object, "resp") - resid(object)
 
-Initialize.gnlsStruct <-
-  function(object, data, ...)
+Initialize.gnlsStruct <- function(object, data, ...)
 {
   if (length(object)) {
     object[] <- lapply(object, Initialize, data)
     theta <- lapply(object, coef)
-    len <- unlist(lapply(theta, length))
+    len <- lengths(theta)
     num <- seq_along(len)
-    if (sum(len) > 0) {
-      pmap <- outer(rep(num, len), num, "==")
-    } else {
-      pmap <- array(FALSE, c(1, length(len)))
-    }
+    pmap <-
+        if (sum(len) > 0)
+            outer(rep(num, len), num, "==")
+        else
+            array(FALSE, c(1, length(len)))
     dimnames(pmap) <- list(NULL, names(object))
     attr(object, "pmap") <- pmap
-    if (needUpdate(object)) {
+    if (needUpdate(object))
       object <- update(object, data)
-    }
   }
   object
 }
@@ -859,15 +874,21 @@ Initialize.gnlsStruct <-
 logLik.gnlsStruct <-
   function(object, Pars, conLin = attr(object, "conLin"), ...)
 {
-  coef(object) <- Pars			# updating parameter values
-  conLin <- recalc(object, conLin)	# updating conLin
-  conLin[["logLik"]] - conLin$dims$N * log(sum(conLin$Xy^2)) / 2
+	coef(object) <- Pars
+	# updating parameter values
+	conLin <- recalc(object, conLin)
+	## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+	if(conLin$sigma == 0) {
+            conLin[["logLik"]] - (conLin$dims$N * log(sum(conLin$Xy^2)))/2
+	} else {
+            conLin[["logLik"]] - conLin$dims$N * log(conLin$sigma) -
+                sum(conLin$Xy^2) / (2 * conLin$sigma^2)
+	}
 }
 
 
-residuals.gnlsStruct <-
-  function(object, ...)
-{
+
+residuals.gnlsStruct <- function(object, ...) {
   c(eval(attr(object, "model")[[2]], envir = attr(object, "local")))
 }
 
@@ -877,14 +898,25 @@ gnlsControl <-
 	   minScale = 0.001, tolerance = 1e-6, nlsTol = 0.001,
            msTol = 1e-7,
            returnObject = FALSE, msVerbose = FALSE,
-           apVar = TRUE, .relStep = (.Machine$double.eps)^(1/3),
+           apVar = TRUE, .relStep = .Machine$double.eps^(1/3),
 	   opt = c("nlminb", "optim"),  optimMethod = "BFGS",
-           minAbsParApVar = 0.05)
+           ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+           minAbsParApVar = 0.05, sigma=NULL)
 {
+  ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+  if(is.null(sigma))
+    sigma <- 0
+  else  if(!is.finite(sigma) || length(sigma) != 1 || sigma < 0)
+    stop("Within-group std. dev. must be a positive numeric value")
   list(maxIter = maxIter, nlsMaxIter = nlsMaxIter, msMaxIter = msMaxIter,
        minScale = minScale, tolerance = tolerance, nlsTol = nlsTol,
        msTol = msTol, returnObject = returnObject,
        msVerbose = msVerbose, apVar = apVar,
        opt = match.arg(opt), optimMethod = optimMethod,
-       .relStep = .relStep, minAbsParApVar = minAbsParApVar)
+       ## 17-11-2015; Fixed sigma patch; SH Heisterkamp; Quantitative Solutions
+       .relStep = .relStep, minAbsParApVar = minAbsParApVar, sigma=sigma)
 }
+
+## Local Variables:
+## ess-indent-offset: 2
+## End:
