@@ -168,6 +168,15 @@
    */
 #define HAVE_DECL_ALLOCA 0
 
+
+/* Define to 1 if you have the declaration of `dladdr', and to 0 if you don't.
+   */
+/* #undef HAVE_DECL_DLADDR */
+
+/* Define to 1 if you have the declaration of `dlsym', and to 0 if you don't.
+   */
+/* #undef HAVE_DECL_DLSYM */
+
 /* Define to 1 if you have the declaration of `feof_unlocked', and to 0 if you
    don't. (For intl) */
 #define HAVE_DECL_FEOF_UNLOCKED 0
@@ -200,6 +209,14 @@
    don't. */
 #define HAVE_DECL_REALPATH 0
 
+/* Define to 1 if you have the declaration of `RTLD_DEFAULT', and to 0 if you
+   don't. */
+/* #undef HAVE_DECL_RTLD_DEFAULT */
+
+/* Define to 1 if you have the declaration of `RTLD_NEXT', and to 0 if you
+   don't. */
+/* #undef HAVE_DECL_RTLD_NEXT */
+
 /* Define to 1 if you have the declaration of `siglongjmp', and to 0 if you
    don't. */
 #define HAVE_DECL_SIGLONGJMP 1
@@ -236,9 +253,15 @@
    */
 #define HAVE_DIRENT_H 1
 
+/* Define to 1 if you have the `dladdr' function. */
+/* #undef HAVE_DLADDR */
+
 /* Define to 1 if you have the <dlfcn.h> header file. */
 /* #undef HAVE_DLFCN_H */
 
+/* Define to 1 if you have the `dlsym' function. */
+/* #undef HAVE_DLSYM */
+
 /* Define to 1 if you have the <dl.h> header file. */
 /* #undef HAVE_DL_H */
 
@@ -283,9 +306,6 @@
    can do arithmetic on the latter. */
 #define HAVE_FORTRAN_DOUBLE_COMPLEX 1
 
-/* Define to 1 if you have the <fpu_control.h> header file. */
-/* #undef HAVE_FPU_CONTROL_H */
-
 /* Define to 1 if fseeko (and presumably ftello) exists and is declared. */
 /* #undef HAVE_FSEEKO */
 
@@ -310,6 +330,9 @@
 /* Define to 1 if you have the `getgid' function. */
 /* #undef HAVE_GETGID */
 
+/* Define to 1 if you have the `getline' function. */
+/* #undef HAVE_GETLINE */
+
 /* Define to 1 if you have the `getgrgid' function. */
 /* #undef HAVE_GETGRGID */
 
@@ -365,6 +388,9 @@
 /* Define if you have the `iconvlist' function. */
 #define HAVE_ICONVLIST 1
 
+/* Define if `iconv' accepts "CP1252". */
+#define HAVE_ICONV_CP1252 1
+
 /* Define to 1 if you have the <iconv.h> header file. */
 #define HAVE_ICONV_H 1
 
@@ -421,7 +447,7 @@
 /* Define if your <locale.h> file defines LC_MESSAGES. */
 /* #undef HAVE_LC_MESSAGES */
 
-/* Define if your system has libcurl >= 7.28.0 with support for https.
+/* Define if your system has libcurl >= 7.22.0 with support for https.
    Set on the command line where supported.
  */
 /* #undef HAVE_LIBCURL */
@@ -562,9 +588,15 @@
 // #define HAVE_OPENMP 1
 #endif
 
+/* Define if your OpenMP 4 implementation fully supports SIMD reduction */
+/* #undef HAVE_OPENMP_SIMDRED */
+
 /* Define to 1 if you have pangocairo. */
 /* #undef HAVE_PANGOCAIRO */
 
+/* Define if your system has pcre2. */
+/* #undef HAVE_PCRE2 */
+
 /* Define to 1 if you have the <pcre.h> header file. */
 /* #undef HAVE_PCRE_H */
 
@@ -625,15 +657,21 @@
 /* Define to 1 if you have the `realpath' function. */
 /* #undef HAVE_REALPATH */
 
-/* Define to 1 if you have the `rint' function. */
-#define HAVE_RINT 1
-
 /* Define to 1 if you have the `rintl' function. */
 #define HAVE_RINTL 1
 
+/* Define to 1 if you have the `rl_callback_sigcleanup' function. */
+/* #undef HAVE_RL_CALLBACK_SIGCLEANUP */
+
 /* Define to 1 if you have the `rl_completion_matches' function. */
 /* #undef HAVE_RL_COMPLETION_MATCHES */
 
+/* Define to 1 if you have the `rl_resize_terminal' function. */
+/* #undef HAVE_RL_RESIZE_TERMINAL */
+
+/* Define to 1 if you have the `rl_sort_completion_matches' function. */
+/* #undef HAVE_RL_SORT_COMPLETION_MATCHES */
+
 /* Define to 1 if you have the `sched_getaffinity' function. */
 #undef HAVE_SCHED_GETAFFINITY
 
@@ -643,6 +681,9 @@
 /* Define to 1 if you have the `sched_setaffinity' function. */
 #undef HAVE_SCHED_SETAFFINITY
 
+/* Define to 1 if you have the `select' function. */
+/* #undef HAVE_SELECT */
+
 /* Define to 1 if you have the `setenv' function. */
 /* #undef HAVE_SETENV */
 
@@ -652,6 +693,9 @@
 /* Define to 1 if you have the `setlocale' function. */
 #define HAVE_SETLOCALE 1
 
+/* Define to 1 if you have the `setrlimit' function. */
+/* #undef HAVE_SETRLIMIT */
+
 /* Define to 1 if you have the `sigaction' function. */
 /* #undef HAVE_SIGACTION */
 
@@ -782,6 +826,12 @@
    to be built. */
 #define HAVE_TCLTK 1
 
+/* Define to 1 if you have the <thread.h> header file. */
+/* #undef HAVE_THREAD_H */
+
+/* Define to 1 if you have the `thr_stksegment' function. */
+/* #undef HAVE_THR_STKSEGMENT */
+
 /* Define this if libtiff is available. */
 #define HAVE_TIFF 1
 
@@ -837,6 +887,9 @@
 /* Define to 1 if you have the `utime' function. */
 /* #undef HAVE_UTIME */
 
+/* Define to 1 if you have the `utimensat' function. */
+/* #undef HAVE_UTIMENSAT */
+
 /* Define to 1 if you have the `utimes' function. */
 /* #undef HAVE_UTIMES */
 
@@ -930,9 +983,18 @@
 /* Define if you have the X11/Xmu headers and libraries. */
 /* #undef HAVE_X11_Xmu */
 
+/* Define to 1 if you have the `__cospi' function. */
+/* #undef HAVE___COSPI */
+
 /* Define to 1 if you have the `__fsetlocking' function. */
 /* #undef HAVE___FSETLOCKING */
 
+/* Define to 1 if you have the `__sinpi' function. */
+/* #undef HAVE___SINPI */
+
+/* Define to 1 if you have the `__tanpi' function. */
+/* #undef HAVE___TANPI */
+
 /* Define as const if the declaration of iconv() needs const. */
 #define ICONV_CONST const
 
@@ -948,15 +1010,12 @@
 /* Define to the sub-directory where libtool stores uninstalled libraries. */
 #define LT_OBJDIR ".libs/"
 
+/* macOS 10.12 (Sierra) or earlier */
+/* #undef MACOS_SIERRA */
+
 /* Define if mktime sets errno. */
 #define MKTIME_SETS_ERRNO 1
 
-/* Define if your system needs __setfpucw() to control FPU rounding. This was
-   used to control floating point precision, rounding and floating point
-   exceptions on older Linux systems. As of GLIBC 2.1 this function is not
-   used anymore. */
-/* #undef NEED___SETFPUCW */
-
 /* Define to disable Valgrind instrumentation */
 #define NVALGRIND 1
 
@@ -987,6 +1046,9 @@
 /* Define to the version of this package. */
 #define PACKAGE_VERSION "@VERSION@"
 
+/* PCRE2 code unit width wanted. */
+/* #undef PCRE2_CODE_UNIT_WIDTH */
+
 /* Define if <inttypes.h> exists and defines unusable PRI* macros. (For intl)
    */
 /* #undef PRI_MACROS_BROKEN */
