@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2014  The R Core Team
+ *  Copyright (C) 1997--2017  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -64,12 +64,12 @@ FILE *R_OpenInitFile(void)
 static int HaveHOME=-1;
 static char UserHOME[PATH_MAX];
 static char newFileName[PATH_MAX];
+
 const char *R_ExpandFileName(const char *s)
 {
     char *p;
 
-    if(s[0] != '~') return s;
-    if(isalpha(s[1])) return s;
+    if(s[0] != '~' || (s[0] && isalpha(s[1]))) return s;
     if(HaveHOME < 0) {
 	HaveHOME = 0;
 	p = getenv("R_USER"); /* should be set so the rest is a safety measure */
@@ -101,6 +101,26 @@ const char *R_ExpandFileName(const char *s)
     } else return s;
 }
 
+/* from sysutils.c */
+void reEnc2(const char *x, char *y, int ny,
+	    cetype_t ce_in, cetype_t ce_out, int subst);
+
+/* The following is a version of R_ExpandFileName that assumes
+   s is in UTF-8 and returns the final result in that encoding as well. */
+const char *R_ExpandFileNameUTF8(const char *s)
+{
+    if (s[0] !='~' || (s[0] && isalpha(s[1]))) return s;
+    else {
+    	char home[PATH_MAX];
+    	reEnc2(R_ExpandFileName("~"), home, PATH_MAX, CE_NATIVE, CE_UTF8, 3);
+    	if (strlen(home) + strlen(s+1) < PATH_MAX) {
+    	    strcpy(newFileName, home);
+    	    strcat(newFileName, s+1);
+    	    return newFileName;
+    	} else return s;
+    }
+}
+
 /*
  *  7) PLATFORM DEPENDENT FUNCTIONS
  */
@@ -173,6 +193,7 @@ SEXP do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
     const char *fout = "", *ferr = "";
     int   vis = 0, flag = 2, i = 0, j, ll = 0;
     SEXP  cmd, fin, Stdout, Stderr, tlist = R_NilValue, tchar, rval;
+    int timeout = 0, timedout = 0;
 
     checkArity(op, args);
     cmd = CAR(args);
@@ -191,7 +212,15 @@ SEXP do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
     Stdout = CAR(args);
     args = CDR(args);
     Stderr = CAR(args);
-    
+    args = CDR(args);
+    timeout = asInteger(CAR(args));
+    if (timeout == NA_INTEGER || timeout < 0 || timeout > 2000000)
+	/* the limit could be increased, but not much as in milliseconds it
+	   has to fit into a 32-bit unsigned integer */
+	errorcall(call, _("invalid '%s' argument"), "timeout");
+    if (timeout && !flag)
+	errorcall(call, "Timeout with background running processes is not supported.");
+
     if (CharacterMode == RGui) {
 	/* This is a rather conservative approach: if
 	   Rgui is launched from a console window it does have
@@ -212,9 +241,11 @@ SEXP do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     if (flag < 2) { /* Neither intern = TRUE nor
 		       show.output.on.console for Rgui */
-	ll = runcmd(CHAR(STRING_ELT(cmd, 0)),
+	ll = runcmd_timeout(CHAR(STRING_ELT(cmd, 0)),
 		    getCharCE(STRING_ELT(cmd, 0)),
-		    flag, vis, CHAR(STRING_ELT(fin, 0)), fout, ferr);
+		    flag, vis, CHAR(STRING_ELT(fin, 0)), fout, ferr,
+		    timeout, &timedout);
+	if (ll == NOLAUNCH) warning(runerror());
     } else {
 	/* read stdout +/- stderr from pipe */
 	int m = 0;
@@ -223,7 +254,7 @@ SEXP do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    m = asLogical(Stderr) ? 2 : 0;
 	if(m  && TYPEOF(Stdout) == LGLSXP && asLogical(Stdout)) m = 3;
 	fp = rpipeOpen(CHAR(STRING_ELT(cmd, 0)), getCharCE(STRING_ELT(cmd, 0)),
-		       vis, CHAR(STRING_ELT(fin, 0)), m, fout, ferr);
+		       vis, CHAR(STRING_ELT(fin, 0)), m, fout, ferr, timeout);
 	if (!fp) {
 	    /* If intern = TRUE generate an error */
 	    if (flag == 3) error(runerror());
@@ -247,13 +278,16 @@ SEXP do_system(SEXP call, SEXP op, SEXP args, SEXP rho)
 		for (i = 0; rpipeGets(fp, buf, INTERN_BUFSIZE); i++)
 		    R_WriteConsole(buf, strlen(buf));
 	    }
-	    ll = rpipeClose(fp);
+	    ll = rpipeClose(fp, &timedout);
 	}
     }
-    if(ll) {
-	warningcall(R_NilValue, 
-		    _("running command '%s' had status %d"), 
-		    CHAR(STRING_ELT(cmd, 0)), ll);
+    if (timedout) {
+	ll = 124;
+	warning(_("command '%s' timed out after %ds"),
+	        CHAR(STRING_ELT(cmd, 0)), timeout);
+    } else if (flag == 3 && ll) {
+	warning(_("running command '%s' had status %d"), 
+	        CHAR(STRING_ELT(cmd, 0)), ll);
     }
     if (flag == 3) { /* intern = TRUE: convert pairlist to list */
 	PROTECT(rval = allocVector(STRSXP, i));
