@@ -10,7 +10,8 @@
 R_CallMethodDef CallMethods[] = {
   {"mgcv_pmmult2", (DL_FUNC) &mgcv_pmmult2,5},
   {"mgcv_Rpiqr", (DL_FUNC) &mgcv_Rpiqr,5}, 
-  { "mgcv_tmm",(DL_FUNC)&mgcv_tmm,5}, 
+  { "mgcv_tmm",(DL_FUNC)&mgcv_tmm,5},
+  { "mgcv_chol_down",(DL_FUNC)&mgcv_chol_down,5}, 
   { "mgcv_Rpbsi",(DL_FUNC)&mgcv_Rpbsi,2},
   { "mgcv_RPPt",(DL_FUNC)&mgcv_RPPt,3},
   { "mgcv_Rpchol",(DL_FUNC)&mgcv_Rpchol,4},
@@ -29,12 +30,14 @@ R_CMethodDef CEntries[] = {
     {"tri_chol",(DL_FUNC) tri_chol,4},
     {"diagXVXt", (DL_FUNC) &diagXVXt,16},
     {"XWXd", (DL_FUNC) &XWXd,18},
+    {"XWXd0", (DL_FUNC) &XWXd0,18},
     {"XWyd", (DL_FUNC) &XWyd,18},
     {"Xbd", (DL_FUNC) &Xbd,15},
     {"vcorr", (DL_FUNC) &vcorr, 5},
     {"dchol", (DL_FUNC) &dchol, 4},
+    {"chol_down", (DL_FUNC) &chol_down, 5},
     {"mgcv_omp", (DL_FUNC) &mgcv_omp, 1},
-    {"coxpred", (DL_FUNC) &coxpred, 13},
+    {"coxpred", (DL_FUNC) &coxpred, 14},
     {"coxpp", (DL_FUNC) &coxpp, 10},
     {"coxlpl", (DL_FUNC) &coxlpl, 17},
     {"mvn_ll", (DL_FUNC) &mvn_ll,15},
