@@ -1038,17 +1038,18 @@ data.frame <-
 	    vjj <- value[[ jvseq[[jjj]] ]]
             if(jj <= nvars) {
                 ## if a column exists, preserve its attributes
-                if(length(dim(x[[jj]])) != 2L) x[[jj]][iseq] <- vjj
+                if(length(dim(x[[jj]])) != 2L)
+                     x[[jj]][iseq  ] <- vjj
                 else x[[jj]][iseq, ] <- vjj
             } else {
                 ## try to make a new column match in length: may be an error
                 x[[jj]] <- vjj[FALSE]
                 if(length(dim(vjj)) == 2L) {
-                    length(x[[j]]) <- nrows * ncol(vjj)
-                    dim(x[[j]]) <- c(nrows, ncol(vjj))
+                    length(x[[jj]]) <- nrows * ncol(vjj)
+                    dim(x[[jj]])  <- c(nrows,  ncol(vjj))
                     x[[jj]][iseq, ] <- vjj
                 } else {
-                    length(x[[j]]) <- nrows
+                    length(x[[jj]]) <- nrows
                     x[[jj]][iseq] <- vjj
                 }
             }
@@ -1477,7 +1478,7 @@ rbind.data.frame <- function(..., deparse.level = 1, make.row.names = TRUE,
 
 print.data.frame <-
     function(x, ..., digits = NULL, quote = FALSE, right = TRUE,
-	     row.names = TRUE)
+	     row.names = TRUE, max = NULL)
 {
     n <- length(row.names(x))
     if(length(x) == 0L) {
@@ -1489,12 +1490,21 @@ print.data.frame <-
 	print.default(names(x), quote = FALSE)
 	cat(gettext("<0 rows> (or 0-length row.names)\n"))
     } else {
+	if(is.null(max)) max <- getOption("max.print", 99999L)
+        if(!is.finite(max)) stop("invalid 'max' / getOption(\"max.print\"): ", max)
 	## format.<*>() : avoiding picking up e.g. format.AsIs
-	m <- as.matrix(format.data.frame(x, digits = digits, na.encode = FALSE))
+	omit <- (n0 <- max %/% length(x)) < n
+	m <- as.matrix(
+	    format.data.frame(if(omit) x[seq_len(n0), , drop=FALSE] else x,
+			      digits = digits, na.encode = FALSE))
 	if(!isTRUE(row.names))
 	    dimnames(m)[[1L]] <-
-		if(isFALSE(row.names)) rep.int("", n) else row.names
-	print(m, ..., quote = quote, right = right)
+		if(isFALSE(row.names)) rep.int("", if(omit) n0 else n)
+		else row.names
+	print(m, ..., quote = quote, right = right, max = max)
+	if(omit)
+	    cat(" [ reached 'max' / getOption(\"max.print\") -- omitted",
+		n - n0, "rows ]\n")
     }
     invisible(x)
 }
@@ -1644,7 +1654,7 @@ Ops.data.frame <- function(e1, e2 = NULL)
 	}
     }
     for(j in seq_along(cn)) {
-	left <- if(!lscalar) e1[[j]] else e1
+	left  <- if(!lscalar) e1[[j]] else e1
 	right <- if(!rscalar) e2[[j]] else e2
 	value[[j]] <- eval(f)
     }
