@@ -1,7 +1,7 @@
 #  File src/library/grDevices/R/cairo.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2015 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -25,34 +25,37 @@ svg <- function(filename = if(onefile) "Rplots.svg" else "Rplot%03d.svg",
     if(!checkIntFormat(filename)) stop("invalid 'filename'")
     antialiases <- eval(formals()$antialias)
     antialias <- match(match.arg(antialias, antialiases), antialiases)
-    invisible(.External(C_devCairo, filename, 4L, 72*width, 72*height, pointsize,
-                        bg, NA_integer_, antialias, onefile, family))
+    invisible(.External(C_devCairo, filename, 4L, 72*width, 72*height,
+                        pointsize, bg, NA_integer_, antialias, onefile,
+                        family, 300))
 }
 
 cairo_pdf <- function(filename = if(onefile) "Rplots.pdf" else "Rplot%03d.pdf",
                       width = 7, height = 7, pointsize = 12,
                       onefile = FALSE, family = "sans", bg = "white",
-                      antialias = c("default", "none", "gray", "subpixel"))
+                      antialias = c("default", "none", "gray", "subpixel"),
+                      fallback_resolution = 300)
 {
     if(!checkIntFormat(filename)) stop("invalid 'filename'")
     antialiases <- eval(formals()$antialias)
     antialias <- match(match.arg(antialias, antialiases), antialiases)
     invisible(.External(C_devCairo, filename, 6L, 72*width, 72*height,
                         pointsize, bg, NA_integer_, antialias, onefile,
-                        family))
+                        family, fallback_resolution))
 }
 
 cairo_ps <- function(filename = if(onefile) "Rplots.ps" else "Rplot%03d.ps",
                      width = 7, height = 7, pointsize = 12,
                      onefile = FALSE, family = "sans", bg = "white",
-                     antialias = c("default", "none", "gray", "subpixel"))
+                     antialias = c("default", "none", "gray", "subpixel"),
+                     fallback_resolution = 300)
 {
     if(!checkIntFormat(filename)) stop("invalid 'filename'")
     antialiases <- eval(formals()$antialias)
     antialias <- match(match.arg(antialias, antialiases), antialiases)
     invisible(.External(C_devCairo, filename, 7L, 72*width, 72*height,
                         pointsize, bg, NA_integer_, antialias, onefile,
-                        family))
+                        family, fallback_resolution))
 }
 
 cairoVersion <- function() .Call(C_cairoVersion)
