@@ -248,8 +248,8 @@ setWindowsFonts <- function(fonts, fontNames)
 printFont <- function(font) paste0(font, "\n")
 
 printFonts <- function(fonts)
-    cat(paste(names(fonts), ": ", unlist(lapply(fonts, printFont)),
-              sep="", collapse=""))
+    cat(paste0(names(fonts), ": ", unlist(lapply(fonts, printFont)),
+               collapse=""))
 
 # If no arguments spec'ed, return entire font database
 # If no named arguments spec'ed, all args should be font names
