@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2015  The R Core Team
+ *  Copyright (C) 1997--2018  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -399,11 +399,13 @@ BMDeviceDriver(pDevDesc dd, int kind, const char *filename,
 
     /* allocate new device description */
     if (!(xd = (pX11Desc) calloc(1, sizeof(X11Desc)))) return FALSE;
-    strcpy(xd->filename, filename);
+    strncpy(xd->filename, filename, PATH_MAX);
+    xd->filename[PATH_MAX - 1] = '\0';
     xd->quality = quality;
     xd->windowWidth = width;
     xd->windowHeight = height;
-    strncpy(xd->basefontfamily, family, 500);
+    strncpy(xd->basefontfamily, family, 499);
+    xd->basefontfamily[499] = '\0';
 #ifdef HAVE_PANGOCAIRO
     /* Pango's default resolution is 96 dpi */
     dps *= res0/96.0;
