@@ -178,8 +178,10 @@ grabDL <- function(warn, wrap, ...) {
           # Includes viewports, vpLists, vpTrees, and vpStacks
           # Check to see if about to push a viewport
           # with existing viewport name
-          if (vpExists(elt))
-            warning("viewport overwritten (grab MAY not be faithful)")
+          if (warn > 1) {
+            if (vpExists(elt))
+              warning("viewport overwritten (grab MAY not be faithful)")
+          }
           grid.draw(elt, recording=FALSE)
         ###########
         # grabPop
@@ -210,9 +212,14 @@ grid.grab <- function(warn=2, wrap=FALSE, ...) {
   grabDL(warn, wrap, ...)
 }
 
-grid.grabExpr <- function(expr, warn=2, wrap=FALSE, ...) {
+offscreen <- function(width, height) {
+    pdf(file=NULL, width=width, height=height)
+}
+
+grid.grabExpr <- function(expr, warn=2, wrap=FALSE,
+                          width=7, height=7, device=offscreen, ...) {
     ## Start an "offline" PDF device for this function
-    pdf(file=NULL)
+    device(width, height)
     on.exit(dev.off())
     ## Run the graphics code in expr
     ## Rely on lazy evaluation for correct "timing"
