@@ -1,7 +1,7 @@
 #  File src/library/grid/R/primitives.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 
 # Function that creates a description of an arrow head
@@ -26,7 +26,7 @@ arrow <- function(angle=30, length=unit(0.25, "inches"),
         stop("'length' must be a 'unit' object")
     ends <- as.integer(match(ends, c("first", "last", "both")))
     type <- as.integer(match(type, c("open", "closed")))
-    if (any(is.na(ends)) || any(is.na(type)) ||
+    if (anyNA(ends) || anyNA(type) ||
         length(ends) == 0 || length(type) == 0)
         stop("invalid 'ends' or 'type' argument")
     a <- list(angle=angle, length=length,
@@ -65,6 +65,12 @@ rep.arrow <- function(x, ...) {
     newa
 }
 
+str.arrow <- function(object, ...) {
+  o <- oldClass(object)
+  oldClass(object) <- setdiff(o, "arrow")
+  str(object)
+}
+
 ######################################
 # move-to and line-to primitives
 ######################################
@@ -79,7 +85,7 @@ validDetails.move.to <- function(x) {
 }
 
 drawDetails.move.to <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_moveTo, x$x, x$y)
+  grid.Call.graphics(C_moveTo, x$x, x$y)
 }
 
 moveToGrob <- function(x=0, y=0,
@@ -116,7 +122,7 @@ validDetails.line.to <- function(x) {
 }
 
 drawDetails.line.to <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_lineTo, x$x, x$y, x$arrow)
+  grid.Call.graphics(C_lineTo, x$x, x$y, x$arrow)
 }
 
 lineToGrob <- function(x=1, y=1,
@@ -155,13 +161,13 @@ validDetails.lines <- function(x) {
 }
 
 drawDetails.lines <- function(x, recording=TRUE) {
-    grid.Call.graphics(L_lines, x$x, x$y,
+    grid.Call.graphics(C_lines, x$x, x$y,
                        list(as.integer(1L:max(length(x$x), length(x$y)))),
                        x$arrow)
 }
 
 xDetails.lines <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -169,7 +175,7 @@ xDetails.lines <- function(x, theta) {
 }
 
 yDetails.lines <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -177,7 +183,7 @@ yDetails.lines <- function(x, theta) {
 }
 
 widthDetails.lines <- function(x) {
-  bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+  bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -185,7 +191,7 @@ widthDetails.lines <- function(x) {
 }
 
 heightDetails.lines <- function(x) {
-  bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+  bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -248,7 +254,7 @@ validDetails.polyline <- function(x) {
 
 drawDetails.polyline <- function(x, recording=TRUE) {
     if (is.null(x$id) && is.null(x$id.lengths))
-        grid.Call.graphics(L_lines, x$x, x$y,
+        grid.Call.graphics(C_lines, x$x, x$y,
                            list(as.integer(seq_along(x$x))),
                            x$arrow)
     else {
@@ -260,12 +266,12 @@ drawDetails.polyline <- function(x, recording=TRUE) {
             id <- x$id
         }
         index <- split(as.integer(seq_along(x$x)), id)
-        grid.Call.graphics(L_lines, x$x, x$y, index, x$arrow)
+        grid.Call.graphics(C_lines, x$x, x$y, index, x$arrow)
     }
 }
 
 xDetails.polyline <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -273,7 +279,7 @@ xDetails.polyline <- function(x, theta) {
 }
 
 yDetails.polyline <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -281,7 +287,7 @@ yDetails.polyline <- function(x, theta) {
 }
 
 widthDetails.polyline <- function(x) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
     if (is.null(bounds))
         unit(0, "inches")
     else
@@ -289,7 +295,7 @@ widthDetails.polyline <- function(x) {
 }
 
 heightDetails.polyline <- function(x) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
     if (is.null(bounds))
         unit(0, "inches")
     else
@@ -328,7 +334,7 @@ validDetails.segments <- function(x) {
 }
 
 drawDetails.segments <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_segments, x$x0, x$y0, x$x1, x$y1, x$arrow)
+  grid.Call.graphics(C_segments, x$x0, x$y0, x$x1, x$y1, x$arrow)
 }
 
 segmentBounds <- function(x, theta) {
@@ -338,7 +344,7 @@ segmentBounds <- function(x, theta) {
     x1 <- rep(x$x1, length.out=n)
     y0 <- rep(x$y0, length.out=n)
     y1 <- rep(x$y1, length.out=n)
-    grid.Call(L_locnBounds, unit.c(x0, x1), unit.c(y0, y1), theta)
+    grid.Call(C_locnBounds, unit.c(x0, x1), unit.c(y0, y1), theta)
 }
 
 xDetails.segments <- function(x, theta) {
@@ -485,11 +491,11 @@ drawDetails.arrows <- function(x, recording=TRUE) {
     yy <- rep(x$y, length.out=n)
     ynm1 <- yy[n - 1]
     yn <- yy[n]
-    grid.Call.graphics(L_lines, x$x, x$y,
+    grid.Call.graphics(C_lines, x$x, x$y,
                        list(as.integer(1L:n)),
                        NULL)
   }
-  grid.Call.graphics(L_arrows, x1, x2, xnm1, xn, y1, y2, ynm1, yn,
+  grid.Call.graphics(C_arrows, x1, x2, xnm1, xn, y1, y2, ynm1, yn,
                      x$angle, x$length, x$ends, x$type)
 }
 
@@ -500,7 +506,7 @@ widthDetails.arrows <- function(x) {
     lineThing <- getGrob(x, childNames(x))
     widthDetails(lineThing)
   } else {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
     if (is.null(bounds))
       unit(0, "inches")
     else
@@ -515,7 +521,7 @@ heightDetails.arrows <- function(x) {
     lineThing <- getGrob(x, childNames(x))
     heightDetails(lineThing)
   } else {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
     if (is.null(bounds))
       unit(0, "inches")
     else
@@ -524,37 +530,12 @@ heightDetails.arrows <- function(x) {
 }
 
 arrowsGrob <- function(x=c(0.25, 0.75), y=0.5,
-                        default.units="npc",
-                        grob=NULL,
-                        angle=30, length=unit(0.25, "inches"),
-                        ends="last", type="open",
-                        name=NULL, gp=gpar(), vp=NULL) {
-    warning("grid.arrows() has been deprecated;  use 'arrow' arguments to line drawing functions", domain = NA)
-  if (is.null(grob)) {
-    if (!is.unit(x))
-      x <- unit(x, default.units)
-    if (!is.unit(y))
-      y <- unit(y, default.units)
-  }
-  # Check the grob here
-  # Not in validDetails.arrows because that is for checking
-  # slots of an arrows object (the grob is a child of the arrows object)
-  # A possible alternative design would have a copy of the grob
-  # stored in a slot of the arrows object;  then it could be checked
-  # in the validDetails AND it could be edited
-  if (!is.null(grob)) {
-    # The grob can only be a "lines" or "segments"
-    # (splines would be another candidate if they existed)
-    if (!(inherits(grob, "lines") ||
-          inherits(grob, "segments") ||
-          inherits(grob, "line.to")))
-      stop("the 'grob' argument must be a 'line.to', 'lines', or 'segments' grob")
-    x <- y <- NULL
-  }
-  gTree(x=x, y=y, children=if (is.null(grob)) NULL else gList(grob),
-       angle=as.numeric(angle), length=length,
-       ends=ends, type=type,
-       name=name, gp=gp, vp=vp, cl="arrows")
+                       default.units="npc",
+                       grob=NULL,
+                       angle=30, length=unit(0.25, "inches"),
+                       ends="last", type="open",
+                       name=NULL, gp=gpar(), vp=NULL) {
+    .Defunct(msg="'arrowsGrob' is defunct; use 'arrow' arguments to line drawing functions")
 }
 
 grid.arrows <- function(x=c(0.25, 0.75), y=0.5,
@@ -563,14 +544,7 @@ grid.arrows <- function(x=c(0.25, 0.75), y=0.5,
                         angle=30, length=unit(0.25, "inches"),
                         ends="last", type="open",
                         name=NULL, gp=gpar(), draw=TRUE, vp=NULL) {
-  ag <- arrowsGrob(x=x, y=y,
-                   default.units=default.units,
-                   grob=grob, angle=angle, length=length,
-                   ends=ends, type=type,
-                   name=name, gp=gp, vp=vp)
-  if (draw)
-    grid.draw(ag)
-  invisible(ag)
+    .Defunct(msg="'grid.arrows' is defunct; use 'arrow' arguments to line drawing functions")
 }
 
 ######################################
@@ -598,7 +572,7 @@ validDetails.polygon <- function(x) {
 
 drawDetails.polygon <- function(x, recording=TRUE) {
   if (is.null(x$id) && is.null(x$id.lengths))
-    grid.Call.graphics(L_polygon, x$x, x$y,
+    grid.Call.graphics(C_polygon, x$x, x$y,
                        list(as.integer(seq_along(x$x))))
   else {
     if (is.null(x$id)) {
@@ -609,12 +583,12 @@ drawDetails.polygon <- function(x, recording=TRUE) {
       id <- x$id
     }
     index <- split(as.integer(seq_along(x$x)), id)
-    grid.Call.graphics(L_polygon, x$x, x$y, index)
+    grid.Call.graphics(C_polygon, x$x, x$y, index)
   }
 }
 
 xDetails.polygon <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -622,7 +596,7 @@ xDetails.polygon <- function(x, theta) {
 }
 
 yDetails.polygon <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -630,7 +604,7 @@ yDetails.polygon <- function(x, theta) {
 }
 
 widthDetails.polygon <- function(x) {
-  bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+  bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -638,7 +612,7 @@ widthDetails.polygon <- function(x) {
 }
 
 heightDetails.polygon <- function(x) {
-  bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+  bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -693,7 +667,7 @@ validDetails.pathgrob <- function(x) {
 }
 
 xDetails.pathgrob <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -701,7 +675,7 @@ xDetails.pathgrob <- function(x, theta) {
 }
 
 yDetails.pathgrob <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -709,7 +683,7 @@ yDetails.pathgrob <- function(x, theta) {
 }
 
 widthDetails.pathgrob <- function(x) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
     if (is.null(bounds))
         unit(0, "inches")
     else
@@ -717,7 +691,7 @@ widthDetails.pathgrob <- function(x) {
 }
 
 heightDetails.pathgrob <- function(x) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
     if (is.null(bounds))
         unit(0, "inches")
     else
@@ -727,7 +701,7 @@ heightDetails.pathgrob <- function(x) {
 
 drawDetails.pathgrob <- function(x, recording=TRUE) {
       if (is.null(x$id) && is.null(x$id.lengths))
-          grid.Call.graphics(L_polygon, x$x, x$y,
+          grid.Call.graphics(C_polygon, x$x, x$y,
                              list(as.integer(seq_along(x$x))))
   else {
     if (is.null(x$id)) {
@@ -738,7 +712,7 @@ drawDetails.pathgrob <- function(x, recording=TRUE) {
       id <- x$id
     }
     index <- split(as.integer(seq_along(x$x)), id)
-    grid.Call.graphics(L_path, x$x, x$y, index,
+    grid.Call.graphics(C_path, x$x, x$y, index,
                        switch(x$rule, winding=1L, evenodd=0L))
   }
 }
@@ -786,7 +760,7 @@ validDetails.xspline <- function(x) {
     x$id.lengths <- as.integer(x$id.lengths)
   if (!(is.null(x$arrow) || inherits(x$arrow, "arrow")))
       stop("invalid 'arrow' argument")
-  if (any(x$shape < -1 || x$shape > 1))
+  if (any(x$shape < -1 | x$shape > 1))
     stop("'shape' must be between -1 and 1")
   x$open <- as.logical(x$open)
   # Force all first and last shapes to be 0 for open xsplines
@@ -817,12 +791,12 @@ xsplineIndex <- function(x) {
 }
 
 drawDetails.xspline <- function(x, recording=TRUE) {
-    grid.Call.graphics(L_xspline, x$x, x$y, x$shape, x$open, x$arrow,
+    grid.Call.graphics(C_xspline, x$x, x$y, x$shape, x$open, x$arrow,
                        x$repEnds, xsplineIndex(x))
 }
 
 xDetails.xspline <- function(x, theta) {
-  bounds <- grid.Call(L_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
+  bounds <- grid.Call(C_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
                       x$repEnds, xsplineIndex(x), theta)
   if (is.null(bounds))
     unit(0.5, "npc")
@@ -831,7 +805,7 @@ xDetails.xspline <- function(x, theta) {
 }
 
 yDetails.xspline <- function(x, theta) {
-  bounds <- grid.Call(L_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
+  bounds <- grid.Call(C_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
                       x$repEnds, xsplineIndex(x), theta)
   if (is.null(bounds))
     unit(0.5, "npc")
@@ -840,7 +814,7 @@ yDetails.xspline <- function(x, theta) {
 }
 
 widthDetails.xspline <- function(x) {
-  bounds <- grid.Call(L_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
+  bounds <- grid.Call(C_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
                       x$repEnds, list(as.integer(seq_along(x$x))), 0)
   if (is.null(bounds))
     unit(0, "inches")
@@ -849,7 +823,7 @@ widthDetails.xspline <- function(x) {
 }
 
 heightDetails.xspline <- function(x) {
-  bounds <- grid.Call(L_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
+  bounds <- grid.Call(C_xsplineBounds, x$x, x$y, x$shape, x$open, x$arrow,
                       x$repEnds, list(as.integer(seq_along(x$x))), 0)
   if (is.null(bounds))
     unit(0, "inches")
@@ -877,13 +851,13 @@ grid.xspline <- function(...) {
 
 xsplinePoints <- function(x) {
     # Mimic drawGrob() to ensure x$vp and x$gp enforced
-    dlon <- grid.Call(L_setDLon, FALSE)
-    on.exit(grid.Call(L_setDLon, dlon))
-    tempgpar <- grid.Call(L_getGPar)
-    on.exit(grid.Call(L_setGPar, tempgpar), add=TRUE)
+    dlon <- grid.Call(C_setDLon, FALSE)
+    on.exit(grid.Call(C_setDLon, dlon))
+    tempgpar <- grid.Call(C_getGPar)
+    on.exit(grid.Call(C_setGPar, tempgpar), add=TRUE)
     preDraw(x)
     # Raw pts in dev coords
-    devPoints <- grid.Call(L_xsplinePoints,
+    devPoints <- grid.Call(C_xsplinePoints,
                            x$x, x$y, x$shape, x$open, x$arrow,
                            x$repEnds, xsplineIndex(x), 0)
     postDraw(x)
@@ -961,16 +935,17 @@ validDetails.beziergrob <- function(x) {
     if (!is.null(x$id.lengths))
         x$id.lengths <- as.integer(x$id.lengths)
     if (is.null(x$id) && is.null(x$id.lengths)) {
-        if (length(x$x) != 4)
+        if (length(x$x) != 4L)
             stop("must have exactly 4 control points")
     } else {
         if (is.null(x$id)) {
+            n <- length(x$id.lengths)
             id <- rep(1L:n, x$id.lengths)
         } else {
             id <- x$id
         }
         xper <- split(x$x, id)
-        if (any(sapply(xper, length) != 4))
+        if (any(lengths(xper) != 4L))
             stop("must have exactly 4 control points per Bezier curve")
     }
     if (!(is.null(x$arrow) || inherits(x$arrow, "arrow")))
@@ -1036,11 +1011,11 @@ validDetails.circle <- function(x) {
 }
 
 drawDetails.circle <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_circle, x$x, x$y, x$r)
+  grid.Call.graphics(C_circle, x$x, x$y, x$r)
 }
 
 xDetails.circle <- function(x, theta) {
-  bounds <- grid.Call(L_circleBounds, x$x, x$y, x$r, theta)
+  bounds <- grid.Call(C_circleBounds, x$x, x$y, x$r, theta)
   if (is.null(bounds))
     unit(0.5, "npc")
   else
@@ -1048,7 +1023,7 @@ xDetails.circle <- function(x, theta) {
 }
 
 yDetails.circle <- function(x, theta) {
-  bounds <- grid.Call(L_circleBounds, x$x, x$y, x$r, theta)
+  bounds <- grid.Call(C_circleBounds, x$x, x$y, x$r, theta)
   if (is.null(bounds))
     unit(0.5, "npc")
   else
@@ -1056,7 +1031,7 @@ yDetails.circle <- function(x, theta) {
 }
 
 widthDetails.circle <- function(x) {
-  bounds <- grid.Call(L_circleBounds, x$x, x$y, x$r, 0)
+  bounds <- grid.Call(C_circleBounds, x$x, x$y, x$r, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -1064,7 +1039,7 @@ widthDetails.circle <- function(x) {
 }
 
 heightDetails.circle <- function(x) {
-  bounds <- grid.Call(L_circleBounds, x$x, x$y, x$r, 0)
+  bounds <- grid.Call(C_circleBounds, x$x, x$y, x$r, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -1112,13 +1087,13 @@ validDetails.rect <- function(x) {
 }
 
 drawDetails.rect <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_rect, x$x, x$y, x$width, x$height,
+  grid.Call.graphics(C_rect, x$x, x$y, x$width, x$height,
                      resolveHJust(x$just, x$hjust),
                      resolveVJust(x$just, x$vjust))
 }
 
 xDetails.rect <- function(x, theta) {
-  bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+  bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
                       theta)
@@ -1129,7 +1104,7 @@ xDetails.rect <- function(x, theta) {
 }
 
 yDetails.rect <- function(x, theta) {
-  bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+  bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
                       theta)
@@ -1140,7 +1115,7 @@ yDetails.rect <- function(x, theta) {
 }
 
 widthDetails.rect <- function(x) {
-  bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+  bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
                       0)
@@ -1151,7 +1126,7 @@ widthDetails.rect <- function(x) {
 }
 
 heightDetails.rect <- function(x) {
-  bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+  bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
                       0)
@@ -1275,7 +1250,7 @@ drawDetails.rastergrob <- function(x, recording=TRUE) {
                              "inches")
         }
     }
-    grid.Call.graphics(L_raster, x$raster,
+    grid.Call.graphics(C_raster, x$raster,
                        x$x, x$y, x$width, x$height,
                        resolveHJust(x$just, x$hjust),
                        resolveVJust(x$just, x$vjust),
@@ -1284,7 +1259,7 @@ drawDetails.rastergrob <- function(x, recording=TRUE) {
 
 xDetails.rastergrob <- function(x, theta) {
     x <- resolveRasterSize(x)
-    bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+    bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                         resolveHJust(x$just, x$hjust),
                         resolveVJust(x$just, x$vjust),
                         theta)
@@ -1296,7 +1271,7 @@ xDetails.rastergrob <- function(x, theta) {
 
 yDetails.rastergrob <- function(x, theta) {
     x <- resolveRasterSize(x)
-    bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+    bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                         resolveHJust(x$just, x$hjust),
                         resolveVJust(x$just, x$vjust),
                         theta)
@@ -1308,7 +1283,7 @@ yDetails.rastergrob <- function(x, theta) {
 
 widthDetails.rastergrob <- function(x) {
     x <- resolveRasterSize(x)
-    bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+    bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                         resolveHJust(x$just, x$hjust),
                         resolveVJust(x$just, x$vjust),
                         0)
@@ -1320,7 +1295,7 @@ widthDetails.rastergrob <- function(x) {
 
 heightDetails.rastergrob <- function(x) {
     x <- resolveRasterSize(x)
-    bounds <- grid.Call(L_rectBounds, x$x, x$y, x$width, x$height,
+    bounds <- grid.Call(C_rectBounds, x$x, x$y, x$width, x$height,
                         resolveHJust(x$just, x$hjust),
                         resolveVJust(x$just, x$vjust),
                         0)
@@ -1392,7 +1367,7 @@ validDetails.text <- function(x) {
 }
 
 drawDetails.text <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_text, as.graphicsAnnot(x$label),
+  grid.Call.graphics(C_text, as.graphicsAnnot(x$label),
                      x$x, x$y,
                      resolveHJust(x$just, x$hjust),
                      resolveVJust(x$just, x$vjust),
@@ -1400,7 +1375,7 @@ drawDetails.text <- function(x, recording=TRUE) {
 }
 
 xDetails.text <- function(x, theta) {
-  bounds <- grid.Call(L_textBounds, as.graphicsAnnot(x$label),
+  bounds <- grid.Call(C_textBounds, as.graphicsAnnot(x$label),
                       x$x, x$y,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
@@ -1412,7 +1387,7 @@ xDetails.text <- function(x, theta) {
 }
 
 yDetails.text <- function(x, theta) {
-  bounds <- grid.Call(L_textBounds, as.graphicsAnnot(x$label),
+  bounds <- grid.Call(C_textBounds, as.graphicsAnnot(x$label),
                       x$x, x$y,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
@@ -1424,7 +1399,7 @@ yDetails.text <- function(x, theta) {
 }
 
 widthDetails.text <- function(x) {
-  bounds <- grid.Call(L_textBounds, as.graphicsAnnot(x$label),
+  bounds <- grid.Call(C_textBounds, as.graphicsAnnot(x$label),
                       x$x, x$y,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
@@ -1436,7 +1411,7 @@ widthDetails.text <- function(x) {
 }
 
 heightDetails.text <- function(x) {
-  bounds <- grid.Call(L_textBounds, as.graphicsAnnot(x$label),
+  bounds <- grid.Call(C_textBounds, as.graphicsAnnot(x$label),
                       x$x, x$y,
                       resolveHJust(x$just, x$hjust),
                       resolveVJust(x$just, x$vjust),
@@ -1449,7 +1424,7 @@ heightDetails.text <- function(x) {
 
 ascentDetails.text <- function(x) {
     if (length(x$label) == 1) {
-        metrics <- grid.Call(L_stringMetric, as.graphicsAnnot(x$label))
+        metrics <- grid.Call(C_stringMetric, as.graphicsAnnot(x$label))
         unit(metrics[[1]], "inches")
     } else {
         heightDetails(x)
@@ -1458,7 +1433,7 @@ ascentDetails.text <- function(x) {
 
 descentDetails.text <- function(x) {
     if (length(x$label) == 1) {
-        metrics <- grid.Call(L_stringMetric, as.graphicsAnnot(x$label))
+        metrics <- grid.Call(C_stringMetric, as.graphicsAnnot(x$label))
         unit(metrics[[2]], "inches")
     } else {
         unit(0, "inches")
@@ -1519,12 +1494,12 @@ validDetails.points <- function(x) {
 }
 
 drawDetails.points <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_points, x$x, x$y, x$pch, x$size)
+  grid.Call.graphics(C_points, x$x, x$y, x$pch, x$size)
 }
 
 # FIXME:  does not take into account the size of the symbols
 xDetails.points <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -1532,7 +1507,7 @@ xDetails.points <- function(x, theta) {
 }
 
 yDetails.points <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -1540,7 +1515,7 @@ yDetails.points <- function(x, theta) {
 }
 
 widthDetails.points <- function(x) {
-  bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+  bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -1548,7 +1523,7 @@ widthDetails.points <- function(x) {
 }
 
 heightDetails.points <- function(x) {
-  bounds <- grid.Call(L_locnBounds, x$x, x$y, 0)
+  bounds <- grid.Call(C_locnBounds, x$x, x$y, 0)
   if (is.null(bounds))
     unit(0, "inches")
   else
@@ -1603,7 +1578,7 @@ validDetails.clip <- function(x) {
 }
 
 drawDetails.clip <- function(x, recording=TRUE) {
-  grid.Call.graphics(L_clip, x$x, x$y, x$width, x$height,
+  grid.Call.graphics(C_clip, x$x, x$y, x$width, x$height,
                      resolveHJust(x$just, x$hjust),
                      resolveVJust(x$just, x$vjust))
 }
@@ -1653,7 +1628,7 @@ drawDetails.null <- function(x, recording=TRUE) {
 }
 
 xDetails.null <- function(x, theta) {
-    bounds <- grid.Call(L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call(C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
@@ -1661,7 +1636,7 @@ xDetails.null <- function(x, theta) {
 }
 
 yDetails.null <- function(x, theta) {
-    bounds <- grid.Call( L_locnBounds, x$x, x$y, theta)
+    bounds <- grid.Call( C_locnBounds, x$x, x$y, theta)
     if (is.null(bounds))
         unit(0.5, "npc")
     else
