@@ -373,8 +373,10 @@ is.unit <- function(unit) {
   inherits(unit, "unit")
 }
 
-print.unit <- function(x, ...)
-  print(as.character(x), quote=FALSE, ...)
+print.unit <- function(x, ...) {
+    print(as.character(x), quote=FALSE, ...)
+    invisible(x)
+}
 
 
 #########################
