@@ -1,7 +1,7 @@
 #  File src/library/methods/R/BasicFunsList.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## Lists of functions and expressions used in dispatch of functions
 ## defined internally (as .Primitive's) for which formal argument lists
@@ -22,9 +22,8 @@
 ## needs to have a special form (e.g., belonging to one of the
 ## predefined groups of functions).
 
-## The list is expanded in .makeBasicFuns by adding the S4 group generics
-## and the remaining primitives.
-
+##' The list is expanded in .makeBasicFuns() -> ./makeBasicFunsList.R by
+##' adding the S4 group generics and the remaining primitives.
 .BasicFunsList <-
 list(
 ### subset/subassignment ops are regarded as language elements
@@ -46,8 +45,7 @@ list(
 , "%*%" = function(x, y) standardGeneric("%*%")
 , "xtfrm" = function(x) standardGeneric("xtfrm")
 ### these have a different arglist from the primitives
-, "c" = structure(function(x, ..., recursive = FALSE) standardGeneric("c"),
-                  signature="x")
+, "c" = structure(function(x, ...) standardGeneric("c"), signature="x")
 , "all" = structure(function(x, ..., na.rm = FALSE) standardGeneric("all"),
                     signature="x")
 , "any" = structure(function(x, ..., na.rm = FALSE) standardGeneric("any"),
@@ -77,26 +75,38 @@ list(
 ## functions.
 
 .addBasicGeneric <-
-    function(funslist, f, fdef, group = list())
+    function(funslist, f, fdef, group = list(), internal = FALSE,
+             internalArgs = names(formals(deflt)))
 {
-    signature <- attr(fdef, "signature") #typically NULL, but see the case for "$"
-    deflt <- get(f, "package:base")
+    deflt <- .BaseNamespaceEnv[[f]]
     ## use the arguments of the base package function
     ##FIXME:  should also deal with the functions having ... as the first
     ## argument, but needs to create a generic with different args from the deflt
     ## => constructing a call to the base function from the default
     if(is.primitive(deflt)) {
-        body(fdef, envir = globalenv()) <-
+        signature <- attr(fdef, "signature") #typically NULL, but see the case for "$"
+        body(fdef, envir = topenv()) <-
             substitute(standardGeneric(FNAME, DEFLT), list(FNAME=f, DEFLT=deflt))
     }
     else {
+        if (internal) {
+	    ## "forgets" the *defaults* of arguments, e.g. the "any" of as.vector():
+	    ## formals(deflt) <- setNames(rep(alist(x=), length(internalArgs)),
+	    ##                            internalArgs)
+            call <- as.call(c(as.name(f), lapply(internalArgs, as.name)))
+            body(deflt, envir = baseenv()) <-
+                substitute(.Internal(CALL), list(CALL=call))
+        }
         fdef <- deflt
-        body(fdef, envir = globalenv()) <-
+        body(fdef, envir = topenv()) <-
             substitute(standardGeneric(FNAME), list(FNAME=f))
     }
-    deflt <- .derivedDefaultMethod(deflt)
-    elNamed(funslist, f) <- makeGeneric(f, fdef, deflt, group = group, package = "base",
-                                        signature = signature)
+    deflt <- .derivedDefaultMethod(deflt, internal = if (internal) f)
+    if (internal) {
+        signature <- names(formals(deflt))[1L]
+    }
+    funslist[[f]] <- makeGeneric(f, fdef, deflt, group = group, package = "base",
+                                 signature = signature)
     funslist
 }
 
@@ -122,24 +132,26 @@ list(
 
 ## temporary versions while primitives are still handled by a global table
 
-genericForPrimitive <- function(f, where = topenv(parent.frame()), mustFind = TRUE) {
-#    if(.matchBasic(f, .ExcludePrimitiveGenerics, FALSE))
-#        stop(gettextf("methods may not be defined for primitive function %s in this version of R", sQuote(f)), domain = NA)
-    env <- .findBasicFuns(where)
-    funs <- get(".BasicFunsList", envir = env)
-    ans <- elNamed(funs, f)
-    ## this element may not exist (yet, during loading), dom't test null
-    if(mustFind && identical(ans, FALSE))
+isBaseFun <- function(fun) {
+    is.primitive(fun) || identical(environment(fun), .BaseNamespaceEnv)
+}
+
+inBasicFuns <- function(f) {
+    !is.null(.BasicFunsList[[f]])
+}
+
+dispatchIsInternal <- function(fdef) {
+    is.primitive(fdef@default) || is(fdef@default, "internalDispatchMethod")
+}
+
+genericForBasic <- function(f, where = topenv(parent.frame()),
+                            mustFind = TRUE)
+{
+    ans <- .BasicFunsList[[f]]
+    ## this element may not exist (yet, during loading), don't test null
+    if(mustFind && isFALSE(ans))
         stop(gettextf("methods may not be defined for primitive function %s in this version of R",
                       sQuote(f)),
              domain = NA)
     ans
 }
-
-.findBasicFuns <- function(where) {
-    allWhere <- .findAll(".BasicFunsList", where = where)
-    if(length(allWhere) == 0)
-        .methodsNamespace
-    else
-        as.environment(allWhere[[1L]])
-}
