@@ -1,7 +1,7 @@
 #  File src/library/parallel/R/unix/mcmapply.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@ mcmapply <-
     function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE, USE.NAMES = TRUE,
              mc.preschedule = TRUE, mc.set.seed = TRUE,
              mc.silent = FALSE, mc.cores = getOption("mc.cores", 2L),
-             mc.cleanup = TRUE)
+             mc.cleanup = TRUE, affinity.list = NULL)
 {
     FUN <- match.fun(FUN)
     dots <- list(...)
@@ -29,8 +29,16 @@ mcmapply <-
     n <- max(lens)
     if(n && min(lens) == 0L)
         stop("Zero-length inputs cannot be mixed with those of non-zero length")
-    answer <- if(n < 2L) .mapply(FUN, dots, MoreArgs)
-    else {
+    answer <- if(n < 2L){ 
+      ## ensure that it runs on the right core 
+      if(!is.null(affinity.list)){
+        save <- mcaffinity()
+        mcaffinity(affinity.list[[1]])
+      }
+      answer <- .mapply(FUN, dots, MoreArgs) 
+      if(!is.null(affinity.list)) mcaffinity(save)
+      answer
+    } else {    
         ## recycle shorter vectors
         X <- if (!all(lens == n))
             lapply(dots, function(x) rep(x, length.out = n))
@@ -41,7 +49,8 @@ mcmapply <-
         }
         answer <- mclapply(seq_len(n), do_one, mc.preschedule = mc.preschedule,
                            mc.set.seed = mc.set.seed, mc.silent = mc.silent,
-                           mc.cores = mc.cores, mc.cleanup = mc.cleanup)
+                           mc.cores = mc.cores, mc.cleanup = mc.cleanup, 
+                           affinity.list = affinity.list)
         do.call(c, answer)
     }
     if (USE.NAMES && length(dots)) {
