@@ -258,7 +258,7 @@ terms.formula <- function(x, specials = NULL, abb = NULL, data = NULL,
             tmp <- c(tmp, tmp2[ind])
         }
 	rhs <- if(length(tmp)) paste(tmp, collapse = " + ") else "1"
-	if(!attr(terms(object), "intercept")) rhs <- paste(rhs, "- 1")
+	if(!attr(Terms, "intercept")) rhs <- paste(rhs, "- 1")
         if(length(form <- formula(object)) > 2L) {
             res <- formula(paste("lhs ~", rhs))
             res[[2L]] <- form[[2L]]
@@ -282,7 +282,12 @@ terms.formula <- function(x, specials = NULL, abb = NULL, data = NULL,
 }
 
 coef <- function(object, ...) UseMethod("coef")
-coef.default <- function(object, ...) object$coefficients
+## 'complete': be compatible with vcov()
+coef.default <- function(object, complete=TRUE, ...) {
+    cf <- object$coefficients
+    if(complete) cf else cf[!is.na(cf)]
+}
+coef.aov <- coef.default; formals(coef.aov)[["complete"]] <- FALSE
 coefficients <- coef
 
 residuals <- function(object, ...) UseMethod("residuals")
@@ -361,6 +366,7 @@ offset <- function(object) object
     }
 }
 
+##' Model Frame Class
 .MFclass <- function(x)
 {
     ## the idea is to identify the relevant classes that model.matrix
@@ -381,6 +387,13 @@ offset <- function(object) object
     return("other")
 }
 
+##' A complete deparse for "models", i.e. for formula and variable names (PR#15377)
+##' @param width.cutoff = 500L: Some people have generated longer variable names
+##' https://stat.ethz.ch/pipermail/r-devel/2010-October/058756.html
+deparse2 <- function(x)
+    paste(deparse(x, width.cutoff = 500L, backtick = !is.symbol(x) && is.language(x)),
+          collapse = " ")
+
 model.frame <- function(formula, ...) UseMethod("model.frame")
 model.frame.default <-
     function(formula, data = NULL, subset = NULL, na.action = na.fail,
@@ -443,10 +456,7 @@ model.frame.default <-
     vars <- attr(formula, "variables")
     predvars <- attr(formula, "predvars")
     if(is.null(predvars)) predvars <- vars
-    ## Some people have generated longer variable names
-    ## https://stat.ethz.ch/pipermail/r-devel/2010-October/058756.html
-    varnames <- sapply(vars, function(x) paste(deparse(x,width.cutoff=500),
-                                               collapse=' '))[-1L]
+    varnames <- vapply(vars, deparse2, " ")[-1L]
     variables <- eval(predvars, data, env)
     resp <- attr(formula, "response")
     if(is.null(rownames) && resp > 0L) {
@@ -550,9 +560,6 @@ model.matrix.default <- function(object, data = environment(object),
     if (is.null(attr(data, "terms")))
 	data <- model.frame(object, data, xlev=xlev)
     else {
-        ## need complete deparse, PR#15377
-        deparse2 <- function(x)
-            paste(deparse(x, width.cutoff = 500L), collapse = " ")
 	reorder <- match(vapply(attr(t, "variables"), deparse2, "")[-1L],
                          names(data))
 	if (anyNA(reorder))
@@ -594,7 +601,7 @@ model.matrix.default <- function(object, data = environment(object),
 	isF <- FALSE
 	data[["x"]] <- raw(nrow(data))
     }
-    ans <- .External2(C_modelmatrix, t, data)
+    ans <- .External2(C_modelmatrix, t, data) # modelmatrix() in ../src/model.c
     cons <- if(any(isF))
 	lapply(data[isF], attr, "contrasts") ## else NULL
     attr(ans, "contrasts") <- cons
@@ -664,9 +671,7 @@ makepredictcall.default  <- function(var, call)
 
 .getXlevels <- function(Terms, m)
 {
-    deparse2 <- function(x)
-        paste(deparse(x, width.cutoff = 500L), collapse = " ")
-    xvars <- sapply(attr(Terms, "variables"), deparse2)[-1L]
+    xvars <- vapply(attr(Terms, "variables"), deparse2, "")[-1L]
     if((yvar <- attr(Terms, "response")) > 0) xvars <- xvars[-yvar]
     if(length(xvars)) {
         xlev <- lapply(m[xvars],
