@@ -27,7 +27,7 @@ splinefun <-
     x <- regularize.values(x, y, ties) # -> (x,y) numeric of same length
     y <- x$y
     x <- x$x
-    nx <- as.integer(length(x))
+    nx <- length(x) # large vectors ==> non-integer
     if(is.na(nx)) stop(gettextf("invalid value of %s", "length(x)"), domain = NA)
     if(nx == 0) stop("zero non-NA points")
     method <- match.arg(method)
