@@ -277,7 +277,7 @@ function(package, dir, lib.loc = NULL)
         ## with a DB of the parsed (and platform processed, see
         ## above) Rd objects.
         db_file <- file.path(dir, "help", package)
-        if(file_test("-f", paste(db_file, "rdx", sep = "."))) {
+        if(file_test("-f", paste0(db_file, ".rdx"))) {
             db <- fetchRdDB(db_file)
             pathfile <- file.path(dir, "help", "paths.rds")
             if(file.exists(pathfile)) {
@@ -352,13 +352,14 @@ function(x, ...)
 }
 
 .build_Rd_db <-
-function(dir = NULL, files = NULL, encoding = "unknown", db_file = NULL,
-         stages = c("build", "install"), os = .OStype(), step = 3L, built_file = NULL)
+function(dir = NULL, files = NULL,
+         encoding = "unknown", db_file = NULL,
+         stages = c("build", "install"), os = .OStype(), step = 3L,
+         built_file = NULL, macros = character())
 {
-    macros <- file.path(R.home("share"), "Rd", "macros", "system.Rd")
     if(!is.null(dir)) {
         dir <- file_path_as_absolute(dir)
-        macros <- loadPkgRdMacros(dir, macros)
+        macros0 <- loadPkgRdMacros(dir)
         man_dir <- file.path(dir, "man")
         if(!dir.exists(man_dir))
             return(structure(list(), names = character()))
@@ -366,9 +367,19 @@ function(dir = NULL, files = NULL, encoding = "unknown", db_file = NULL,
             files <- list_files_with_type(man_dir, "docs", OS_subdirs=os)
         encoding <- .get_package_metadata(dir, FALSE)["Encoding"]
         if(is.na(encoding)) encoding <- "unknown"
-    } else if(is.null(files))
+    } else if(!is.null(files))
+        macros0 <- initialRdMacros()
+    else
         stop("you must specify 'dir' or 'files'")
 
+    if(length(macros)) {
+        con <- textConnection(macros)
+        macros <- loadRdMacros(con, macros0)
+        close(con)
+    } else {
+        macros <- macros0
+    }
+
     .fetch_Rd_object <- function(f) {
         ## This calls parse_Rd if f is a filename
         Rd <- prepare_Rd(f, encoding = encoding,
@@ -387,10 +398,10 @@ function(dir = NULL, files = NULL, encoding = "unknown", db_file = NULL,
         ## Files in the db in need of updating:
         indf <- (files %in% db_names) & file_test("-nt", files, db_file)
         ## Also files not in the db:
-        indf <- indf | !(files %in% db_names)
+        indf <- indf | (files %notin% db_names)
 
         ## Db elements missing from files:
-        ind <- !(db_names %in% files) | (db_names %in% files[indf])
+        ind <- (db_names %notin% files) | (db_names %in% files[indf])
 	if(any(ind))
             db <- db[!ind]
 	files <- files[indf]
@@ -414,7 +425,7 @@ function(dir = NULL, files = NULL, encoding = "unknown", db_file = NULL,
  	    built <- built[!some_os]
  	    names_built <- names(built)
  	}
- 	built[!(names_built %in% basenames)] <- NULL
+ 	built[names_built %notin% basenames] <- NULL
  	if (length(built)) {
  	    which <- match(names(built), basenames)
  	    if (all(file_test("-nt", built_file, files[which]))) {
@@ -692,6 +703,8 @@ function(x)
 # textConnection converts to the local encoding, and we convert back,
 # so unrepresentable characters will be lost
 
+## FIXME: use out = tempfile(), like .Rd_get_latex.
+
 .Rd_get_text <-
 function(x) {
     # Handle easy cases first
@@ -768,7 +781,7 @@ function(db)
             stop(sprintf(ngettext(sum(idx),
                                   "missing/empty \\name field in Rd file\n%s",
                                   "missing/empty \\name field in Rd files\n%s"),
-                         paste(" ", Rd_paths[idx], collapse = "\n")),
+                         paste0("  ", Rd_paths[idx], collapse = "\n")),
                  call. = FALSE, domain = NA)
         }
     }
@@ -816,7 +829,7 @@ function(filebase, key = NULL)
             lazyLoadDBfetch(vals[key][[1L]], datafile, compressed, envhook)
 
         if(length(key)) {
-            if(! key %in% vars)
+            if(key %notin% vars)
                 stop(gettextf("No help on %s found in RdDB %s",
                               sQuote(key), sQuote(filebase)),
                      domain = NA)
@@ -867,22 +880,49 @@ loadRdMacros <- function(file, macros = TRUE) {
     attr(Rd, "macros")
 }
 
-
-loadPkgRdMacros <- function(pkgdir, macros) {
-    others <- try(.read_description(file.path(pkgdir, "DESCRIPTION"))["RdMacros"], silent=TRUE)
-    if (inherits(others, "try-error"))
-    	others <- .read_description(file.path(pkgdir, "DESCRIPTION.in"))["RdMacros"]
-
-    if (!is.na(others)) {
-    	others <- trimws(unlist(strsplit(others, ",")))
+initialRdMacros <- function(pkglist = NULL,
+                            macros = file.path(R.home("share"), "Rd", "macros", "system.Rd")
+                            ) {
+    if (length(pkglist)) {
+    	others <- trimws(unlist(strsplit(pkglist, ",")))
 
     	for (p in others) {
-    	    if (dir.exists(system.file("help/macros", package = p)))
+            if((fp <- system.file(package = p)) == "")
+                warning(gettextf("Rd macro package '%s' is not installed.",
+                                 p),
+                        call. = FALSE)
+            else if(dir.exists(file.path(fp, "help", "macros")))
     	    	macros <- loadPkgRdMacros(system.file(package = p), macros)
     	    else
-    	    	warning(gettextf("No Rd macros in package '%s'.", p), call. = FALSE)
+    	    	warning(gettextf("No Rd macros in package '%s'.", p),
+                        call. = FALSE)
         }
-    }
+    } else if (is.character(macros))
+    	macros <- loadRdMacros(file = macros)
+    macros
+}
+
+loadPkgRdMacros <- function(pkgdir, macros = NULL) {
+    ## this does get called on any directory,
+    ## e.g. a man directory in package 'diveMove'.
+    pkglist <- try(.read_description(file.path(pkgdir, "DESCRIPTION")),
+                   silent = TRUE)
+    if (inherits(pkglist, "try-error"))
+    	pkglist <-  try(.read_description(file.path(pkgdir, "DESCRIPTION.in")),
+                        silent = TRUE)
+    ## may check for 'macros' subdirectory?
+    if (inherits(pkglist, "try-error")) return(macros)
+
+    pkglist <- pkglist["RdMacros"]
+
+    if (is.na(pkglist))
+        pkglist <- NULL
+
+    if (is.null(macros))
+        macros <- initialRdMacros(pkglist)
+    else
+        macros <- initialRdMacros(pkglist, macros)
+
     files <- c(list.files(file.path(pkgdir, "man", "macros"), pattern = "\\.Rd$", full.names = TRUE),
                list.files(file.path(pkgdir, "help", "macros"), pattern = "\\.Rd$", full.names = TRUE))
 
