@@ -299,7 +299,7 @@ function(dir, outDir)
     enc <- as.vector(db["Encoding"])
     need_enc <- !is.na(enc) # Encoding was specified
     ## assume that if locale is 'C' we can used 8-bit encodings unchanged.
-    if(need_enc && !(Sys.getlocale("LC_CTYPE") %in% c("C", "POSIX"))) {
+    if(need_enc && (Sys.getlocale("LC_CTYPE") %notin% c("C", "POSIX"))) {
         con <- file(outFile, "a")
         on.exit(close(con))  # Windows does not like files left open
         for(f in codeFiles) {
@@ -875,7 +875,7 @@ function(dir)
         ## of length 3.
         if(length(depends) > 1L) {
             ## .check_package_description will insist on these operators
-            if(!depends$op %in% c("<=", ">=", "<", ">", "==", "!="))
+            if(depends$op %notin% c("<=", ">=", "<", ">", "==", "!="))
                 message("WARNING: malformed 'Depends' field in 'DESCRIPTION'")
             else {
                 status <- if(inherits(depends$version, "numeric_version"))
@@ -933,7 +933,7 @@ checkRdaFiles <- function(paths)
         paths <- Sys.glob(c(file.path(paths, "*.rda"),
                             file.path(paths, "*.RData")))
         ## Exclude .RData, which this may or may not match
-        paths <- grep("/[.]RData$", paths, value = TRUE, invert = TRUE)
+        paths <- filtergrep("/[.]RData$", paths)
     }
     res <- data.frame(size = NA_real_, ASCII = NA,
                       compress = NA_character_, version = NA_integer_,
@@ -965,29 +965,41 @@ checkRdaFiles <- function(paths)
 
 resaveRdaFiles <- function(paths,
                            compress = c("auto", "gzip", "bzip2", "xz"),
-                           compression_level)
+                           compression_level, version = NULL)
 {
     if(length(paths) == 1L && dir.exists(paths))
         paths <- Sys.glob(c(file.path(paths, "*.rda"),
                             file.path(paths, "*.RData")))
     compress <- match.arg(compress)
     if (missing(compression_level))
-        compression_level <- switch(compress, "gzip" = 6, 9)
+        compression_level <- switch(compress, "gzip" = 6L, 9L)
+
+    getVerLoad <- function(file)
+    {
+        con <- gzfile(file, "rb"); on.exit(close(con))
+        ## The .Internal gives an errror on version-1 files
+        tryCatch(.Internal(loadInfoFromConn2(con))$version,
+                 error = function(e) 1L)
+    }
+    if(is.null(version)) version <- 2L # for maximal back-compatibility
+
     for(p in paths) {
+        ver <- max(version, getVerLoad(p))  # to avoid losing features
         env <- new.env(hash = TRUE) # probably small, need not be
         suppressPackageStartupMessages(load(p, envir = env))
         if(compress == "auto") {
             f1 <- tempfile()
-            save(file = f1, list = ls(env, all.names = TRUE), envir = env)
+            save(file = f1, list = ls(env, all.names = TRUE), envir = env,
+                 version = ver)
             f2 <- tempfile()
             save(file = f2, list = ls(env, all.names = TRUE), envir = env,
-                 compress = "bzip2")
+                 compress = "bzip2", version = ver)
             ss <- file.size(c(f1, f2)) * c(0.9, 1.0)
             names(ss) <- c(f1, f2)
             if(ss[1L] > 10240) {
                 f3 <- tempfile()
                 save(file = f3, list = ls(env, all.names = TRUE), envir = env,
-                     compress = "xz")
+                     compress = "xz", version = ver)
                 ss <- c(ss, file.size(f3))
 		names(ss) <- c(f1, f2, f3)
             }
@@ -997,7 +1009,8 @@ resaveRdaFiles <- function(paths,
             unlink(nm)
         } else
             save(file = p, list = ls(env, all.names = TRUE), envir = env,
-                 compress = compress, compression_level = compression_level)
+                 compress = compress, compression_level = compression_level,
+                 version = ver)
     }
 }
 
@@ -1010,6 +1023,21 @@ compactPDF <-
              gs_extras = character())
 {
     use_qpdf <- nzchar(qpdf)
+    qpdf_flags <- "--object-streams=generate"
+    if(use_qpdf) {
+        ## <NOTE>
+        ## Before 2018-09, we passed
+        ##   --stream-data=compress
+        ## to qpdf: but this is now deprecated, corresponds to
+        ## the default since at least qpdf 6.0.0, and it at
+        ## least one case made less compression when given.
+        ## OTOH, people were using versions as old as 2.2.2.
+        ## </NOTE>
+        ver <- system2(qpdf, "--version", TRUE)[1L]
+        ver <- as.numeric_version(sub("qpdf version ", "", ver))
+        if(!is.na(ver) && ver < "6.0.0")
+            qpdf_flags <- c("--stream-data=compress", qpdf_flags)
+    }
     gs_quality <- match.arg(gs_quality, c("none", "printer", "ebook", "screen"))
     use_gs <- if(gs_quality != "none") nzchar(gs_cmd <- find_gs_cmd(gs_cmd)) else FALSE
     if (!use_gs && !use_qpdf) return()
@@ -1031,15 +1059,11 @@ compactPDF <-
             if(!res && use_qpdf) {
                 unlink(tf2) # precaution
                 file.rename(tf, tf2)
-                res <- system2(qpdf, c("--stream-data=compress",
-                                       "--object-streams=generate",
-                                       tf2, tf), FALSE, FALSE)
+                res <- system2(qpdf, c(qpdf_flags, tf2, tf), FALSE, FALSE)
                 unlink(tf2)
             }
         } else if(use_qpdf) {
-            res <- system2(qpdf, c("--stream-data=compress",
-                                   "--object-streams=generate",
-                                   p, tf), FALSE, FALSE)
+            res <- system2(qpdf, c(qpdf_flags, p, tf), FALSE, FALSE)
         }
         if(!res && file.exists(tf)) {
             old <- file.size(p); new <-  file.size(tf)
