@@ -1,7 +1,7 @@
 #  File src/library/tools/R/checktools.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2013-2014 The R Core Team
+#  Copyright (C) 2013-2018 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ function(dir,
     if(os_type == "windows")
         xvfb <- FALSE
     else if(is.logical(xvfb)) {
-        if(!identical(xvfb, TRUE))
+        if(!isTRUE(xvfb))
             xvfb <- FALSE
     } else {
         xvfb_options <- as.character(xvfb)
@@ -110,19 +110,21 @@ function(dir,
     ## Build a package db from the source packages in the working
     ## directory.
     write_PACKAGES(dir, type = "source")
-
+    if(dir.exists(depdir <- file.path(dir, "Depends"))) {
+        write_PACKAGES(depdir, type = "source")
+        curl <- c(curl, paste0(curl, "/Depends"))
+    }
     ## Determine packages available locally (for checking) and in the
     ## repositories, and merge the information giving preference to the
     ## former.
-    curls <- utils::contrib.url(getOption("repos"), type = "source")
     localones <- utils::available.packages(contriburl = curl,
                                            type = "source")
+    curls <- utils::contrib.url(getOption("repos"), type = "source")
     available <- utils::available.packages(contriburl = curls,
                                            type = "source")
-    pos <- match(localones[, "Package"], available[, "Package"])
-    if(length(pos <- pos[!is.na(pos)]))
-        available <- available[-pos, , drop = FALSE]
     available <- rbind(localones, available)
+    available <-
+        available[!duplicated(available[, "Package"]), , drop = FALSE]
     curls <- c(curl, curls)
 
     ## As of c52164, packages with OS_type different from the current
@@ -137,7 +139,7 @@ function(dir,
     pnames_using_install_fake <-
         intersect(pnames_using_install_fake, available[, "Package"])
 
-    if(!is.null(reverse) && !identical(reverse, FALSE)) {
+    if(!is.null(reverse) && !isFALSE(reverse)) {
         ## Determine and download reverse dependencies to be checked as
         ## well.
 
@@ -146,6 +148,7 @@ function(dir,
         defaults <- list(which = c("Depends", "Imports", "LinkingTo"),
                          recursive = FALSE,
                          repos = getOption("repos"))
+        defaults0 <- defaults
         pos <- pmatch(names(reverse), names(defaults), nomatch = 0L)
         defaults[pos] <- reverse[pos > 0L]
 
@@ -172,6 +175,22 @@ function(dir,
                                  reverse = TRUE)
         }
 
+        add_recommended_maybe <-
+            config_val_to_logical(Sys.getenv("_R_TOOLS_C_P_I_D_ADD_RECOMMENDED_MAYBE_",
+                                             "FALSE"))
+        if(add_recommended_maybe) {
+            ## Add all recommended packages with any dependency on the
+            ## packages to be checked.
+            rnames <-
+                c(rnames,
+                  names(Filter(length,
+                               lapply(package_dependencies(.get_standard_package_names()$recommended,
+                                                           available,
+                                                           which = "all"),
+                                      intersect,
+                                      pnames))))
+        }
+
         rnames <- intersect(unlist(rnames, use.names = FALSE),
                             available[, "Package"])
         rnames <- setdiff(rnames, pnames)
@@ -180,11 +199,10 @@ function(dir,
         if(!identical(defaults$repos, getOption("repos"))) {
             pos <- split(pos[pos > 0L], available[pos, "Repository"])
             ## Only want the reverse dependencies for which Repository
-            ## is pmatched by contrib.url(defaults$repos).
+            ## starts with an entry in defaults$repos.
             nms <- names(pos)
-            pos <- unlist(pos[unique(c(outer(defaults$repos, nms,
-                                             pmatch, nomatch = 0L)))],
-                          use.names = FALSE)
+            ind <- (rowSums(outer(nms, defaults$repos, startsWith)) > 0)
+            pos <- unlist(pos[ind], use.names = FALSE)
         }
         rnames <- available[pos, "Package"]
         rfiles <- sprintf("%s_%s.tar.gz",
@@ -199,7 +217,8 @@ function(dir,
             for(i in seq_along(rfiles)) {
                 message(sprintf("downloading %s ... ", rfiles[i]),
                         appendLF = FALSE)
-                status <- if(!utils::download.file(rfurls[i], rfiles[i]))
+                status <- if(!utils::download.file(rfurls[i], rfiles[i],
+                                                   quiet = TRUE))
                     "ok" else "failed"
                 message(status)
             }
@@ -222,8 +241,7 @@ function(dir,
     depends <-
         package_dependencies(pnames, available, which = "most")
     depends <- setdiff(unique(unlist(depends, use.names = FALSE)),
-                       unlist(.get_standard_package_names(),
-                              use.names = FALSE))
+                       .get_standard_package_names()$base)
 
     ## Need to install depends which are not installed or installed but
     ## old.
@@ -236,7 +254,7 @@ function(dir,
                                         available = available)[, "Package"]))
     if(length(depends)) {
         message(paste(strwrap(sprintf("installing dependencies %s",
-                                      paste(sQuote(depends),
+                                      paste(sQuote(sort(depends)),
                                             collapse = ", ")),
                               exdent = 2L),
                       collapse = "\n"), domain = NA)
@@ -305,6 +323,7 @@ function(dir,
                          sprintf("check_%s_stderr.txt", pname))
         env <- c(check_env_db[[pname]],
                  sprintf("R_LIBS=%s", shQuote(libdir)))
+        lim <- get_timeout(Sys.getenv("_R_CHECK_ELAPSED_TIMEOUT_"))
         system.time(system2(file.path(R.home("bin"), "R"),
                             c("CMD",
                               "check",
@@ -313,7 +332,8 @@ function(dir,
                               pfile),
                             stdout = out,
                             stderr = err,
-                            env = env))
+                            env = env,
+                            timeout = lim))
     }
 
     if(Ncpus > 1L) {
@@ -341,7 +361,7 @@ function(dir,
 
     timings <- do.call(rbind, lapply(timings, summary))
     rownames(timings) <- pnames
-    write.table(timings, "timings.tab")
+    utils::write.table(timings, "timings.tab")
 
     file.rename(sprintf("%s.Rcheck", rnames),
                 sprintf("rdepends_%s.Rcheck", rnames))
@@ -415,7 +435,7 @@ function(options)
         options <- c(num, options)
     }
 
-    dis <- Sys.getenv("DISPLAY", unset = NA)
+    dis <- Sys.getenv("DISPLAY", unset = NA_character_)
 
     ## We need to start Xvfb with the given options and obtain its pid
     ## so that we can terminate it when done checking.
@@ -467,7 +487,7 @@ function(dir, all = FALSE, invert = FALSE)
 {
     dir <- normalizePath(dir)
     outdirs <- dir(dir, pattern = "\\.Rcheck")
-    ind <- grepl("^rdepends_", basename(outdirs))
+    ind <- startsWith(basename(outdirs), "rdepends_")
     ## Re-arrange to have reverse dependencies last if at all.
     outdirs <- if(invert)
         c(if(all) outdirs[!ind], outdirs[ind])
@@ -540,17 +560,7 @@ function(dir, all = TRUE, full = FALSE)
             "OK")) {
         writeLines(c("", "Check results details:"))
         details <- check_packages_in_dir_details(logs = logs)
-        flags <- details$Flags
-        out <- cbind(sprintf("Package: %s %s",
-                             details$Package, details$Version),
-                     ifelse(nzchar(flags),
-                            sprintf("Flags: %s\n", flags),
-                            ""),
-                     sprintf("Check: %s, Result: %s",
-                             details$Check, details$Status),
-                     c(gsub("\n", "\n  ", details$Output,
-                            perl = TRUE, useBytes = TRUE)))
-        cat(t(out), sep = c("\n", "", "\n  ", "\n\n"))
+        writeLines(paste(format(details), collapse = "\n\n"))
         invisible(TRUE)
     } else {
         invisible(FALSE)
@@ -582,7 +592,7 @@ function(dir, all = FALSE, full = FALSE)
         tfiles <- Sys.glob(file.path(R_check_outdirs(dir, all = all),
                                      "*-Ex.timings"))
         if(length(tfiles)) message("")
-        timings <- lapply(tfiles, read.table, header = TRUE)
+        timings <- lapply(tfiles, utils::read.table, header = TRUE)
         ## Order by CPU time.
         timings <- lapply(timings,
                           function(x)
@@ -606,6 +616,15 @@ function(dir, all = FALSE, full = FALSE)
 
 ### ** check_packages_in_dir_results
 
+## <FIXME>
+## For new-style logs from successful check runs (a '* DONE' line
+## followed by a 'Status: ' line), we could simply get the status from
+## the 'Status: ' line.
+## Change to preferably rely on the new format eventually.
+## Note that check logs can end up incomplete in which case there is no
+## final status line ...
+## </FIXME>
+
 check_packages_in_dir_results <-
 function(dir, logs = NULL)
 {
@@ -619,20 +638,26 @@ function(dir, logs = NULL)
 
     results <- lapply(logs, function(log) {
         lines <- read_check_log(log)
-        ## Should this be anchored with $ as well?
-        re <- "^\\*.*\\.\\.\\. *(\\[.*\\])? *(NOTE|WARNING|ERROR)"
-        ## Note that we use WARN instead of WARNING for the summary.
+        ## See analyze_lines() inside analyze_check_log():
+        re <- "^\\* (loading checks for arch|checking (examples|tests) \\.\\.\\.$)"
+        pos <- grep(re, lines, perl = TRUE, useBytes = TRUE)
+        if(length(pos <- pos[pos < length(lines)]))
+            lines <- lines[-pos]
+        re <- "^\\*\\*? ((checking|creating|running examples for arch|running tests for arch) .*) \\.\\.\\.( (\\[[^ ]*\\]))?( (NOTE|WARNING|ERROR)|)$"
         m <- regexpr(re, lines, perl = TRUE, useBytes = TRUE)
         ind <- (m > 0L)
-        status <- if(any(ind)) {
-            category <- sub(re, "\\2", lines[ind],
-                            perl = TRUE, useBytes = TRUE)
-            if(any(category == "ERROR")) "ERROR"
-            else if(any(category == "WARNING")) "WARN"
-            else "NOTE"
-        } else {
-            "OK"
-        }
+        ## Note that we use WARN instead of WARNING for the summary.
+        status <-
+            if(any(ind)) {
+                status <- sub(re, "\\6", lines[ind],
+                              perl = TRUE, useBytes = TRUE)
+                if(any(status == "")) "FAIL"
+                else if(any(status == "ERROR")) "ERROR"
+                else if(any(status == "WARNING")) "WARN"
+                else "NOTE"
+            } else {
+                "OK"
+            }
         list(status = status, lines = lines[ind])
     })
     names(results) <- sub("\\.Rcheck$", "", basename(dirname(logs)))
@@ -647,12 +672,12 @@ function(results)
 {
     if(!length(results)) return()
     status <- vapply(results, `[[`, "", "status")
-    ind <- grepl("^rdepends_", names(results))
+    ind <- startsWith(names(results), "rdepends_")
     tab <- table(ifelse(ind, "Reverse depends", "Source packages"),
                  status, deparse.level = 0L)
     tab <- tab[match(c("Source packages", "Reverse depends"),
                      rownames(tab), nomatch = 0L),
-               match(c("ERROR", "WARN", "NOTE", "OK"),
+               match(c("FAIL", "ERROR", "WARN", "NOTE", "OK"),
                      colnames(tab), nomatch = 0L),
                drop = FALSE]
     names(dimnames(tab)) <- NULL
@@ -662,25 +687,46 @@ function(results)
 ### ** read_check_log
 
 read_check_log <-
-function(log)
+function(log, drop = TRUE)
 {
     lines <- readLines(log, warn = FALSE)
 
-    ## Drop CRAN check status footer.
-    ## Ideally, we would have a more general mechanism to detect footer
-    ## information to be skipped (e.g., a line consisting of a single
-    ## non-printing control character?)
-    pos <- grep("^Current CRAN status:", lines,
-                perl = TRUE, useBytes = TRUE)
-    if(length(pos) && lines[pos <- (pos[1L] - 1L)] == "") {
-        lines <- lines[seq_len(pos - 1L)]
+    if(drop) {
+        ## Drop CRAN check status footer.
+        ## Ideally, we would have a more general mechanism to detect
+        ## footer information to be skipped (e.g., a line consisting of
+        ## a single non-printing control character?)
+        pos <- grep("^Current CRAN status:", lines,
+                    perl = TRUE, useBytes = TRUE)
+        if(length(pos) && lines[pos <- (pos[1L] - 1L)] == "") {
+            lines <- lines[seq_len(pos - 1L)]
+        }
     }
 
+    ## <FIXME>
+    ## Remove eventually.
+    len <- length(lines)
+    end <- lines[len]
+    if(length(end) &&
+       grepl(re <- "^(\\*.*\\.\\.\\.)(\\* elapsed time.*)$", end,
+             perl = TRUE, useBytes = TRUE)) {
+        lines <- c(lines[seq_len(len - 1L)],
+                   sub(re, "\\1", end, perl = TRUE, useBytes = TRUE),
+                   sub(re, "\\2", end, perl = TRUE, useBytes = TRUE))
+    }
+    ## </FIXME
+
     lines
 }
 
 ### ** analyze_check_log
 
+## <FIXME>
+## New-style check logs should have a '* DONE' line followed by a
+## 'Status:' line.  If not, a check failure occurred.
+## Change to fully rely on the new format eventually.
+## </FIXME>
+
 analyze_check_log <-
 function(log, drop_ok = TRUE)
 {
@@ -688,12 +734,26 @@ function(log, drop_ok = TRUE)
         list(Package = package, Version = version,
              Flags = flags, Chunks = chunks)
 
+    ## <FIXME>
+    ## All calls to grep(), grepl() and sub() use arguments
+    ## perl = TRUE and useBytes = TRUE.
+    ## Simplify by using inlined functions pgrep() etc, a la
+    psub <- function(pattern, replacement, x)
+        sub(pattern, replacement, x, perl = TRUE, useBytes = TRUE)
+        ## .Internal(sub(pattern, replacement, x, FALSE, TRUE, FALSE, TRUE))
+    ## </FIXME>
+
     ## Alternatives for left and right quotes.
-    lqa <- "'|\xe2\x80\x98"
-    rqa <- "'|\xe2\x80\x99"
+    lqa <- paste0("'|", intToUtf8(0x2018))
+    rqa <- paste0("'|", intToUtf8(0x2019))
     ## Group when used ...
 
-    drop_ok_status_tags <- c("OK", "NONE", "SKIPPED")
+    if(is.character(drop_ok)) {
+        drop_ok_status_tags <- drop_ok
+        drop_ok <- TRUE
+    } else {
+        drop_ok_status_tags <- c("OK", "NONE", "SKIPPED")
+    }
 
     ## Start by reading in.
     lines <- read_check_log(log)
@@ -708,9 +768,23 @@ function(log, drop_ok = TRUE)
         lines <- iconv(lines, enc, "UTF-8", sub = "byte")
         ## If the check log uses ASCII, there should be no non-ASCII
         ## characters in the message lines: could check for this.
-    } else return()
+        if(any(bad <- !validEnc(lines)))
+            lines[bad] <- iconv(lines[bad], to = "ASCII", sub = "byte")
+    } else {
+        ## In case of a fundamental immediate problem which renders
+        ## further checking pointless, we currently do not provide the
+        ## header information with the session charset.  (Perhaps this
+        ## should be changed.)
+        if(!any(grepl("^\\* checking ", lines,
+                      perl = TRUE, useBytes = TRUE)))
+            return()
+    }
+
+    package <- "???"
+    version <- ""
 
     ## Get header.
+    header <- lines
     re <- sprintf("^\\* this is package (%s)(.*)(%s) version (%s)(.*)(%s)$",
                   lqa, rqa, lqa, rqa)
     pos <- grep(re, lines, perl = TRUE, useBytes = TRUE)
@@ -721,52 +795,86 @@ function(log, drop_ok = TRUE)
         version <- sub(re, "\\5", txt, perl = TRUE, useBytes = TRUE)
         header <- lines[seq_len(pos - 1L)]
         lines <- lines[-seq_len(pos)]
-        ## Get check options from header.
-        re <- sprintf("^\\* using options? (%s)(.*)(%s)$", lqa, rqa)
-        flags <- if(length(pos <- grep(re, header,
-                                       perl = TRUE, useBytes = TRUE))) {
+    } else {
+        ## If there was no 'this is package %s version %s' line, then
+        ## either there was a fundamental immediate problem, or an error
+        ## in check_description().  In the latter case there should be a
+        ## line like
+        ##   * checking for file '%s/DESCRIPTION'
+        ## with %s the package name implied by the invocation, but not
+        ## necessarily the one recorded in DESCRIPTION: let's use that
+        ## package name nevertheless, as it is better than nothing.
+        re <- sprintf("^\\* checking for file (%s)(.*)/DESCRIPTION(%s).*$",
+                      lqa, rqa)
+        pos <- grep(re, lines, perl = TRUE, useBytes = TRUE)
+        if(length(pos)) {
+            pos <- pos[1L]
+            txt <- lines[pos]
+            package <- sub(re, "\\2", txt, perl = TRUE, useBytes = TRUE)
+            header <- lines[seq_len(pos - 1L)]
+        } else if(!any(grepl("^\\* checking ", lines,
+                             perl = TRUE, useBytes = TRUE)))
+            return()
+    }
+    ## Get check options from header.
+    re <- sprintf("^\\* using options? (%s)(.*)(%s)$", lqa, rqa)
+    flags <-
+        if(length(pos <- grep(re, header,
+                              perl = TRUE, useBytes = TRUE))) {
             sub(re, "\\2", header[pos[1L]],
                 perl = TRUE, useBytes = TRUE)
         } else ""
-    } else return()
 
     ## Get footer.
     len <- length(lines)
-    ## Some check systems explicitly record the elapsed time in the
-    ## last line:
-    if(grepl("^\\* elapsed time ", lines[len],
-             perl = TRUE, useBytes = TRUE)) {
-        lines <- lines[-len]
-        len <- len - 1L
-    }
-    ## Summary footers.
-    if(grepl("^Status: ", lines[len],
-             perl = TRUE, useBytes = TRUE)) {
-        ## New-style status summary.
-        lines <- lines[-len]
-        len <- len - 1L
-    } else {
-        ## Old-style status summary.
-        num <- length(grep("^(NOTE|WARNING): There",
-                           lines[c(len - 1L, len)]))
-        if(num > 0L) {
-            pos <- seq.int(len - num + 1L, len)
-            lines <- lines[-pos]
-            len <- len - num
+    pos <- which(lines == "* DONE")
+    if(length(pos) &&
+       ((pos <- pos[length(pos)]) < len) &&
+       startsWith(lines[pos + 1L], "Status: "))
+        lines <- lines[seq_len(pos - 1L)]
+    else {
+        ## Not really new style, or failure ... argh.
+        ## Some check systems explicitly record the elapsed time in the
+        ## last line:
+        if(grepl("^\\* elapsed time ", lines[len],
+                 perl = TRUE, useBytes = TRUE)) {
+            lines <- lines[-len]
+            len <- len - 1L
+            while(grepl("^[[:space:]]*$", lines[len])) {
+                lines <- lines[-len]
+                len <- len - 1L
+            }
         }
+        ## Summary footers.
+        if(startsWith(lines[len], "Status: ")) {
+            ## New-style status summary.
+            lines <- lines[-len]
+            len <- len - 1L
+        } else {
+            ## Old-style status summary.
+            num <- length(grep("^(NOTE|WARNING): There",
+                               lines[c(len - 1L, len)]))
+            if(num > 0L) {
+                pos <- seq.int(len - num + 1L, len)
+                lines <- lines[-pos]
+                len <- len - num
+            }
+        }
+        if(lines[len] == "* DONE")
+            lines <- lines[-len]
     }
-    ## New-style end-of-check tag.
-    if(lines[len] == "* DONE")
-        lines <- lines[-len]
 
     analyze_lines <- function(lines) {
         ## Windows has
         ##   * loading checks for arch
         ##   * checking examples ...
         ##   * checking tests ...
-        ## headers: drop these.
+        ## headers: drop these (unless in the last line, where they
+        ## indicate failure).
         re <- "^\\* (loading checks for arch|checking (examples|tests) \\.\\.\\.$)"
-        lines <- lines[!grepl(re, lines, perl = TRUE, useBytes = TRUE)]
+        pos <- grep(re, lines, perl = TRUE, useBytes = TRUE)
+        if(length(pos <- pos[pos < length(lines)]))
+            lines <- lines[-pos]
         ## We might still have
         ##   * package encoding:
         ## entries for packages declaring a package encoding.
@@ -777,7 +885,7 @@ function(log, drop_ok = TRUE)
         ##   ** running examples for arch
         ##   ** running tests for arch
         ## So let's drop everything up to the first such entry.
-        re <- "^\\*\\*? ((checking|creating|running examples for arch|running tests for arch) .*) \\.\\.\\.( (\\[[^ ]*\\]))? (.*)$"
+        re <- "^\\*\\*? ((checking|creating|running examples for arch|running tests for arch) .*) \\.\\.\\.( (\\[[^ ]*\\]))?( (.*)|)$"
         ind <- grepl(re, lines, perl = TRUE, useBytes = TRUE)
         csi <- cumsum(ind)
         ind <- (csi > 0)
@@ -789,23 +897,27 @@ function(log, drop_ok = TRUE)
                        ##   _R_CHECK_VIGNETTE_TIMING_=yes
                        ## will result in a different chunk format ...
                        line <- s[1L]
-                       list(check =
-                            sub(re, "\\1", line, perl = TRUE, useBytes = TRUE),
-                            status =
-                            sub(re, "\\5", line, perl = TRUE, useBytes = TRUE),
+                       check <- sub(re, "\\1", line,
+                                    perl = TRUE, useBytes = TRUE)
+                       status <- sub(re, "\\6", line,
+                                     perl = TRUE, useBytes = TRUE)
+                       if(status == "") status <- "FAIL"
+                       list(check = check,
+                            status = status,
                             output = paste(s[-1L], collapse = "\n"))
                    })
 
         status <- vapply(chunks, `[[`, "", "status")
-        if(identical(drop_ok, TRUE) ||
-           (is.na(drop_ok) && all(status != "ERROR")))
+        if(isTRUE(drop_ok) ||
+           (is.na(drop_ok)
+               && all(is.na(match(c("ERROR", "FAIL"), status)))))
             chunks <- chunks[is.na(match(status, drop_ok_status_tags))]
 
         chunks
     }
 
     chunks <- analyze_lines(lines)
-    if(!length(chunks) && !identical(drop_ok, FALSE)) {
+    if(!length(chunks) && !isFALSE(drop_ok)) {
         chunks <- list(list(check = "*", status = "OK", output = ""))
     }
 
@@ -851,12 +963,12 @@ function(dir, logs = NULL, drop_ok = TRUE)
     ## Now some cleanups.
 
     ## Alternatives for left and right quotes.
-    lqa <- "'|\xe2\x80\x98"
-    rqa <- "'|\xe2\x80\x99"
+    lqa <- paste0("'|", intToUtf8(0x2018))
+    rqa <- paste0("'|", intToUtf8(0x2019))
     ## Group when used ...
 
     mysub <- function(p, r, x) sub(p, r, x, perl = TRUE, useBytes = TRUE)
-    
+
     checks <- db[, "Check"]
     checks <- mysub(sprintf("checking whether package (%s).*(%s) can be installed",
                             lqa, rqa),
@@ -873,24 +985,49 @@ function(dir, logs = NULL, drop_ok = TRUE)
     db[, "Check"] <- checks
     ## In fact, for tabulation purposes it would even be more convenient
     ## to shorten the check names ...
-    
+
     db[, "Output"] <- mysub("[[:space:]]+$", "", db[, "Output"])
-    
+
     db <- as.data.frame(db, stringsAsFactors = FALSE)
     db$Check <- as.factor(db$Check)
     db$Status <- as.factor(db$Status)
 
+    class(db) <- c("check_details", "data.frame")
     db
 }
 
+format.check_details <-
+function(x, ...)
+{
+    flags <- x$Flags
+    flavor <- x$Flavor
+    paste0(sprintf("Package: %s %s\n",
+                   x$Package, x$Version),
+           ifelse(nzchar(flavor),
+                  sprintf("Flavor: %s\n", flavor),
+                  ""),
+           ifelse(nzchar(flags),
+                  sprintf("Flags: %s\n", flags),
+                  ""),
+           sprintf("Check: %s, Result: %s\n",
+                   x$Check, x$Status),
+           sprintf("  %s",
+                   gsub("\n", "\n  ", x$Output,
+                        perl = TRUE, useBytes = TRUE)))
+}
+
+print.check_details <-
+function(x, ...)
+{
+    writeLines(paste(format(x, ...), collapse = "\n\n"))
+    invisible(x)
+}
+
 ### ** check_packages_in_dir_changes
 
 check_packages_in_dir_changes <-
 function(dir, old, outputs = FALSE, sources = FALSE)
 {
-    check_packages_in_dir_changes_classes <-
-        c("check_packages_in_dir_changes", "data.frame")
-
     dir <- if(inherits(dir, "check_packages_in_dir"))
         dir <- attr(dir, "dir")
     else
@@ -908,6 +1045,20 @@ function(dir, old, outputs = FALSE, sources = FALSE)
     if(!inherits(old, "check_details"))
         old <- check_packages_in_dir_details(old, drop_ok = FALSE)
 
+    check_details_changes(new, old, outputs)
+}
+
+### ** check_details_changes
+
+check_details_changes <-
+function(new, old, outputs = FALSE)
+{
+    check_details_changes_classes <-
+        c("check_details_changes", "data.frame")
+
+    if(!inherits(new, "check_details")) stop("wrong class")
+    if(!inherits(old, "check_details")) stop("wrong class")
+
     ## Simplify matters by considering only "changes" in *available*
     ## results/details.
 
@@ -919,7 +1070,7 @@ function(dir, old, outputs = FALSE, sources = FALSE)
                          Old = character(),
                          New = character(),
                          stringsAsFactors = FALSE)
-        class(db) <- check_packages_in_dir_changes_classes
+        class(db) <- check_details_changes_classes
         return(db)
     }
 
@@ -941,12 +1092,14 @@ function(dir, old, outputs = FALSE, sources = FALSE)
                    e
                })
     db <- do.call(rbind, chunks)
-    
+
     ## Drop checks that are OK in both versions
-    x.issue <- !is.na(match(db$Status.x, c("NOTE","ERROR","WARNING")))
-    y.issue <- !is.na(match(db$Status.y, c("NOTE","ERROR","WARNING")))
+    x.issue <- !is.na(match(db$Status.x,
+                            c("ERROR","FAIL","NOTE","WARNING")))
+    y.issue <- !is.na(match(db$Status.y,
+                            c("ERROR","FAIL","NOTE","WARNING")))
     db <- db[x.issue | y.issue,]
-    
+
     ## Even with the above simplification, missing entries do not
     ## necessarily indicate "OK" (checks could have been skipped).
     ## Hence leave as missing and show as empty in the diff.
@@ -956,10 +1109,12 @@ function(dir, old, outputs = FALSE, sources = FALSE)
     sx <- as.character(db$Status.x)
     sy <- as.character(db$Status.y)
     if(outputs) {
-        sx <- sprintf("%s\n  %s", sx,
-                      gsub("\n", "\n  ", db$Output.x, fixed = TRUE))
-        sy <- sprintf("%s\n  %s", sy,
-                      gsub("\n", "\n  ", db$Output.y, fixed = TRUE))
+        ind <- nzchar(ox <- db$Output.x)
+        sx[ind] <- sprintf("%s\n  %s", sx[ind],
+                           gsub("\n", "\n  ", ox[ind], fixed = TRUE))
+        ind <- nzchar(oy <- db$Output.y)
+        sy[ind] <- sprintf("%s\n  %s", sy[ind],
+                           gsub("\n", "\n  ", oy[ind], fixed = TRUE))
     }
     sx[is.na(db$Status.x)] <- ""
     sy[is.na(db$Status.y)] <- ""
@@ -982,12 +1137,31 @@ function(dir, old, outputs = FALSE, sources = FALSE)
 
     db <- db[c("Package", "Check", "Old", "New")]
 
-    class(db) <- check_packages_in_dir_changes_classes
+    class(db) <- check_details_changes_classes
 
     db
 }
 
-format.check_packages_in_dir_changes <-
+`[.check_details_changes` <-
+function(x, i, j, drop = FALSE)
+{
+    if(((na <- nargs() - (!missing(drop))) == 3L)
+       && (length(i) == 1L)
+       && any(!is.na(match(i, c("==", "!=", "<", "<=", ">", ">="))))) {
+        levels <- c("", "OK", "NOTE", "WARNING", "ERROR", "FAIL")
+        encode <- function(s) {
+            s <- sub("\n.*", "", s)
+            s[is.na(match(s, levels))] <- ""
+            ordered(s, levels)
+        }
+        old <- encode(x$Old)
+        new <- encode(x$New)
+        i <- do.call(i, list(old, new))
+    }
+    NextMethod()
+}
+
+format.check_details_changes <-
 function(x, ...)
 {
     if(!nrow(x)) return(character())
@@ -1002,7 +1176,7 @@ function(x, ...)
                    ""))
 }
 
-print.check_packages_in_dir_changes <-
+print.check_details_changes <-
 function(x, ...)
 {
     if(length(y <- format(x)))
