@@ -1,7 +1,7 @@
 #  File src/library/tools/R/xgettext.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ function(dir, verbose = FALSE, asCall = TRUE)
              suppress <- !is.null(domain) && !is.name(domain) && is.na(domain)
              ## remove named args
              if(!is.null(names(e)))
-                 e <- e[!names(e) %in% c("call.", "immediate.", "domain")]
+                 e <- e[names(e) %notin% c("call.", "immediate.", "domain")]
              if(asCall) {
                  if(!suppress) strings <<- c(strings, as.character(e)[-1L])
              } else {
@@ -93,20 +93,19 @@ function(dir, verbose = FALSE, asCall = TRUE)
     out[lengths(out) > 0L]
 }
 
-print.xgettext <-
-function(x, ...)
+print.xgettext <- function(x, ...)
 {
-    cat(x, sep = "\n")
+    cat(encodeString(x), sep = "\n")
     invisible(x)
 }
 
-print.xngettext <-
-function(x, ...)
+print.xngettext <- function(x, ...)
 {
-    lapply(x, function(x)
-           cat("\nmsgid        = ", x[1L],
-               "\nmsgid_plural = ", x[2L],
-               "\n", sep = ""))
+    lapply(x, function(x) {
+        e <- encodeString(x)
+        cat("\nmsgid        = ", e[1L],
+            "\nmsgid_plural = ", e[2L], "\n", sep = "")
+    })
     invisible(x)
 }
 
@@ -119,7 +118,7 @@ function(dir, verbose = FALSE)
     R_files <- list_files_with_exts(dir, exts)
     for(d in c("unix", "windows", "aqua")) {
         OSdir <- file.path(dir, d)
-        if(dir.exists(OSdir))
+       if(dir.exists(OSdir))
             R_files <- c(R_files, list_files_with_exts(OSdir, exts))
     }
     out <- vector("list", length = length(R_files))
@@ -212,17 +211,17 @@ checkPoFile <- function(f, strictPlural = FALSE)
     while (i < length(lines)) {
 	i <- i + 1L
 
-	if (grepl("^#,", lines[i], useBytes = TRUE)) {
+	if (startsWith(lines[i], "#,")) { # useBytes=TRUE (speedup ?)
 	    noCformat <- noCformat || grepl("no-c-format", lines[i], useBytes = TRUE)
 	    fuzzy <- fuzzy || grepl("fuzzy", lines[i], useBytes = TRUE)
-	} else if (grepl("^#:", lines[i], useBytes = TRUE)) {
+	} else if (startsWith(lines[i], "#:")) {
 	    if (!is.na(ref))
 		ref <- paste(ref, "etc.")
 	    else
 		ref <- sub("^#:[[:blank:]]*", "", lines[i])
-	} else if (grepl("^msgid ", lines[i], useBytes = TRUE)) {
+	} else if (startsWith(lines[i], "msgid ")) {
 	    s1 <- sub('^msgid[[:blank:]]+["](.*)["][[:blank:]]*$', "\\1", lines[i])
-	    while (grepl('^["]', lines[i+1L], useBytes = TRUE)) {
+	    while (startsWith(lines[i+1L], '"')) {
 		i <- i + 1L
 		s1 <- paste0(s1, sub('^["](.*)["][[:blank:]]*$', "\\1", lines[i]))
 	    }
@@ -246,7 +245,7 @@ checkPoFile <- function(f, strictPlural = FALSE)
 
 		s2 <- sub( paste0("^", statement, "[[:blank:]]+[\"](.*)[\"][[:blank:]]*$"),
 		                 "\\1", lines[j])
-		while (grepl('^["]', lines[j+1L], useBytes = TRUE)) {
+		while (!is.na(lines[j+1L]) && startsWith(lines[j+1L], '"')) {
 		    j <- j+1L
 		    s2 <- paste0(s2, sub('^["](.*)["][[:blank:]]*$', "\\1", lines[j]))
 		}
@@ -320,7 +319,7 @@ checkPoFiles <- function(language, dir=".")
                         full.names = TRUE, recursive = TRUE)
     result <- matrix(character(), ncol = 5L, nrow = 0L)
     for (f in files) {
-	errs <- checkPoFile(f, strictPlural = grepl("^R-", basename(f)))
+	errs <- checkPoFile(f, strictPlural = startsWith(basename(f), "R-"))
 	if (nrow(errs)) result <- rbind(result, errs)
     }
     structure(result, class = "check_po_files")
@@ -332,8 +331,7 @@ print.check_po_files <- function(x, ...)
 	cat("No errors\n")
     else
 	for (i in 1:nrow(x)) {
-	    if (is.na(x[i, 2L])) cols <- c(1L, 3:5)
-	    else cols <- 1:5
+	    cols <- if(is.na(x[i, 2L])) c(1L, 3:5) else 1:5
 	    cat(x[i, cols], sep = "\n")
 	    cat("\n")
 	}
