@@ -1,7 +1,7 @@
 #  File src/library/utils/R/windows/install.packages.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2017 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -170,8 +170,8 @@ unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
             ret <- unlink(instPath, recursive=TRUE, force=TRUE)
             if (ret == 0) {
                 ## Move the new package to the install lib
-                ## To avoid anti-virus interference, wait a little
-                Sys.sleep(0.5)
+                ## file.rename automatically retries few times if necessary
+		## due to anti-virus interference
                 ret <- file.rename(file.path(tmpDir, pkgname), instPath)
                 if(!ret) {
                     warning(gettextf("unable to move temporary installation %s to %s",
@@ -244,7 +244,7 @@ unpackPkgZip <- function(pkg, pkgname, lib, libs_only = FALSE,
 
     if(is.null(available))
         available <- available.packages(contriburl = contriburl,
-                                        method = method)
+                                        method = method, ...)
     pkgs <- getDependencies(pkgs, dependencies, available, lib, binary = TRUE)
 
     foundpkgs <- download.packages(pkgs, destdir = tmpd, available = available,
@@ -285,7 +285,7 @@ menuInstallLocal <- function()
     zips <- grepl("[.]zip$", files)
     tarballs <- grepl("[.]tar[.]gz$", files)
     bad <- !(zips | tarballs)
-    if (any(bad)) 
+    if (any(bad))
         stop("Only '*.zip' and '*.tar.gz' files can be installed.")
     if (any(zips)) install.packages(files[zips],
         .libPaths()[1L], repos = NULL, type = "binary")
