@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995--1997 Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998--2016 The R Core Team.
+ *  Copyright (C) 1998--2018 The R Core Team.
  *  Copyright (C) 2003--2016 The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -215,9 +215,10 @@ double R_pow(double x, double y) /* = x ^ y */
 	   gcc 4.3.0 -g -O2 mis-compiled it.  Showed up with
 	   100^0.5 as 3.162278, example(pbirthday) failed. */
 #ifdef USE_POWL_IN_R_POW
-    return powl(x, y);
+	// this is used only on 64-bit Windows (so has powl).
+	return powl(x, y);
 #else
-    return pow(x, y);
+	return pow(x, y);
 #endif
     }
     if (ISNAN(x) || ISNAN(y))
@@ -405,66 +406,63 @@ SEXP attribute_hidden do_arith(SEXP call, SEXP op, SEXP args, SEXP env)
     else if (argc == 2) {
 	/* Handle some scaler operations immediately */
 	if (IS_SCALAR(arg1, REALSXP)) {
+	    double x1 = SCALAR_DVAL(arg1);
 	    if (IS_SCALAR(arg2, REALSXP)) {
-		double x1 = REAL(arg1)[0];
-		double x2 = REAL(arg2)[0];
+		double x2 = SCALAR_DVAL(arg2);
 		ans = ScalarValue2(arg1, arg2);
 		switch (PRIMVAL(op)) {
-		case PLUSOP: REAL(ans)[0] = x1 + x2; return ans;
-		case MINUSOP: REAL(ans)[0] = x1 - x2; return ans;
-		case TIMESOP: REAL(ans)[0] = x1 * x2; return ans;
-		case DIVOP: REAL(ans)[0] = x1 / x2; return ans;
+		case PLUSOP: SET_SCALAR_DVAL(ans, x1 + x2); return ans;
+		case MINUSOP: SET_SCALAR_DVAL(ans, x1 - x2); return ans;
+		case TIMESOP: SET_SCALAR_DVAL(ans, x1 * x2); return ans;
+		case DIVOP: SET_SCALAR_DVAL(ans, x1 / x2); return ans;
 		}
 	    }
 	    else if (IS_SCALAR(arg2, INTSXP)) {
-		double x1 = REAL(arg1)[0];
-		double x2 = INTEGER(arg2)[0] != NA_INTEGER ?
-		    (double) INTEGER(arg2)[0] : NA_REAL;
+		int i2 = SCALAR_IVAL(arg2);
+		double x2 = i2 != NA_INTEGER ? (double) i2 : NA_REAL;
 		ans = ScalarValue1(arg1);
 		switch (PRIMVAL(op)) {
-		case PLUSOP: REAL(ans)[0] = x1 + x2; return ans;
-		case MINUSOP: REAL(ans)[0] = x1 - x2; return ans;
-		case TIMESOP: REAL(ans)[0] = x1 * x2; return ans;
-		case DIVOP: REAL(ans)[0] = x1 / x2; return ans;
+		case PLUSOP: SET_SCALAR_DVAL(ans, x1 + x2); return ans;
+		case MINUSOP: SET_SCALAR_DVAL(ans, x1 - x2); return ans;
+		case TIMESOP: SET_SCALAR_DVAL(ans, x1 * x2); return ans;
+		case DIVOP: SET_SCALAR_DVAL(ans, x1 / x2); return ans;
 		}
 	    }
 	}
 	else if (IS_SCALAR(arg1, INTSXP)) {
+	    int i1 = SCALAR_IVAL(arg1);
 	    if (IS_SCALAR(arg2, REALSXP)) {
-		double x1 = INTEGER(arg1)[0] != NA_INTEGER ?
-		    (double) INTEGER(arg1)[0] : NA_REAL;
-		double x2 = REAL(arg2)[0];
+		double x1 = i1 != NA_INTEGER ? (double) i1 : NA_REAL;
+		double x2 = SCALAR_DVAL(arg2);
 		ans = ScalarValue1(arg2);
 		switch (PRIMVAL(op)) {
-		case PLUSOP: REAL(ans)[0] = x1 + x2; return ans;
-		case MINUSOP: REAL(ans)[0] = x1 - x2; return ans;
-		case TIMESOP: REAL(ans)[0] = x1 * x2; return ans;
-		case DIVOP: REAL(ans)[0] = x1 / x2; return ans;
+		case PLUSOP: SET_SCALAR_DVAL(ans, x1 + x2); return ans;
+		case MINUSOP: SET_SCALAR_DVAL(ans, x1 - x2); return ans;
+		case TIMESOP: SET_SCALAR_DVAL(ans, x1 * x2); return ans;
+		case DIVOP: SET_SCALAR_DVAL(ans, x1 / x2); return ans;
 		}
 	    }
 	    else if (IS_SCALAR(arg2, INTSXP)) {
 		Rboolean naflag = FALSE;
-		int x1 = INTEGER(arg1)[0];
-		int x2 = INTEGER(arg2)[0];
+		int i2 = SCALAR_IVAL(arg2);
 		switch (PRIMVAL(op)) {
 		case PLUSOP:
 		    ans = ScalarValue2(arg1, arg2);
-		    INTEGER(ans)[0] = R_integer_plus(x1, x2, &naflag);
+		    SET_SCALAR_IVAL(ans, R_integer_plus(i1, i2, &naflag));
 		    CHECK_INTEGER_OVERFLOW(call, ans, naflag);
 		    return ans;
 		case MINUSOP:
 		    ans = ScalarValue2(arg1, arg2);
-		    INTEGER(ans)[0] = R_integer_minus(x1, x2, &naflag);
+		    SET_SCALAR_IVAL(ans, R_integer_minus(i1, i2, &naflag));
 		    CHECK_INTEGER_OVERFLOW(call, ans, naflag);
 		    return ans;
 		case TIMESOP:
 		    ans = ScalarValue2(arg1, arg2);
-		    INTEGER(ans)[0] = R_integer_times(x1, x2, &naflag);
+		    SET_SCALAR_IVAL(ans, R_integer_times(i1, i2, &naflag));
 		    CHECK_INTEGER_OVERFLOW(call, ans, naflag);
 		    return ans;
 		case DIVOP:
-		    ans = ScalarReal(R_integer_divide(x1, x2));
-		    return ans;
+		    return ScalarReal(R_integer_divide(i1, i2));
 		}
 	    }
 	}
@@ -475,17 +473,18 @@ SEXP attribute_hidden do_arith(SEXP call, SEXP op, SEXP args, SEXP env)
 	    case PLUSOP: return(arg1);
 	    case MINUSOP:
 		ans = ScalarValue1(arg1);
-		REAL(ans)[0] = -REAL(arg1)[0];
+		SET_SCALAR_DVAL(ans, -SCALAR_DVAL(arg1));
 		return ans;
 	    }
 	}
 	else if (IS_SCALAR(arg1, INTSXP)) {
+	    int ival;
 	    switch(PRIMVAL(op)) {
 	    case PLUSOP: return(arg1);
 	    case MINUSOP:
+		ival = SCALAR_IVAL(arg1);
 		ans = ScalarValue1(arg1);
-		INTEGER(ans)[0] = INTEGER(arg1)[0] == NA_INTEGER ?
-		    NA_INTEGER : -INTEGER(arg1)[0];
+		SET_SCALAR_IVAL(ans, ival == NA_INTEGER ? NA_INTEGER : -ival);
 		return ans;
 	    }
 	}
@@ -627,8 +626,7 @@ SEXP attribute_hidden R_binary(SEXP call, SEXP op, SEXP x, SEXP y)
     SEXP klass = NULL, tsp = NULL; // -Wall
     if (xts || yts) {
 	if (xts && yts) {
-	    if (!tsConform(x, y))
-		errorcall(call, _("non-conformable time-series"));
+	    /* could check ts conformance here */
 	    PROTECT(tsp = getAttrib(x, R_TspSymbol));
 	    PROTECT(klass = getAttrib(x, R_ClassSymbol));
 	}
@@ -733,14 +731,17 @@ static SEXP logical_unary(ARITHOP_TYPE code, SEXP s1, SEXP call)
     if(dimnames != R_NilValue) setAttrib(ans, R_DimNamesSymbol, dimnames);
     UNPROTECT(3);
 
+    int *pa = INTEGER(ans);
+    const int *px = LOGICAL_RO(s1);
+
     switch (code) {
     case PLUSOP:
-	for (R_xlen_t  i = 0; i < n; i++) INTEGER(ans)[i] = LOGICAL(s1)[i];
+	for (R_xlen_t  i = 0; i < n; i++) pa[i] = px[i];
 	break;
     case MINUSOP:
 	for (R_xlen_t  i = 0; i < n; i++) {
-	    int x = LOGICAL(s1)[i];
-	    INTEGER(ans)[i] = (x == NA_INTEGER) ?
+	    int x = px[i];
+	    pa[i] = (x == NA_INTEGER) ?
 		NA_INTEGER : ((x == 0.0) ? 0 : -x);
 	}
 	break;
@@ -754,7 +755,6 @@ static SEXP logical_unary(ARITHOP_TYPE code, SEXP s1, SEXP call)
 static SEXP integer_unary(ARITHOP_TYPE code, SEXP s1, SEXP call)
 {
     R_xlen_t i, n;
-    int x;
     SEXP ans;
 
     switch (code) {
@@ -762,10 +762,12 @@ static SEXP integer_unary(ARITHOP_TYPE code, SEXP s1, SEXP call)
 	return s1;
     case MINUSOP:
 	ans = NO_REFERENCES(s1) ? s1 : duplicate(s1);
+	int *pa = INTEGER(ans);
+	const int *px = INTEGER_RO(s1);
 	n = XLENGTH(s1);
 	for (i = 0; i < n; i++) {
-	    x = INTEGER(s1)[i];
-	    INTEGER(ans)[i] = (x == NA_INTEGER) ?
+	    int x = px[i];
+	    pa[i] = (x == NA_INTEGER) ?
 		NA_INTEGER : ((x == 0.0) ? 0 : -x);
 	}
 	return ans;
@@ -784,9 +786,11 @@ static SEXP real_unary(ARITHOP_TYPE code, SEXP s1, SEXP lcall)
     case PLUSOP: return s1;
     case MINUSOP:
 	ans = NO_REFERENCES(s1) ? s1 : duplicate(s1);
+	double *pa = REAL(ans);
+	const double *px = REAL_RO(s1);
 	n = XLENGTH(s1);
 	for (i = 0; i < n; i++)
-	    REAL(ans)[i] = -REAL(s1)[i];
+	    pa[i] = -px[i];
 	return ans;
     default:
 	errorcall(lcall, _("invalid unary operator"));
@@ -815,73 +819,108 @@ static SEXP integer_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2, SEXP lcall)
 
     switch (code) {
     case PLUSOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		x1 = INTEGER(s1)[i1];
-		x2 = INTEGER(s2)[i2];
-		INTEGER(ans)[i] = R_integer_plus(x1, x2, &naflag);
-	    });
-	if (naflag)
-	    warningcall(lcall, INTEGER_OVERFLOW_WARNING);
+	{
+	    int *pa = INTEGER(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    x1 = px1[i1];
+		    x2 = px2[i2];
+		    pa[i] = R_integer_plus(x1, x2, &naflag);
+		});
+	    if (naflag)
+		warningcall(lcall, INTEGER_OVERFLOW_WARNING);
+	}
 	break;
     case MINUSOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		x1 = INTEGER(s1)[i1];
-		x2 = INTEGER(s2)[i2];
-		INTEGER(ans)[i] = R_integer_minus(x1, x2, &naflag);
-	    });
-	if (naflag)
-	    warningcall(lcall, INTEGER_OVERFLOW_WARNING);
+	{
+	    int *pa = INTEGER(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    x1 = px1[i1];
+		    x2 = px2[i2];
+		    pa[i] = R_integer_minus(x1, x2, &naflag);
+		});
+	    if (naflag)
+		warningcall(lcall, INTEGER_OVERFLOW_WARNING);
+	}
 	break;
     case TIMESOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		x1 = INTEGER(s1)[i1];
-		x2 = INTEGER(s2)[i2];
-		INTEGER(ans)[i] = R_integer_times(x1, x2, &naflag);
-	    });
-	if (naflag)
-	    warningcall(lcall, INTEGER_OVERFLOW_WARNING);
+	{
+	    int *pa = INTEGER(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    x1 = px1[i1];
+		    x2 = px2[i2];
+		    pa[i] = R_integer_times(x1, x2, &naflag);
+		});
+	    if (naflag)
+		warningcall(lcall, INTEGER_OVERFLOW_WARNING);
+	}
 	break;
     case DIVOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		x1 = INTEGER(s1)[i1];
-		x2 = INTEGER(s2)[i2];
-		REAL(ans)[i] = R_integer_divide(x1, x2);
-	    });
+	{
+	    double *pa = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    x1 = px1[i1];
+		    x2 = px2[i2];
+		    pa[i] = R_integer_divide(x1, x2);
+		});
+	}
 	break;
     case POWOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		if((x1 = INTEGER(s1)[i1]) == 1 || (x2 = INTEGER(s2)[i2]) == 0)
-		    REAL(ans)[i] = 1.;
-		else if (x1 == NA_INTEGER || x2 == NA_INTEGER)
-		    REAL(ans)[i] = NA_REAL;
-		else
-		    REAL(ans)[i] = R_POW((double) x1, (double) x2);
-	    });
+	{
+	    double *pa = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    if((x1 = px1[i1]) == 1 || (x2 = px2[i2]) == 0)
+			pa[i] = 1.;
+		    else if (x1 == NA_INTEGER || x2 == NA_INTEGER)
+			pa[i] = NA_REAL;
+		    else
+			pa[i] = R_POW((double) x1, (double) x2);
+		});
+	}
 	break;
     case MODOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		x1 = INTEGER(s1)[i1];
-		x2 = INTEGER(s2)[i2];
-		if (x1 == NA_INTEGER || x2 == NA_INTEGER || x2 == 0)
-		    INTEGER(ans)[i] = NA_INTEGER;
-		else {
-		    INTEGER(ans)[i] = /* till 0.63.2:	x1 % x2 */
-			(x1 >= 0 && x2 > 0) ? x1 % x2 :
-			(int)myfmod((double)x1,(double)x2);
-		}
-	    });
+	{
+	    int *pa = INTEGER(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    x1 = px1[i1];
+		    x2 = px2[i2];
+		    if (x1 == NA_INTEGER || x2 == NA_INTEGER || x2 == 0)
+			pa[i] = NA_INTEGER;
+		    else {
+			pa[i] = /* till 0.63.2:	x1 % x2 */
+			    (x1 >= 0 && x2 > 0) ? x1 % x2 :
+			    (int)myfmod((double)x1,(double)x2);
+		    }
+		});
+	}
 	break;
     case IDIVOP:
-	MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
-		x1 = INTEGER(s1)[i1];
-		x2 = INTEGER(s2)[i2];
-		/* This had x %/% 0 == 0 prior to 2.14.1, but
-		   it seems conventionally to be undefined */
-		if (x1 == NA_INTEGER || x2 == NA_INTEGER || x2 == 0)
-		    INTEGER(ans)[i] = NA_INTEGER;
-		else
-		    INTEGER(ans)[i] = (int) floor((double)x1 / (double)x2);
-	    });
+	{
+	    int *pa = INTEGER(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
+	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2, {
+		    x1 = px1[i1];
+		    x2 = px2[i2];
+		    /* This had x %/% 0 == 0 prior to 2.14.1, but
+		       it seems conventionally to be undefined */
+		    if (x1 == NA_INTEGER || x2 == NA_INTEGER || x2 == 0)
+			pa[i] = NA_INTEGER;
+		    else
+			pa[i] = (int) floor((double)x1 / (double)x2);
+		});
+	}
 	break;
     }
     UNPROTECT(1);
@@ -900,7 +939,7 @@ static SEXP integer_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2, SEXP lcall)
     return ans;
 }
 
-#define R_INTEGER(robj, i) (double) (INTEGER(robj)[i] == NA_INTEGER ? NA_REAL : INTEGER(robj)[i])
+#define R_INTEGER(x) (double) ((x) == NA_INTEGER ? NA_REAL : (x))
 
 static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 {
@@ -921,8 +960,8 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
     case PLUSOP:
 	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
 	    double *da = REAL(ans);
-	    double *dx = REAL(s1);
-	    double *dy = REAL(s2);
+	    const double *dx = REAL_RO(s1);
+	    const double *dy = REAL_RO(s2);
 	    if (n2 == 1) {
 		double tmp = dy[0];
 		R_ITERATE_CHECK(NINTERRUPT, n, i, da[i] = dx[i] + tmp;);
@@ -937,18 +976,26 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 		MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
 				  da[i] = dx[i1] + dy[i2];);
 	}
-	else if(TYPEOF(s1) == INTSXP )
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = R_INTEGER(s1, i1) + REAL(s2)[i2];);
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = R_INTEGER(px1[i1]) + px2[i2];);
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = REAL(s1)[i1] + R_INTEGER(s2, i2););
+			       da[i] = px1[i1] + R_INTEGER(px2[i2]););
+	}
 	break;
     case MINUSOP:
 	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
 	    double *da = REAL(ans);
-	    double *dx = REAL(s1);
-	    double *dy = REAL(s2);
+	    const double *dx = REAL_RO(s1);
+	    const double *dy = REAL_RO(s2);
 	    if (n2 == 1) {
 		double tmp = dy[0];
 		R_ITERATE_CHECK(NINTERRUPT, n, i, da[i] = dx[i] - tmp;);
@@ -963,24 +1010,32 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 		MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
 				  da[i] = dx[i1] - dy[i2];);
 	}
-	else if(TYPEOF(s1) == INTSXP )
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = R_INTEGER(s1, i1) - REAL(s2)[i2];);
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = R_INTEGER(px1[i1]) - px2[i2];);
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = REAL(s1)[i1] - R_INTEGER(s2, i2););
+			       da[i] = px1[i1] - R_INTEGER(px2[i2]););
+	}
 	break;
     case TIMESOP:
 	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
 	    double *da = REAL(ans);
-	    double *dx = REAL(s1);
-	    double *dy = REAL(s2);
+	    const double *dx = REAL_RO(s1);
+	    const double *dy = REAL_RO(s2);
 	    if (n2 == 1) {
 		double tmp = dy[0];
 		R_ITERATE_CHECK(NINTERRUPT, n, i, da[i] = dx[i] * tmp;);
 	    }
 	    else if (n1 == 1) {
-		double tmp = REAL(s1)[0];
+		double tmp = dx[0];
 		R_ITERATE_CHECK(NINTERRUPT, n, i, da[i] = tmp * dy[i];);
 	    }
 	    else if (n1 == n2)
@@ -989,18 +1044,26 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 		MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
 				  da[i] = dx[i1] * dy[i2];);
 	}
-	else if(TYPEOF(s1) == INTSXP )
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = R_INTEGER(s1, i1) * REAL(s2)[i2];);
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = R_INTEGER(px1[i1]) * px2[i2];);
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = REAL(s1)[i1] * R_INTEGER(s2, i2););
+			       da[i] = px1[i1] * R_INTEGER(px2[i2]););
+	}
 	break;
     case DIVOP:
 	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
 	    double *da = REAL(ans);
-	    double *dx = REAL(s1);
-	    double *dy = REAL(s2);
+	    const double *dx = REAL_RO(s1);
+	    const double *dy = REAL_RO(s2);
 	    if (n2 == 1) {
 		double tmp = dy[0];
 		R_ITERATE_CHECK(NINTERRUPT, n, i, da[i] = dx[i] / tmp;);
@@ -1015,18 +1078,26 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 		MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
 				  da[i] = dx[i1] / dy[i2];);
 	}
-	else if(TYPEOF(s1) == INTSXP )
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = R_INTEGER(s1, i1) / REAL(s2)[i2];);
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = R_INTEGER(px1[i1]) / px2[i2];);
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = REAL(s1)[i1] / R_INTEGER(s2, i2););
+			       da[i] = px1[i1] / R_INTEGER(px2[i2]););
+	}
 	break;
     case POWOP:
 	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
 	    double *da = REAL(ans);
-	    double *dx = REAL(s1);
-	    double *dy = REAL(s2);
+	    const double *dx = REAL_RO(s1);
+	    const double *dy = REAL_RO(s2);
 	    if (n2 == 1) {
 		double tmp = dy[0];
 		R_ITERATE_CHECK(NINTERRUPT, n, i, da[i] = R_POW(dx[i], tmp););
@@ -1041,42 +1112,66 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 		MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
 				  da[i] = R_POW(dx[i1], dy[i2]););
 	}
-	else if(TYPEOF(s1) == INTSXP )
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = R_POW( R_INTEGER(s1, i1),
-						    REAL(s2)[i2]););
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = R_POW( R_INTEGER(px1[i1]), px2[i2]););
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = R_POW(REAL(s1)[i1],
-						   R_INTEGER(s2, i2)););
+			       da[i] = R_POW(px1[i1], R_INTEGER(px2[i2])););
+	}
 	break;
     case MODOP:
-	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP)
+	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = myfmod(REAL(s1)[i1],
-						    REAL(s2)[i2]););
-	else if(TYPEOF(s1) == INTSXP )
+			       da[i] = myfmod(px1[i1], px2[i2]););
+	}
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = myfmod( R_INTEGER(s1, i1),
-						     REAL(s2)[i2]););
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = myfmod(R_INTEGER(px1[i1]), px2[i2]););
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = myfmod(REAL(s1)[i1],
-						    R_INTEGER(s2, i2)););
+			       da[i] = myfmod(px1[i1], R_INTEGER(px2[i2])););
+	}
 	break;
     case IDIVOP:
-	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP)
+	if(TYPEOF(s1) == REALSXP && TYPEOF(s2) == REALSXP) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = myfloor(REAL(s1)[i1],
-						     REAL(s2)[i2]););
-	else if(TYPEOF(s1) == INTSXP )
+			       da[i] = myfloor(px1[i1], px2[i2]););
+	}
+	else if(TYPEOF(s1) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const int *px1 = INTEGER_RO(s1);
+	    const double *px2 = REAL_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = myfloor(R_INTEGER(s1, i1),
-						     REAL(s2)[i2]););
-	else if(TYPEOF(s2) == INTSXP )
+			       da[i] = myfloor(R_INTEGER(px1[i1]), px2[i2]););
+	}
+	else if(TYPEOF(s2) == INTSXP ) {
+	    double *da = REAL(ans);
+	    const double *px1 = REAL_RO(s1);
+	    const int *px2 = INTEGER_RO(s2);
 	    MOD_ITERATE2_CHECK(NINTERRUPT, n, n1, n2, i, i1, i2,
-			      REAL(ans)[i] = myfloor(REAL(s1)[i1],
-						     R_INTEGER(s2,i2)););
+			       da[i] = myfloor(px1[i1], R_INTEGER(px2[i2])););
+	}
 	break;
     }
     UNPROTECT(1);
@@ -1101,7 +1196,6 @@ static SEXP real_binary(ARITHOP_TYPE code, SEXP s1, SEXP s2)
 static SEXP math1(SEXP sa, double(*f)(double), SEXP lcall)
 {
     SEXP sy;
-    double *y, *a;
     R_xlen_t i, n;
     int naflag;
 
@@ -1112,8 +1206,8 @@ static SEXP math1(SEXP sa, double(*f)(double), SEXP lcall)
     /* coercion can lose the object bit */
     PROTECT(sa = coerceVector(sa, REALSXP));
     PROTECT(sy = NO_REFERENCES(sa) ? sa : allocVector(REALSXP, n));
-    a = REAL(sa);
-    y = REAL(sy);
+    const double *a = REAL_RO(sa);
+    double *y = REAL(sy);
     naflag = 0;
     for (i = 0; i < n; i++) {
 	double x = a[i]; /* in case y == a */
@@ -1237,15 +1331,19 @@ SEXP attribute_hidden do_abs(SEXP call, SEXP op, SEXP args, SEXP env)
 	    x : allocVector(INTSXP, n);
 	PROTECT(s);
 	/* Note: relying on INTEGER(.) === LOGICAL(.) : */
+	int *pa = INTEGER(s);
+	const int *px = INTEGER_RO(x);
 	for(i = 0 ; i < n ; i++) {
-	    int xi = INTEGER(x)[i];
-	    INTEGER(s)[i] = (xi == NA_INTEGER) ? xi : abs(xi);
+	    int xi = px[i];
+	    pa[i] = (xi == NA_INTEGER) ? xi : abs(xi);
 	}
     } else if (TYPEOF(x) == REALSXP) {
 	R_xlen_t i, n = XLENGTH(x);
 	PROTECT(s = NO_REFERENCES(x) ? x : allocVector(REALSXP, n));
+	double *pa = REAL(s);
+	const double *px = REAL_RO(x);
 	for(i = 0 ; i < n ; i++)
-	    REAL(s)[i] = fabs(REAL(x)[i]);
+	    pa[i] = fabs(px[i]);
     } else if (isComplex(x)) {
 	SET_TAG(args, R_NilValue); /* cmathfuns want "z"; we might have "x" PR#16047 */
 	return do_cmathfuns(call, op, args, env);
@@ -1273,7 +1371,8 @@ static SEXP math2(SEXP sa, SEXP sb, double (*f)(double, double),
 {
     SEXP sy;
     R_xlen_t i, ia, ib, n, na, nb;
-    double ai, bi, *a, *b, *y;
+    double ai, bi, *y;
+    const double *a, *b;
     int naflag;
 
     if (!isNumeric(sa) || !isNumeric(sb))
@@ -1294,8 +1393,8 @@ static SEXP math2(SEXP sa, SEXP sb, double (*f)(double, double),
     PROTECT(sa = coerceVector(sa, REALSXP));		\
     PROTECT(sb = coerceVector(sb, REALSXP));		\
     PROTECT(sy = allocVector(REALSXP, n));		\
-    a = REAL(sa);					\
-    b = REAL(sb);					\
+    a = REAL_RO(sa);					\
+    b = REAL_RO(sb);					\
     y = REAL(sy);					\
     naflag = 0
 
@@ -1328,7 +1427,8 @@ static SEXP math2_1(SEXP sa, SEXP sb, SEXP sI,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, n, na, nb;
-    double ai, bi, *a, *b, *y;
+    double ai, bi, *y;
+    const double *a, *b;
     int m_opt;
     int naflag;
 
@@ -1357,7 +1457,8 @@ static SEXP math2_2(SEXP sa, SEXP sb, SEXP sI1, SEXP sI2,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, n, na, nb;
-    double ai, bi, *a, *b, *y;
+    double ai, bi, *y;
+    const double *a, *b;
     int i_1, i_2;
     int naflag;
     if (!isNumeric(sa) || !isNumeric(sb))
@@ -1388,7 +1489,8 @@ static SEXP math2B(SEXP sa, SEXP sb, double (*f)(double, double, double *),
 {
     SEXP sy;
     R_xlen_t i, ia, ib, n, na, nb;
-    double ai, bi, *a, *b, *y;
+    double ai, bi, *y;
+    const double *a, *b;
     int naflag;
     double amax, *work;
     size_t nw;
@@ -1695,9 +1797,9 @@ SEXP attribute_hidden do_log_builtin(SEXP call, SEXP op, SEXP args, SEXP env)
     PROTECT(sb = coerceVector(sb, REALSXP));			\
     PROTECT(sc = coerceVector(sc, REALSXP));			\
     PROTECT(sy = allocVector(REALSXP, n));			\
-    a = REAL(sa);						\
-    b = REAL(sb);						\
-    c = REAL(sc);						\
+    a = REAL_RO(sa);						\
+    b = REAL_RO(sb);						\
+    c = REAL_RO(sc);						\
     y = REAL(sy);						\
     naflag = 0
 
@@ -1714,7 +1816,8 @@ static SEXP math3_1(SEXP sa, SEXP sb, SEXP sc, SEXP sI,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, n, na, nb, nc;
-    double ai, bi, ci, *a, *b, *c, *y;
+    double ai, bi, ci, *y;
+    const double *a, *b, *c;
     int i_1;
     int naflag;
 
@@ -1742,7 +1845,8 @@ static SEXP math3_2(SEXP sa, SEXP sb, SEXP sc, SEXP sI, SEXP sJ,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, n, na, nb, nc;
-    double ai, bi, ci, *a, *b, *c, *y;
+    double ai, bi, ci, *y;
+    const double *a, *b, *c;
     int i_1,i_2;
     int naflag;
 
@@ -1773,7 +1877,8 @@ static SEXP math3B(SEXP sa, SEXP sb, SEXP sc,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, n, na, nb, nc;
-    double ai, bi, ci, *a, *b, *c, *y;
+    double ai, bi, ci, *y;
+    const double *a, *b, *c;
     int naflag;
     double amax, *work;
     size_t nw;
@@ -1902,7 +2007,8 @@ static SEXP math4(SEXP sa, SEXP sb, SEXP sc, SEXP sd,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, id, n, na, nb, nc, nd;
-    double ai, bi, ci, di, *a, *b, *c, *d, *y;
+    double ai, bi, ci, di, *y;
+    const double *a, *b, *c, *d;
     int naflag;
 
 #define SETUP_Math4							\
@@ -1924,10 +2030,10 @@ static SEXP math4(SEXP sa, SEXP sb, SEXP sc, SEXP sd,
     PROTECT(sc = coerceVector(sc, REALSXP));				\
     PROTECT(sd = coerceVector(sd, REALSXP));				\
     PROTECT(sy = allocVector(REALSXP, n));				\
-    a = REAL(sa);							\
-    b = REAL(sb);							\
-    c = REAL(sc);							\
-    d = REAL(sd);							\
+    a = REAL_RO(sa);							\
+    b = REAL_RO(sb);							\
+    c = REAL_RO(sc);							\
+    d = REAL_RO(sd);							\
     y = REAL(sy);							\
     naflag = 0
 
@@ -1964,7 +2070,8 @@ static SEXP math4_1(SEXP sa, SEXP sb, SEXP sc, SEXP sd, SEXP sI, double (*f)(dou
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, id, n, na, nb, nc, nd;
-    double ai, bi, ci, di, *a, *b, *c, *d, *y;
+    double ai, bi, ci, di, *y;
+    const double *a, *b, *c, *d;
     int i_1;
     int naflag;
 
@@ -1992,7 +2099,8 @@ static SEXP math4_2(SEXP sa, SEXP sb, SEXP sc, SEXP sd, SEXP sI, SEXP sJ,
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, id, n, na, nb, nc, nd;
-    double ai, bi, ci, di, *a, *b, *c, *d, *y;
+    double ai, bi, ci, di, *y;
+    const double *a, *b, *c, *d;
     int i_1, i_2;
     int naflag;
 
@@ -2074,7 +2182,8 @@ static SEXP math5(SEXP sa, SEXP sb, SEXP sc, SEXP sd, SEXP se, double (*f)())
 {
     SEXP sy;
     R_xlen_t i, ia, ib, ic, id, ie, n, na, nb, nc, nd, ne;
-    double ai, bi, ci, di, ei, *a, *b, *c, *d, *e, *y;
+    double ai, bi, ci, di, ei, *y;
+    const double *a, *b, *c, *d, *e;
 
 #define SETUP_Math5							\
     if (!isNumeric(sa) || !isNumeric(sb) || !isNumeric(sc) ||		\
@@ -2099,11 +2208,11 @@ static SEXP math5(SEXP sa, SEXP sb, SEXP sc, SEXP sd, SEXP se, double (*f)())
     PROTECT(sd = coerceVector(sd, REALSXP));				\
     PROTECT(se = coerceVector(se, REALSXP));				\
     PROTECT(sy = allocVector(REALSXP, n));				\
-    a = REAL(sa);							\
-    b = REAL(sb);							\
-    c = REAL(sc);							\
-    d = REAL(sd);							\
-    e = REAL(se);							\
+    a = REAL_RO(sa);							\
+    b = REAL_RO(sb);							\
+    c = REAL_RO(sc);							\
+    d = REAL_RO(sd);							\
+    e = REAL_RO(se);							\
     y = REAL(sy);							\
     naflag = 0
 
