@@ -64,6 +64,7 @@
 #include <config.h>
 #endif
 
+#define R_USE_SIGNALS 1
 #include "Defn.h"
 #include <Internal.h>
 #include "Print.h"
@@ -338,9 +339,10 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		break;
 	    case LGLSXP:
 		if (LENGTH(tmp) == 1) {
-		    formatLogical(LOGICAL(tmp), 1, &w);
+		    const int *x = LOGICAL_RO(tmp);
+		    formatLogical(x, 1, &w);
 		    snprintf(pbuf, 115, "%s",
-			     EncodeLogical(LOGICAL(tmp)[0], w));
+			     EncodeLogical(x[0], w));
 		} else
 		    snprintf(pbuf, 115, "Logical,%d", LENGTH(tmp));
 		break;
@@ -350,24 +352,26 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		    snprintf(pbuf, 115, "factor,%d", LENGTH(tmp));
 		} else {
 		    if (LENGTH(tmp) == 1) {
-			formatInteger(INTEGER(tmp), 1, &w);
+			const int *x = INTEGER_RO(tmp);
+			formatInteger(x, 1, &w);
 			snprintf(pbuf, 115, "%s",
-				 EncodeInteger(INTEGER(tmp)[0], w));
+				 EncodeInteger(x[0], w));
 		    } else
 			snprintf(pbuf, 115, "Integer,%d", LENGTH(tmp));
 		}
 		break;
 	    case REALSXP:
 		if (LENGTH(tmp) == 1) {
-		    formatReal(REAL(tmp), 1, &w, &d, &e, 0);
+		    const double *x = REAL_RO(tmp);
+		    formatReal(x, 1, &w, &d, &e, 0);
 		    snprintf(pbuf, 115, "%s",
-			     EncodeReal0(REAL(tmp)[0], w, d, e, OutDec));
+			     EncodeReal0(x[0], w, d, e, OutDec));
 		} else
 		    snprintf(pbuf, 115, "Numeric,%d", LENGTH(tmp));
 		break;
 	    case CPLXSXP:
 		if (LENGTH(tmp) == 1) {
-		    Rcomplex *x = COMPLEX(tmp);
+		    const Rcomplex *x = COMPLEX_RO(tmp);
 		    if (ISNA(x[0].r) || ISNA(x[0].i))
 			/* formatReal(NA) --> w=R_print.na_width, d=0, e=0 */
 			snprintf(pbuf, 115, "%s",
@@ -452,6 +456,18 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		       const char *ss = translateChar(STRING_ELT(names, i));
 		    */
 		    const char *ss = EncodeChar(STRING_ELT(names, i));
+#ifdef Win32
+		    /* FIXME: double translation to native encoding, in
+		         EncodeChar and translateChar; it is however necessary
+			 to call isValidName() on a string without Rgui
+			 escapes, because Rgui escapes cause a name to be
+			 regarded invalid;
+			 note also differences with printList
+		    */
+		    const char *st = ss;
+		    if (WinUTF8out)
+			st = translateChar(STRING_ELT(names, i));
+#endif
 		    if (taglen + strlen(ss) > TAGBUFLEN) {
 			if (taglen <= TAGBUFLEN)
 			    sprintf(ptag, "$...");
@@ -460,7 +476,11 @@ static void PrintGenericVector(SEXP s, SEXP env)
 			   is a valid (if non-syntactic) name */
 			if (STRING_ELT(names, i) == NA_STRING)
 			    sprintf(ptag, "$<NA>");
+#ifdef Win32
+			else if( isValidName(st) )
+#else
 			else if( isValidName(ss) )
+#endif
 			    sprintf(ptag, "$%s", ss);
 			else
 			    sprintf(ptag, "$`%s`", ss);
@@ -476,11 +496,18 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		}
 		Rprintf("%s\n", tagbuf);
 		if(isObject(VECTOR_ELT(s, i))) {
+		    SEXP x = VECTOR_ELT(s, i);
+		    int nprot = 0;
+		    if (TYPEOF(x) == LANGSXP) {
+			// quote(x)  to not accidentally evaluate it with newcall() below:
+			x = PROTECT(lang2(R_Primitive("quote"), x)); nprot++;
+		    }
 		    /* need to preserve tagbuf */
 		    strcpy(save, tagbuf);
-		    SETCADR(newcall, VECTOR_ELT(s, i));
+		    SETCADR(newcall, x);
 		    eval(newcall, env);
 		    strcpy(tagbuf, save);
+		    UNPROTECT(nprot);
 		}
 		else PrintValueRec(VECTOR_ELT(s, i), env);
 		*ptag = '\0';
@@ -629,8 +656,14 @@ static void printList(SEXP s, SEXP env)
 	    }
 	    Rprintf("%s\n", tagbuf);
 	    if(isObject(CAR(s))) {
-		SETCADR(newcall, CAR(s));
+		SEXP x = CAR(s);
+		int nprot = 0;
+		if (TYPEOF(x) == LANGSXP) {
+		    x = PROTECT(lang2(R_Primitive("quote"), x)); nprot++;
+		}
+		SETCADR(newcall, x);
 		eval(newcall, env);
+		UNPROTECT(nprot);
 	    }
 	    else PrintValueRec(CAR(s),env);
 	    *ptag = '\0';
@@ -681,7 +714,7 @@ static void PrintSpecial(SEXP s)
     if(s2 != R_UnboundValue) {
 	SEXP t;
 	PROTECT(s2);
-	t = deparse1(s2, 0, DEFAULTDEPARSE);
+	t = deparse1m(s2, 0, DEFAULTDEPARSE); // or deparse1() ?
 	Rprintf("%s ", CHAR(STRING_ELT(t, 0))); /* translated */
 	Rprintf(".Primitive(\"%s\")\n", PRIMNAME(s));
 	UNPROTECT(1);
@@ -690,6 +723,13 @@ static void PrintSpecial(SEXP s)
     UNPROTECT(1);
 }
 
+#ifdef Win32
+static void print_cleanup(void *data)
+{
+    WinUTF8out = *(Rboolean *)data;
+}
+#endif
+
 /* PrintValueRec -- recursively print an SEXP
 
  * This is the "dispatching" function for  print.default()
@@ -699,7 +739,18 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
     SEXP t;
 
 #ifdef Win32
+    RCNTXT cntxt;
+    Rboolean havecontext = FALSE;
+    Rboolean saveWinUTF8out = WinUTF8out;
+
     WinCheckUTF8();
+    if (WinUTF8out != saveWinUTF8out) {
+	begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
+	             R_NilValue, R_NilValue);
+	cntxt.cend = &print_cleanup;
+	cntxt.cenddata = &saveWinUTF8out;
+	havecontext = TRUE;
+    }
 #endif
     if(!isMethodsDispatchOn() && (IS_S4_OBJECT(s) || TYPEOF(s) == S4SXP) ) {
 	SEXP cl = getAttrib(s, R_ClassSymbol);
@@ -720,7 +771,7 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 			CHAR(STRING_ELT(cl, 0)), CHAR(STRING_ELT(pkg, 0)));
 	    }
 	}
-	return;
+	goto done;
     }
     switch (TYPEOF(s)) {
     case NILSXP:
@@ -728,7 +779,7 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 	break;
     case SYMSXP: /* Use deparse here to handle backtick quotification
 		  * of "weird names" */
-	t = deparse1(s, 0, SIMPLEDEPARSE);
+	t = deparse1(s, 0, SIMPLEDEPARSE); // TODO ? rather deparse1m()
 	Rprintf("%s\n", CHAR(STRING_ELT(t, 0))); /* translated */
 	break;
     case SPECIALSXP:
@@ -739,8 +790,8 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 	Rprintf("<CHARSXP: ");
 	Rprintf("%s", EncodeString(s, 0, '"', Rprt_adj_left));
 	Rprintf(">\n");
-	return; /* skip attribute printing for CHARSXP; they are used */
-		/* in managing the CHARSXP cache. */
+	goto done; /* skip attribute printing for CHARSXP; they are used */
+		   /* in managing the CHARSXP cache. */
     case EXPRSXP:
 	PrintExpression(s);
 	break;
@@ -761,7 +812,7 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 	break;
     case VECSXP:
 	PrintGenericVector(s, env); /* handles attributes/slots */
-	return;
+	goto done;
     case LISTSXP:
 	printList(s,env);
 	break;
@@ -833,9 +884,15 @@ void attribute_hidden PrintValueRec(SEXP s, SEXP env)
 	UNIMPLEMENTED_TYPE("PrintValueRec", s);
     }
     printAttributes(s, env, FALSE);
+
+done:
+
 #ifdef Win32
-    WinUTF8out = FALSE;
+    if (havecontext)
+	endcontext(&cntxt);
+    print_cleanup(&saveWinUTF8out);
 #endif
+    return; /* needed when Win32 is not defined */
 }
 
 /* 2000-12-30 PR#715: remove list tags from tagbuf here
@@ -933,14 +990,19 @@ static void printAttributes(SEXP s, SEXP env, Rboolean useSlots)
 		    na_width_noquote = R_print.na_width_noquote;
 		Rprt_adj right = R_print.right;
 
-		PROTECT(t = s = allocList(3));
+		SEXP x = CAR(a);
+		int nprot = 0;
+		if (TYPEOF(x) == LANGSXP) {
+		    x = PROTECT(lang2(R_Primitive("quote"), x)); nprot++;
+		}
+		PROTECT(t = s = allocList(3)); nprot++;
 		SET_TYPEOF(s, LANGSXP);
 		SETCAR(t, install("print")); t = CDR(t);
-		SETCAR(t,  CAR(a)); t = CDR(t);
+		SETCAR(t, x); t = CDR(t);
 		SETCAR(t, ScalarInteger(digits));
 		SET_TAG(t, install("digits"));
 		eval(s, env);
-		UNPROTECT(1);
+		UNPROTECT(nprot);
 		R_print.quote = quote;
 		R_print.right = right;
 		R_print.digits = digits;
