@@ -951,8 +951,9 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if(!R_FINITE(rfrom)) errorcall(call, _("'%s' must be a finite number"), "from");
 	if(!R_FINITE(rby))   errorcall(call, _("'%s' must be a finite number"), "by");
 	rto = rfrom + (double)(lout-1)*rby;
-	if(rby == (int)rby && rfrom <= INT_MAX && rfrom >= INT_MIN
-	   && rto <= INT_MAX && rto >= INT_MIN) {
+	if(rby == (int)rby && rfrom == (int)rfrom
+	   && rfrom <= INT_MAX && rfrom >= INT_MIN
+	   &&  rto  <= INT_MAX &&  rto  >= INT_MIN) {
 	    ans = allocVector(INTSXP, lout);
 	    for(i = 0; i < lout; i++) {
 //		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
@@ -970,8 +971,9 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    rfrom = rto - (double)(lout-1)*rby;
 	if(!R_FINITE(rto)) errorcall(call, _("'%s' must be a finite number"), "to");
 	if(!R_FINITE(rby)) errorcall(call, _("'%s' must be a finite number"), "by");
-	if(rby == (int)rby && rfrom <= INT_MAX && rfrom >= INT_MIN
-	   && rto <= INT_MAX && rto >= INT_MIN) {
+	if(rby == (int)rby && rto == (int)rto
+	   && rfrom <= INT_MAX && rfrom >= INT_MIN
+	   &&  rto  <= INT_MAX &&  rto  >= INT_MIN) {
 	    ans = allocVector(INTSXP, lout);
 	    for(i = 0; i < lout; i++) {
 //		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
