@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 1995--2015  The R Core Team
+ *  Copyright (C) 1995--2018  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <Rmath.h>
 #include <Fileio.h>
 #include <Rversion.h>
+#include <R_ext/Riconv.h>
 #include <R_ext/RS.h>           /* for CallocCharBuf, Free */
 #include <errno.h>
 #include <ctype.h>		/* for isspace */
@@ -56,10 +57,18 @@
  *    versions are stored as an integer packed by the R_Version macro
  *    from Rversion.h.  Some workspace formats may only exist
  *    temporarily in the development stage.  If readers are not
- *    provided in a release version, then these should specify the
+ *    provided in a released version, then these should specify the
  *    oldest reader R version as -1.
  */
-
+ 
+ /* It is now customary that the version (1, 2, 3) of the format is
+  * reflected also in magic numbers (such as RDX2, RDX3, ...), together with
+  * type (xdr/ascii/binary).  Adding a new serialization format thus now
+  * also requires adding a new set of magic numbers, yet in principle this
+  * could be changed in the future.  The code in this file does not need the
+  * magic numbers, it relies on version and type information in the
+  * serialization header (version 2 and 3).
+  */
 
 /* ----- V e r s i o n -- T w o -- S a v e / R e s t o r e ----- */
 
@@ -138,7 +147,7 @@
    consists of a function pointer and a data value.  The serialization
    function pointer is called with the reference object and the data
    value as arguments.  It should return R_NilValue for standard
-   handling and an STRSXP for special handling.  In an STRSXP is
+   handling and an STRSXP for special handling.  If an STRSXP is
    returned, then a special handing mark is written followed by the
    strings in the STRSXP (attributes are ignored).  On unserializing,
    any specially marked entry causes a call to the hook function with
@@ -154,6 +163,21 @@
    initialization can be separated--we don't really have that option
    at the R level.  */
 
+/* ----- V e r s i o n -- T h r e e -- S a v e / R e s t o r e ----- */
+
+/* This format extends version 2 format by adding an identifier of the
+   current native encoding to the serialization header.  On deserialization,
+   strings without an encoding flag will be converted to the current native
+   encoding, if possible, or to (flagged) UTF-8.  The conversion may fail
+   when the original encoding is not supported by iconv (unlikely) or when
+   the string is not valid in its declared encoding, which unfortunately is
+   not uncommon.  The conversion code now deliberately does not check
+   whether strings are valid when no conversion is needed, but such check
+   could be added in the future without changing the format.
+
+   Version 3 also adds support for custom ALTREP serialization. Under
+   version 2 ALTREP objects are serialied like non-ALTREP ones. */
+
 /*
  * Forward Declarations
  */
@@ -171,7 +195,22 @@ static SEXP ReadBC(SEXP ref_table, R_inpstream_t stream);
 /* The default version used when a stream Init function is called with
    version = 0 */
 
-static const int R_DefaultSerializeVersion = 2;
+static int defaultSerializeVersion()
+{
+    static int dflt = -1;
+
+    if (dflt < 0) {
+	char *valstr = getenv("R_DEFAULT_SERIALIZE_VERSION");
+	int val = -1;
+	if (valstr != NULL)
+	    val = atoi(valstr);
+	if (val == 2 || val == 3)
+	    dflt = val;
+	else
+	    dflt = 2; /* the default */
+    }
+    return dflt;
+}
 
 /*
  * Utility Functions
@@ -371,7 +410,7 @@ static int InInteger(R_inpstream_t stream)
     switch (stream->type) {
     case R_pstream_ascii_format:
 	InWord(stream, word, sizeof(word));
-	if(sscanf(word, "%s", buf) != 1) error(_("read error"));
+	if(sscanf(word, "%127s", buf) != 1) error(_("read error"));
 	if (strcmp(buf, "NA") == 0)
 	    return NA_INTEGER;
 	else
@@ -402,7 +441,7 @@ static double InReal(R_inpstream_t stream)
     switch (stream->type) {
     case R_pstream_ascii_format:
 	InWord(stream, word, sizeof(word));
-	if(sscanf(word, "%s", buf) != 1) error(_("read error"));
+	if(sscanf(word, "%127s", buf) != 1) error(_("read error"));
 	if (strcmp(buf, "NA") == 0)
 	    return NA_REAL;
 	else if (strcmp(buf, "NaN") == 0)
@@ -597,8 +636,8 @@ static void InFormat(R_inpstream_t stream)
 
 #define PTRHASH(obj) (((R_size_t) (obj)) >> 2)
 
-#define HASH_TABLE_COUNT(ht) TRUELENGTH(CDR(ht))
-#define SET_HASH_TABLE_COUNT(ht, val) SET_TRUELENGTH(CDR(ht), val)
+#define HASH_TABLE_COUNT(ht) ((int) TRUELENGTH(CDR(ht)))
+#define SET_HASH_TABLE_COUNT(ht, val) SET_TRUELENGTH(CDR(ht), ((int) (val)))
 
 #define HASH_TABLE_SIZE(ht) LENGTH(CDR(ht))
 
@@ -665,16 +704,18 @@ static int HashGet(SEXP item, SEXP ht)
 /* The following are needed to preserve attribute information on
    expressions in the constant pool of byte code objects. This is
    mainly for preserving source references attributes.  The original
-   implementation of the sharing-preserving writing and reading of yte
+   implementation of the sharing-preserving writing and reading of byte
    code objects did not account for the need to preserve attributes,
    so there is now a work-around using these SXP types to flag when
    the ATTRIB field has been written out. Object bits and S4 bits are
-   still not preserved.  It the long run in might be better to change
+   still not preserved.  In the long run it might be better to change
    to a scheme in which all sharing is preserved and byte code objects
    don't need to be handled as a special case.  LT */
 #define ATTRLANGSXP       240
 #define ATTRLISTSXP       239
 
+#define ALTREP_SXP	  238
+
 /*
  * Type/Flag Packing and Unpacking
  *
@@ -853,7 +894,7 @@ OutIntegerVec(R_outpstream_t stream, SEXP s, R_xlen_t length)
     switch (stream->type) {
     case R_pstream_xdr_format:
     {
-        static char buf[CHUNK_SIZE * sizeof(int)];
+	static char buf[CHUNK_SIZE * sizeof(int)];
 	R_xlen_t done, this;
 	XDR xdrs;
 	for (done = 0; done < length; done += this) {
@@ -890,10 +931,10 @@ OutRealVec(R_outpstream_t stream, SEXP s, R_xlen_t length)
     switch (stream->type) {
     case R_pstream_xdr_format:
     {
-        static char buf[CHUNK_SIZE * sizeof(double)];
+	static char buf[CHUNK_SIZE * sizeof(double)];
 	R_xlen_t done, this;
 	XDR xdrs;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    xdrmem_create(&xdrs, buf, (int)(this * sizeof(double)), XDR_ENCODE);
 	    for(int cnt = 0; cnt < this; cnt++)
@@ -907,7 +948,7 @@ OutRealVec(R_outpstream_t stream, SEXP s, R_xlen_t length)
     case R_pstream_binary_format:
     {
 	R_xlen_t done, this;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->OutBytes(stream, REAL(s) + done,
 			     (int)(sizeof(double) * this));
@@ -926,11 +967,11 @@ OutComplexVec(R_outpstream_t stream, SEXP s, R_xlen_t length)
     switch (stream->type) {
     case R_pstream_xdr_format:
     {
-        static char buf[CHUNK_SIZE * sizeof(Rcomplex)];
+	static char buf[CHUNK_SIZE * sizeof(Rcomplex)];
 	R_xlen_t done, this;
 	XDR xdrs;
 	Rcomplex *c = COMPLEX(s);
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    xdrmem_create(&xdrs, buf, (int)(this * sizeof(Rcomplex)), XDR_ENCODE);
 	    for(int cnt = 0; cnt < this; cnt++) {
@@ -946,7 +987,7 @@ OutComplexVec(R_outpstream_t stream, SEXP s, R_xlen_t length)
     case R_pstream_binary_format:
     {
 	R_xlen_t done, this;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->OutBytes(stream, COMPLEX(s) + done,
 			     (int)(sizeof(Rcomplex) * this));
@@ -964,10 +1005,12 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
     int i;
     SEXP t;
 
-    if (R_compile_pkgs && TYPEOF(s) == CLOSXP && TYPEOF(BODY(s)) != BCODESXP) {
+    if (R_compile_pkgs && TYPEOF(s) == CLOSXP && TYPEOF(BODY(s)) != BCODESXP &&
+        !R_disable_bytecode) {
+
 	SEXP new_s;
 	R_compile_pkgs = FALSE;
-	PROTECT(new_s = R_cmpfun(s));
+	PROTECT(new_s = R_cmpfun1(s));
 	WriteItem (new_s, ref_table, stream);
 	UNPROTECT(1);
 	R_compile_pkgs = TRUE;
@@ -976,6 +1019,22 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
 
  tailcall:
     R_CheckStack();
+    if (ALTREP(s) && stream->version >= 3) {
+	SEXP info = ALTREP_SERIALIZED_CLASS(s);
+	SEXP state = ALTREP_SERIALIZED_STATE(s);
+	if (info != NULL && state != NULL) {
+	    int flags = PackFlags(ALTREP_SXP, LEVELS(s), OBJECT(s), 0, 0);
+	    PROTECT(state);
+	    PROTECT(info);
+	    OutInteger(stream, flags);
+	    WriteItem(info, ref_table, stream);
+	    WriteItem(state, ref_table, stream);
+	    WriteItem(ATTRIB(s), ref_table, stream);
+	    UNPROTECT(2); /* state, info */
+	    return;
+	}
+	/* else fall through to standard processing */
+    }
     if ((t = GetPersistentName(stream, s)) != R_NilValue) {
 	R_assert(TYPEOF(t) == STRSXP && LENGTH(t) > 0);
 	PROTECT(t);
@@ -1309,6 +1368,17 @@ void R_Serialize(SEXP s, R_outpstream_t stream)
 	OutInteger(stream, R_VERSION);
 	OutInteger(stream, R_Version(2,3,0));
 	break;
+    case 3:
+    {
+	OutInteger(stream, version);
+	OutInteger(stream, R_VERSION);
+	OutInteger(stream, R_Version(3,5,0));
+	const char *natenc = R_nativeEncoding();
+	int nelen = (int) strlen(natenc);
+	OutInteger(stream, nelen);
+	OutString(stream, natenc, nelen);
+	break;
+    }
     default: error(_("version %d not supported"), version);
     }
 
@@ -1347,9 +1417,9 @@ static SEXP GetReadRef(SEXP table, int index)
 static void AddReadRef(SEXP table, SEXP value)
 {
     SEXP data = CAR(table);
-    int count = TRUELENGTH(data) + 1;
+    R_xlen_t count = TRUELENGTH(data) + 1;
     if (count >= LENGTH(data)) {
-	int i, len;
+	R_xlen_t i, len;
 	SEXP newdata;
 
 	PROTECT(value);
@@ -1388,10 +1458,10 @@ InIntegerVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     switch (stream->type) {
     case R_pstream_xdr_format:
     {
-        static char buf[CHUNK_SIZE * sizeof(int)];
+	static char buf[CHUNK_SIZE * sizeof(int)];
 	R_xlen_t done, this;
 	XDR xdrs;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->InBytes(stream, buf, (int)(sizeof(int) * this));
 	    xdrmem_create(&xdrs, buf, (int)(this * sizeof(int)), XDR_DECODE);
@@ -1405,7 +1475,7 @@ InIntegerVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     case R_pstream_binary_format:
     {
 	R_xlen_t done, this;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->InBytes(stream, INTEGER(obj) + done,
 			    (int)(sizeof(int) * this));
@@ -1424,10 +1494,10 @@ InRealVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     switch (stream->type) {
     case R_pstream_xdr_format:
     {
-        static char buf[CHUNK_SIZE * sizeof(double)];
+	static char buf[CHUNK_SIZE * sizeof(double)];
 	R_xlen_t done, this;
 	XDR xdrs;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->InBytes(stream, buf, (int)(sizeof(double) * this));
 	    xdrmem_create(&xdrs, buf, (int)(this * sizeof(double)), XDR_DECODE);
@@ -1441,7 +1511,7 @@ InRealVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     case R_pstream_binary_format:
     {
 	R_xlen_t done, this;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->InBytes(stream, REAL(obj) + done,
 			    (int)(sizeof(double) * this));
@@ -1460,7 +1530,7 @@ InComplexVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     switch (stream->type) {
     case R_pstream_xdr_format:
     {
-        static char buf[CHUNK_SIZE * sizeof(Rcomplex)];
+	static char buf[CHUNK_SIZE * sizeof(Rcomplex)];
 	R_xlen_t done, this;
 	XDR xdrs;
 	Rcomplex *output = COMPLEX(obj);
@@ -1480,7 +1550,7 @@ InComplexVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     case R_pstream_binary_format:
     {
 	R_xlen_t done, this;
-        for (done = 0; done < length; done += this) {
+	for (done = 0; done < length; done += this) {
 	    this = min2(CHUNK_SIZE, length - done);
 	    stream->InBytes(stream, COMPLEX(obj) + done,
 			    (int)(sizeof(Rcomplex) * this));
@@ -1493,6 +1563,136 @@ InComplexVec(R_inpstream_t stream, SEXP obj, R_xlen_t length)
     }
 }
 
+static int TryConvertString(void *obj, const char *inp, size_t inplen,
+                            char *buf, size_t *bufleft)
+{
+    if (Riconv(obj, NULL, NULL, &buf, bufleft) == -1)
+	return -1;
+    return (int) Riconv(obj, &inp, &inplen, &buf, bufleft);
+}
+
+static SEXP
+ConvertChar(void *obj, char *inp, size_t inplen, cetype_t enc)
+{
+    size_t buflen = inplen;
+
+    for(;;) {
+	size_t bufleft = buflen;
+	if (buflen < 1000) {
+	    char buf[buflen + 1];
+	    if (TryConvertString(obj, inp, inplen, buf, &bufleft) == -1) {
+		if (errno == E2BIG) {
+		    buflen *= 2;
+		    continue;
+		} else
+		    return R_NilValue;
+	    }
+	    return mkCharLenCE(buf, (int)(buflen - bufleft), enc);
+	} else {
+	    char *buf = CallocCharBuf(buflen);
+	    if (TryConvertString(obj, inp, inplen, buf, &bufleft) == -1) {
+		Free(buf);
+		if (errno == E2BIG) {
+		    buflen *= 2;
+		    continue;
+		} else
+		    return R_NilValue;
+	    }
+	    SEXP ans = mkCharLenCE(buf, (int)(buflen - bufleft), enc);
+	    Free(buf);
+	    return ans;
+	}
+    }
+}
+
+static char *native_fromcode(R_inpstream_t stream)
+{
+    char *from = stream->native_encoding;
+#ifdef HAVE_ICONV_CP1252
+    if (!strcmp(from, "ISO-8859-1"))
+	from = "CP1252";
+#endif
+    return from;
+}
+
+/* Read string into pre-allocated buffer, convert encoding if necessary, and
+   return a CHARSXP */
+static SEXP
+ReadChar(R_inpstream_t stream, char *buf, int length, int levs)
+{ 
+    InString(stream, buf, length);
+    buf[length] = '\0';
+    if (levs & UTF8_MASK)
+	return mkCharLenCE(buf, length, CE_UTF8);
+    if (levs & LATIN1_MASK)
+	return mkCharLenCE(buf, length, CE_LATIN1);
+    if (levs & BYTES_MASK)
+	return mkCharLenCE(buf, length, CE_BYTES);
+    if (levs & ASCII_MASK)
+	return mkCharLenCE(buf, length, CE_NATIVE);
+
+    /* native encoding, not ascii */
+    if (!stream->native_encoding[0] || /* original native encoding unknown */
+        (stream->nat2nat_obj == (void *)-1 && /* translation impossible or disabled */
+         stream->nat2utf8_obj == (void *)-1))
+	return mkCharLenCE(buf, length, CE_NATIVE);
+    /* try converting to native encoding */
+    if (!stream->nat2nat_obj &&
+        !strcmp(stream->native_encoding, R_nativeEncoding())) {
+	/* No translation needed. Performance optimization but also leaves
+	   invalid strings in their encoding undetected. */
+	stream->nat2nat_obj = (void *)-1;
+	stream->nat2utf8_obj = (void *)-1;
+#ifdef WARN_DESERIALIZE_INVALID_UTF8
+	if (known_to_be_utf8 && !utf8Valid(buf))
+	    warning(_("deserializing invalid UTF-8 string '%s'"), buf);
+#endif
+    }
+    if (!stream->nat2nat_obj) {
+	char *from = native_fromcode(stream);
+	stream->nat2nat_obj = Riconv_open("", from);
+	if (stream->nat2nat_obj == (void *)-1)
+	    warning(_("unsupported conversion from '%s' to '%s'"), from, "");
+    }
+    if (stream->nat2nat_obj != (void *)-1) {
+	cetype_t enc = CE_NATIVE;
+	if (known_to_be_utf8) enc = CE_UTF8;
+	else if (known_to_be_latin1) enc = CE_LATIN1;
+	SEXP ans = ConvertChar(stream->nat2nat_obj, buf, length, enc);
+	if (ans != R_NilValue)
+	    return ans;
+	if (known_to_be_utf8) {
+	    /* nat2nat_obj is converting to UTF-8, no need to use nat2utf8_obj */
+	    stream->nat2utf8_obj = (void *)-1;
+	    char *from = native_fromcode(stream);
+	    warning(_("input string '%s' cannot be translated to UTF-8, is it valid in '%s'?"),
+	            buf, from);
+	}
+    }
+    /* try converting to UTF-8 */
+    if (!stream->nat2utf8_obj) {
+	char *from = native_fromcode(stream);
+	stream->nat2utf8_obj = Riconv_open("UTF-8", from);
+	if (stream->nat2utf8_obj == (void *)-1) {
+	    /* very unlikely */
+	    warning(_("unsupported conversion from '%s' to '%s'"),
+	            from, "UTF-8");
+	    warning(_("strings not representable in native encoding will not be translated"));
+	} else
+	    warning(_("strings not representable in native encoding will be translated to UTF-8"));	
+    }
+    if (stream->nat2utf8_obj != (void *)-1) {
+	SEXP ans = ConvertChar(stream->nat2utf8_obj, buf, length, CE_UTF8);
+	if (ans != R_NilValue)
+	    return ans;
+	char *from = native_fromcode(stream);
+	warning(_("input string '%s' cannot be translated to UTF-8, is it valid in '%s' ?"),
+	        buf, from);
+    }
+    /* no translation possible */
+    return mkCharLenCE(buf, length, CE_NATIVE); 
+}
+
 static R_xlen_t ReadLENGTH (R_inpstream_t stream)
 {
     int len = InInteger(stream);
@@ -1560,11 +1760,22 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	UNPROTECT(1);
 	AddReadRef(ref_table, s);
 	return s;
+    case ALTREP_SXP:
+	{
+	    R_ReadItemDepth++;
+	    SEXP info = PROTECT(ReadItem(ref_table, stream));
+	    SEXP state = PROTECT(ReadItem(ref_table, stream));
+	    SEXP attr = PROTECT(ReadItem(ref_table, stream));
+	    s = ALTREP_UNSERIALIZE_EX(info, state, attr, objf, levs);
+	    UNPROTECT(3); /* info, state, attr */
+	    R_ReadItemDepth--;
+	    return s;
+	}
     case SYMSXP:
-        R_ReadItemDepth++;
+	R_ReadItemDepth++;
 	PROTECT(s = ReadItem(ref_table, stream)); /* print name */
 	R_ReadItemDepth--;
-	s = installChar(s);
+	s = installTrChar(s);
 	AddReadRef(ref_table, s);
 	UNPROTECT(1);
 	return s;
@@ -1684,22 +1895,11 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    if (length == -1)
 		PROTECT(s = NA_STRING);
 	    else if (length < 1000) {
-		int enc = CE_NATIVE;
 		char cbuf[length+1];
-		InString(stream, cbuf, length);
-		cbuf[length] = '\0';
-		if (levs & UTF8_MASK) enc = CE_UTF8;
-		else if (levs & LATIN1_MASK) enc = CE_LATIN1;
-		else if (levs & BYTES_MASK) enc = CE_BYTES;
-		PROTECT(s = mkCharLenCE(cbuf, length, enc));
+		PROTECT(s = ReadChar(stream, cbuf, length, levs));
 	    } else {
-		int enc = CE_NATIVE;
 		char *cbuf = CallocCharBuf(length);
-		InString(stream, cbuf, length);
-		if (levs & UTF8_MASK) enc = CE_UTF8;
-		else if (levs & LATIN1_MASK) enc = CE_LATIN1;
-		else if (levs & BYTES_MASK) enc = CE_BYTES;
-		PROTECT(s = mkCharLenCE(cbuf, length, enc));
+		PROTECT(s = ReadChar(stream, cbuf, length, levs));
 		Free(cbuf);
 	    }
 	    break;
@@ -1749,6 +1949,17 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	case RAWSXP:
 	    len = ReadLENGTH(stream);
 	    PROTECT(s = allocVector(type, len));
+	    switch (stream->type) {
+	    case R_pstream_ascii_format:
+		for (R_xlen_t ix = 0; ix < len; ix++) {
+		    char word[128];
+		    unsigned int i; // unsigned to avoid compiler warnings
+		    InWord(stream, word, sizeof(word));
+		    if(sscanf(word, "%2x", &i) != 1) error(_("read error"));
+		    RAW(s)[ix] = (Rbyte) i;
+		}
+		break;
+	    default:
 	    {
 		R_xlen_t done, this;
 		for (done = 0; done < len; done += this) {
@@ -1756,6 +1967,7 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 		    stream->InBytes(stream, RAW(s) + done, (int) this);
 		}
 	    }
+	    }
 	    break;
 	case S4SXP:
 	    PROTECT(s = allocS4Object());
@@ -1783,6 +1995,8 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    R_ReadItemDepth--;
 	}
 	UNPROTECT(1); /* s */
+	if (TYPEOF(s) == BCODESXP && !R_BCVersionOK(s))
+	    return R_BytecodeExpr(s);
 	return s;
     }
 }
@@ -1815,7 +2029,7 @@ static SEXP ReadBCLang(int type, SEXP ref_table, SEXP reps,
 	    PROTECT(ans = allocSExp(type));
 	    if (pos >= 0)
 		SET_VECTOR_ELT(reps, pos, ans);
-            R_ReadItemDepth++;
+	    R_ReadItemDepth++;
 	    if (hasattr)
 		SET_ATTRIB(ans, ReadItem(ref_table, stream));
 	    SET_TAG(ans, ReadItem(ref_table, stream));
@@ -1830,10 +2044,10 @@ static SEXP ReadBCLang(int type, SEXP ref_table, SEXP reps,
     default:
 	{
 	    R_ReadItemDepth++;
-            SEXP res = ReadItem(ref_table, stream);
-            R_ReadItemDepth--;
-            return res;
-        }
+	    SEXP res = ReadItem(ref_table, stream);
+	    R_ReadItemDepth--;
+	    return res;
+	}
     }
 }
 
@@ -1876,10 +2090,12 @@ static SEXP ReadBC1(SEXP ref_table, SEXP reps, R_inpstream_t stream)
     R_ReadItemDepth++;
     SETCAR(s, ReadItem(ref_table, stream)); /* code */
     R_ReadItemDepth--;
-    SETCAR(s, R_bcEncode(CAR(s)));
+    SEXP bytes = PROTECT(CAR(s));
+    SETCAR(s, R_bcEncode(bytes));
     SETCDR(s, ReadBCConsts(ref_table, reps, stream)); /* consts */
     SET_TAG(s, R_NilValue); /* expr */
-    UNPROTECT(1);
+    R_registerBC(bytes, s);
+    UNPROTECT(2);
     return s;
 }
 
@@ -1902,7 +2118,7 @@ static void DecodeVersion(int packed, int *v, int *p, int *s)
 SEXP R_Unserialize(R_inpstream_t stream)
 {
     int version;
-    int writer_version, release_version;
+    int writer_version, min_reader_version;
     SEXP obj, ref_table;
 
     InFormat(stream);
@@ -1910,18 +2126,29 @@ SEXP R_Unserialize(R_inpstream_t stream)
     /* Read the version numbers */
     version = InInteger(stream);
     writer_version = InInteger(stream);
-    release_version = InInteger(stream);
+    min_reader_version = InInteger(stream); 
     switch (version) {
     case 2: break;
+    case 3:
+    {
+	int nelen = InInteger(stream);
+	char nbuf[nelen + 1];
+	InString(stream, nbuf, nelen);
+	nbuf[nelen] = '\0';
+	nelen = nelen < (R_CODESET_MAX + 1) ? nelen : (R_CODESET_MAX + 1);
+	strncpy(stream->native_encoding, nbuf, nelen);
+	stream->native_encoding[nelen] = '\0';
+	break;
+    }
     default:
-	if (version != 2) {
+	{
 	    int vw, pw, sw;
 	    DecodeVersion(writer_version, &vw, &pw, &sw);
-	    if (release_version < 0)
+	    if (min_reader_version < 0)
 		error(_("cannot read unreleased workspace version %d written by experimental R %d.%d.%d"), version, vw, pw, sw);
 	    else {
 		int vm, pm, sm;
-		DecodeVersion(release_version, &vm, &pm, &sm);
+		DecodeVersion(min_reader_version, &vm, &pm, &sm);
 		error(_("cannot read workspace version %d written by R %d.%d.%d; need R %d.%d.%d or newer"),
 		      version, vw, pw, sw, vm, pm, sm);
 	    }
@@ -1931,11 +2158,83 @@ SEXP R_Unserialize(R_inpstream_t stream)
     /* Read the actual object back */
     PROTECT(ref_table = MakeReadRefTable());
     obj =  ReadItem(ref_table, stream);
+
+    if (version == 3) {
+	if (stream->nat2nat_obj && stream->nat2nat_obj != (void *)-1) {
+	    Riconv_close(stream->nat2nat_obj);
+	    stream->nat2nat_obj = NULL;
+	}
+	if (stream->nat2utf8_obj && stream->nat2utf8_obj != (void *)-1) {
+	    Riconv_close(stream->nat2utf8_obj);
+	    stream->nat2utf8_obj = NULL;
+	}
+    }
     UNPROTECT(1);
 
     return obj;
 }
 
+SEXP R_SerializeInfo(R_inpstream_t stream)
+{
+    int version;
+    int writer_version, min_reader_version, vv, vp, vs;
+    int anslen = 4;
+    SEXP ans, names;
+    char buf[128];
+
+    InFormat(stream);
+
+    /* Read the version numbers */
+    version = InInteger(stream);
+    if (version == 3)
+	anslen++;
+    writer_version = InInteger(stream);
+    min_reader_version = InInteger(stream);
+
+    PROTECT(ans = allocVector(VECSXP, anslen));
+    PROTECT(names = allocVector(STRSXP, anslen));
+    SET_STRING_ELT(names, 0, mkChar("version"));
+    SET_VECTOR_ELT(ans, 0, ScalarInteger(version));
+    SET_STRING_ELT(names, 1, mkChar("writer_version"));
+    DecodeVersion(writer_version, &vv, &vp, &vs);
+    snprintf(buf, 128, "%d.%d.%d", vv, vp, vs); 
+    SET_VECTOR_ELT(ans, 1, mkString(buf));
+    SET_STRING_ELT(names, 2, mkChar("min_reader_version"));
+    if (min_reader_version < 0)
+	/* unreleased version of R */
+	SET_VECTOR_ELT(ans, 2, ScalarString(NA_STRING));
+    else { 
+	DecodeVersion(min_reader_version, &vv, &vp, &vs);
+	snprintf(buf, 128, "%d.%d.%d", vv, vp, vs);
+	SET_VECTOR_ELT(ans, 2, mkString(buf));
+    }
+    SET_STRING_ELT(names, 3, mkChar("format"));
+    switch(stream->type) {
+    case R_pstream_ascii_format:
+	SET_VECTOR_ELT(ans, 3, mkString("ascii"));
+	break;
+    case R_pstream_binary_format:
+	SET_VECTOR_ELT(ans, 3, mkString("binary"));
+	break;
+    case R_pstream_xdr_format:
+	SET_VECTOR_ELT(ans, 3, mkString("xdr"));
+	break;
+    default:
+	error(_("unknown input format"));
+    }
+    if (version == 3) {
+	SET_STRING_ELT(names, 4, mkChar("native_encoding"));
+	int nelen = InInteger(stream);
+	char nbuf[nelen + 1];
+	InString(stream, nbuf, nelen);
+	nbuf[nelen] = '\0';
+	SET_VECTOR_ELT(ans, 4, mkString(nbuf));
+    }
+    setAttrib(ans, R_NamesSymbol, names);
+    UNPROTECT(2); /* ans, names */
+
+    return ans;
+}
 
 /*
  * Generic Persistent Stream Initializers
@@ -1954,6 +2253,9 @@ R_InitInPStream(R_inpstream_t stream, R_pstream_data_t data,
     stream->InBytes = inbytes;
     stream->InPersistHookFunc = phook;
     stream->InPersistHookData = pdata;
+    stream->native_encoding[0] = 0;
+    stream->nat2nat_obj = NULL;
+    stream->nat2utf8_obj = NULL; 
 }
 
 void
@@ -1965,7 +2267,7 @@ R_InitOutPStream(R_outpstream_t stream, R_pstream_data_t data,
 {
     stream->data = data;
     stream->type = type;
-    stream->version = version != 0 ? version : R_DefaultSerializeVersion;
+    stream->version = version != 0 ? version : defaultSerializeVersion();
     stream->OutChar = outchar;
     stream->OutBytes = outbytes;
     stream->OutPersistHookFunc = phook;
@@ -2059,10 +2361,10 @@ static void InBytesConn(R_inpstream_t stream, void *buf, int length)
 	if (stream->type == R_pstream_ascii_format) {
 	    char linebuf[4];
 	    unsigned char *p = buf;
-	    int i, ncread;
+	    int i;
 	    unsigned int res;
 	    for (i = 0; i < length; i++) {
-		ncread = Rconn_getline(con, linebuf, 3);
+		size_t ncread = Rconn_getline(con, linebuf, 3);
 		if (ncread != 2)
 		    error(_("error reading from ascii connection"));
 		if (!sscanf(linebuf, "%02x", &res))
@@ -2193,7 +2495,7 @@ do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
     else type = R_pstream_xdr_format;
 
     if (CADDDR(args) == R_NilValue)
-	version = R_DefaultSerializeVersion;
+	version = defaultSerializeVersion();
     else
 	version = asInteger(CADDDR(args));
     if (version == NA_INTEGER || version <= 0)
@@ -2233,13 +2535,14 @@ do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
     return R_NilValue;
 }
 
-/* Used from readRDS().
-   This became public in R 2.13.0, and that version added support for
+/* unserializeFromConn(conn, hook) used from readRDS().
+   It became public in R 2.13.0, and that version added support for
    connections internally */
 SEXP attribute_hidden
 do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    /* unserializeFromConn(conn, hook) */
+    /* 0 .. unserializeFromConn(conn, hook) */
+    /* 1 .. serializeInfoFromConn(conn) */
 
     struct R_inpstream_st in;
     Rconnection con;
@@ -2252,9 +2555,6 @@ do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
 
     con = getConnection(asInteger(CAR(args)));
 
-    fun = CADR(args);
-    hook = fun != R_NilValue ? CallHook : NULL;
-
     /* Now we need to do some sanity checking of the arguments.
        A filename will already have been opened, so anything
        not open was specified as a connection directly.
@@ -2274,14 +2574,19 @@ do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
     }
     if(!con->canread) error(_("connection not open for reading"));
 
+    fun = PRIMVAL(op) == 0 ? CADR(args) : R_NilValue;
+    hook = fun != R_NilValue ? CallHook : NULL;
     R_InitConnInPStream(&in, con, R_pstream_any_format, hook, fun);
-    PROTECT(ans = R_Unserialize(&in)); /* paranoia about next line */
-    if(!wasopen) {endcontext(&cntxt); con->close(con);}
-    UNPROTECT(1);
+    ans = PRIMVAL(op) == 0 ? R_Unserialize(&in) : R_SerializeInfo(&in);    
+    if(!wasopen) {
+	PROTECT(ans); /* paranoia about next line */
+	endcontext(&cntxt);
+	con->close(con);
+	UNPROTECT(1);
+    }
     return ans;
 }
 
-
 /*
  * Persistent Buffered Binary Connection Streams
  */
@@ -2347,7 +2652,7 @@ R_serializeb(SEXP object, SEXP icon, SEXP xdr, SEXP Sversion, SEXP fun)
     int version;
 
     if (Sversion == R_NilValue)
-	version = R_DefaultSerializeVersion;
+	version = defaultSerializeVersion();
     else version = asInteger(Sversion);
     if (version == NA_INTEGER || version <= 0)
 	error(_("bad version value"));
@@ -2496,7 +2801,7 @@ R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP Sversion, SEXP fun)
     int version;
 
     if (Sversion == R_NilValue)
-	version = R_DefaultSerializeVersion;
+	version = defaultSerializeVersion();
     else version = asInteger(Sversion);
     if (version == NA_INTEGER || version <= 0)
 	error(_("bad version value"));
@@ -2537,7 +2842,7 @@ R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP Sversion, SEXP fun)
     }
     else {
 	Rconnection con = getConnection(asInteger(icon));
-	R_InitConnOutPStream(&out, con, type, 0, hook, fun);
+	R_InitConnOutPStream(&out, con, type, version, hook, fun);
 	R_Serialize(object, &out);
 	return R_NilValue;
     }
@@ -2764,7 +3069,7 @@ static SEXP R_getVarsFromFrame(SEXP vars, SEXP env, SEXP forcesxp)
     len = LENGTH(vars);
     PROTECT(val = allocVector(VECSXP, len));
     for (i = 0; i < len; i++) {
-	sym = installChar(STRING_ELT(vars, i));
+	sym = installTrChar(STRING_ELT(vars, i));
 
 	tmp = findVarInFrame(env, sym);
 	if (tmp == R_UnboundValue) {
@@ -2775,11 +3080,10 @@ static SEXP R_getVarsFromFrame(SEXP vars, SEXP env, SEXP forcesxp)
 	if (force && TYPEOF(tmp) == PROMSXP) {
 	    PROTECT(tmp);
 	    tmp = eval(tmp, R_GlobalEnv);
-	    SET_NAMED(tmp, 2);
+	    ENSURE_NAMEDMAX(tmp);
 	    UNPROTECT(1);
 	}
-	else if (TYPEOF(tmp) != NILSXP && NAMED(tmp) < 1)
-	    SET_NAMED(tmp, 1);
+	else ENSURE_NAMED(tmp); /* should not really be needed - LT */
 	SET_VECTOR_ELT(val, i, tmp);
     }
     setAttrib(val, R_NamesSymbol, vars);
@@ -2854,7 +3158,7 @@ do_lazyLoadDBfetch(SEXP call, SEXP op, SEXP args, SEXP env)
     if (TYPEOF(val) == PROMSXP) {
 	REPROTECT(val, vpi);
 	val = eval(val, R_GlobalEnv);
-	SET_NAMED(val, 2);
+	ENSURE_NAMEDMAX(val);
     }
     UNPROTECT(1);
     return val;
