@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2017  The R Core Team
+ *  Copyright (C) 1997--2018  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,23 +55,9 @@ void NORET F77_SYMBOL(rexitc)(char *msg, int *nchar);
 }
 #endif
 
-/* Many small functions are included from ../include/Rinlinedfuns.h */
+#include <rlocale.h>
 
-attribute_hidden
-Rboolean tsConform(SEXP x, SEXP y)
-{
-    if ((x = getAttrib(x, R_TspSymbol)) != R_NilValue &&
-	(y = getAttrib(y, R_TspSymbol)) != R_NilValue) {
-	/* tspgets should enforce this, but prior to 2.4.0
-	   had INTEGER() here */
-	if(TYPEOF(x) == REALSXP && TYPEOF(y) == REALSXP)
-	    return REAL(x)[0] == REAL(x)[0] &&
-		REAL(x)[1] == REAL(x)[1] &&
-		REAL(x)[2] == REAL(x)[2];
-	/* else fall through */
-    }
-    return FALSE;
-}
+/* Many small functions are included from ../include/Rinlinedfuns.h */
 
 int nrows(SEXP s)
 {
@@ -269,13 +255,13 @@ void InitTypeTables(void) {
 	    SEXP rstr = ScalarString(rchar);
 	    MARK_NOT_MUTABLE(rstr);
 	    R_PreserveObject(rstr);
-	    UNPROTECT(1); /* rchar */
 	    SEXP rsym = install(cstr);
 
 	    Type2Table[type].cstrName = cstr;
 	    Type2Table[type].rcharName = rchar;
 	    Type2Table[type].rstrName = rstr;
 	    Type2Table[type].rsymName = rsym;
+	    UNPROTECT(1); /* rchar */
 	} else {
 	    Type2Table[type].cstrName = NULL;
 	    Type2Table[type].rcharName = NULL;
@@ -740,7 +726,7 @@ SEXP static intern_getwd(void)
 	wchar_t wbuf[PATH_MAX+1];
 	int res = GetCurrentDirectoryW(PATH_MAX, wbuf);
 	if(res > 0) {
-	    wcstoutf8(buf, wbuf, PATH_MAX+1);
+	    wcstoutf8(buf, wbuf, sizeof(buf));
 	    R_UTF8fixslash(buf);
 	    PROTECT(rval = allocVector(STRSXP, 1));
 	    SET_STRING_ELT(rval, 0, mkCharCE(buf, CE_UTF8));
@@ -803,7 +789,7 @@ SEXP attribute_hidden do_setwd(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_basename(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, s = R_NilValue;	/* -Wall */
-    char sp[4*PATH_MAX];
+    char sp[4*PATH_MAX+1];
     wchar_t  buf[PATH_MAX], *p;
     const wchar_t *pp;
     int i, n;
@@ -826,8 +812,7 @@ SEXP attribute_hidden do_basename(SEXP call, SEXP op, SEXP args, SEXP rho)
 		while (p >= buf && *p == L'/') *(p--) = L'\0';
 	    }
 	    if ((p = wcsrchr(buf, L'/'))) p++; else p = buf;
-	    memset(sp, 0, 4*PATH_MAX); /* safety */
-	    wcstoutf8(sp, p, 4*wcslen(p) + 1);
+	    wcstoutf8(sp, p, sizeof(sp));
 	    SET_STRING_ELT(ans, i, mkCharCE(sp, CE_UTF8));
 	}
     }
@@ -880,7 +865,7 @@ SEXP attribute_hidden do_dirname(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP ans, s = R_NilValue;	/* -Wall */
     wchar_t buf[PATH_MAX], *p;
     const wchar_t *pp;
-    char sp[4*PATH_MAX];
+    char sp[4*PATH_MAX+1];
     int i, n;
 
     checkArity(op, args);
@@ -909,7 +894,7 @@ SEXP attribute_hidden do_dirname(SEXP call, SEXP op, SEXP args, SEXP rho)
 			  && (p > buf+2 || *(p-1) != L':')) --p;
 		    p[1] = L'\0';
 		}
-		wcstoutf8(sp, buf, 4*wcslen(buf)+1);
+		wcstoutf8(sp, buf, sizeof(sp));
 	    }
 	    SET_STRING_ELT(ans, i, mkCharCE(sp, CE_UTF8));
 	}
@@ -1028,44 +1013,26 @@ SEXP attribute_hidden do_normalizepath(SEXP call, SEXP op, SEXP args, SEXP rho)
 #ifdef USE_INTERNAL_MKTIME
 const char *getTZinfo(void)
 {
-    const char *p = getenv("TZ");
-    if(p) return p;
-#ifdef HAVE_REALPATH
-    // This works on Linux, macOS and *BSD: other known OSes set TZ.
     static char def_tz[PATH_MAX+1] = "";
-    if(def_tz[0]) return def_tz;
+    if (def_tz[0]) return def_tz;
 
-    char abspath[PATH_MAX + 1];
-    const char* lt = realpath("/etc/localtime", abspath);
-# ifdef __APPLE__
-    // macOS <= 10.12 expands to /usr/share/zoneinfo/<zone name>
-    // macOS 10.13 expands to /usr/share/zoneinfo.default/<zone name>
-    // but 10.13.1 expands to /private/var/db/timezone/tz/2017c.1.0/zoneinfo/<zone name>
-    if(lt) {
-	if(strstr(abspath, ".default/"))
-	    strncpy(def_tz, abspath + 28, PATH_MAX);
-	else {
-	    // So guess is of the form .../zoneinfo/<real tz>
-	    p = strstr(abspath, "/zoneinfo/");
-	    if(p) {
-		strncpy(def_tz, p + 10, PATH_MAX);
-	    } else {
-		warning("system timezone name is unknown: set environment variable TZ");
-		return "unknown";
-	    }
+    // call Sys.timezone()
+    SEXP expr = PROTECT(install("Sys.timezone"));
+    SEXP call = PROTECT(lang1(expr));
+    SEXP ans = PROTECT(eval(call, R_GlobalEnv));
+    if(TYPEOF(ans) == STRSXP && LENGTH(ans) == 1) {
+	SEXP el = STRING_ELT(ans, 0);
+	if (el != NA_STRING) {
+	    strcpy(def_tz, CHAR(el));
+	    // printf("tz is %s\n", CHAR(el));
+	    UNPROTECT(3);
+	    return def_tz;
 	}
-//	printf("abspath = %s\n", abspath); printf("def_tz = %s\n", def_tz);
-	return def_tz;
     }
-# else
-    if(lt) {
-	strncpy(def_tz, abspath + 20, PATH_MAX); // strip "/usr/share/zoneinfo/"
-	return def_tz;
-    }
-# endif
-#endif
+    UNPROTECT(3);
     warning("system timezone name is unknown: set environment variable TZ");
-    return "unknown";
+    strcpy(def_tz, "unknown");  // code will then use TZDEFAULT, which is "UTC"
+    return def_tz;
 }
 #endif
 
@@ -1224,8 +1191,27 @@ int attribute_hidden utf8clen(char c)
     return 1 + utf8_table4[c & 0x3f];
 }
 
-/* These return the result in wchar_t, but does assume
-   wchar_t is UCS-2/4 and so are for internal use only */
+static Rwchar_t
+utf16toucs(wchar_t high, wchar_t low) 
+{
+    return 0x10000 + ((int) (high & 0x3FF) << 10 ) + (int) (low & 0x3FF);
+}
+
+/* Return the low UTF-16 surrogate from a UTF-8 string; assumes all testing has been done. */
+static wchar_t
+utf8toutf16low(const char *s)
+{
+    return (unsigned int) LOW_SURROGATE_START | ((s[2] & 0x0F) << 6) | (s[3] & 0x3F);
+}
+
+Rwchar_t attribute_hidden
+utf8toucs32(wchar_t high, const char *s)
+{
+    return utf16toucs(high, utf8toutf16low(s));
+}
+
+/* These return the result in wchar_t.  If wchar_t is 16 bit (e.g. UTF-16LE on Windows
+   only the high surrogate is returned; call utf8toutf16low next. */
 size_t attribute_hidden
 utf8toucs(wchar_t *wc, const char *s)
 {
@@ -1258,17 +1244,24 @@ utf8toucs(wchar_t *wc, const char *s)
 	    if(byte == 0xFFFE || byte == 0xFFFF) return (size_t)-1;
 	    return 3;
 	} else return (size_t)-1;
-    }
-    if(sizeof(wchar_t) < 4) return (size_t)-2;
-    /* So now handle 4,5.6 byte sequences with no testing */
-    if (byte < 0xf8) {
+    
+    } else if (byte < 0xf8) {
 	if(strlen(s) < 4) return (size_t)-2;
-	*w = (wchar_t) (((byte & 0x0F) << 18)
+	if (((s[1] & 0xC0) == 0x80) && ((s[2] & 0xC0) == 0x80) && ((s[3] & 0xC0) == 0x80)) {
+	    unsigned int cvalue = (((byte & 0x0F) << 18)
 			| (unsigned int) ((s[1] & 0x3F) << 12)
 			| (unsigned int) ((s[2] & 0x3F) << 6)
 			| (s[3] & 0x3F));
-	return 4;
-    } else if (byte < 0xFC) {
+	    if(sizeof(wchar_t) < 4) /* Assume UTF-16 and return high surrogate.  Users need to call utf8toutf16low next. */
+		*w = (wchar_t) ((cvalue - 0x10000) >> 10) | 0xD800;
+	    else
+		*w = (wchar_t) cvalue;
+	    return 4;
+	} else return (size_t)-1;
+    }
+    if(sizeof(wchar_t) < 4) return (size_t)-2;
+    /* So now handle 5.6 byte sequences with no testing */
+    if (byte < 0xFC) {
 	if(strlen(s) < 5) return (size_t)-2;
 	*w = (wchar_t) (((byte & 0x0F) << 24)
 			| (unsigned int) ((s[1] & 0x3F) << 12)
@@ -1303,6 +1296,11 @@ utf8towcs(wchar_t *wc, const char *s, size_t n)
 	    if (m == 0) break;
 	    res ++;
 	    if (res >= n) break;
+	    if (IS_HIGH_SURROGATE(*p)) {
+	    	*(++p) = utf8toutf16low(t);
+	    	res ++;
+	    	if (res >= n) break;
+	    }
 	}
     else
 	for(t = s; ; res++, t += m) {
@@ -1318,50 +1316,54 @@ static const unsigned int utf8_table1[] =
   { 0x7f, 0x7ff, 0xffff, 0x1fffff, 0x3ffffff, 0x7fffffff};
 static const unsigned int utf8_table2[] = { 0, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc};
 
-static size_t Rwcrtomb(char *s, const wchar_t wc)
+/* s is NULL, or it contains at least n bytes.  Just write a a terminator if it's not big enough. */
+
+static size_t Rwcrtomb32(char *s, Rwchar_t cvalue, size_t n)
 {
     register size_t i, j;
-    unsigned int cvalue = (unsigned int) wc;
-    char buf[10], *b;
-
-    b = s ? s : buf;
-    if(cvalue == 0) {*b = 0; return 0;}
+    if (!n) return 0;
+    if (s) *s = 0;    /* Simplifies exit later */
+    if(cvalue == 0) return 0;
     for (i = 0; i < sizeof(utf8_table1)/sizeof(int); i++)
 	if (cvalue <= utf8_table1[i]) break;
-    b += i;
-    for (j = i; j > 0; j--) {
-	*b-- = (char) (0x80 | (cvalue & 0x3f));
-	cvalue >>= 6;
+    if (i >= n - 1) return 0;  /* need space for terminal null */
+    if (s) {
+    	s += i;
+    	for (j = i; j > 0; j--) {
+	    *s-- = (char) (0x80 | (cvalue & 0x3f));
+	    cvalue >>= 6;
+        }
+    	*s = (char) (utf8_table2[i] | cvalue);
     }
-    *b = (char) (utf8_table2[i] | cvalue);
     return i + 1;
 }
 
+/* on input, wc is a string encoded in UTF-16 or UCS-2 or UCS-4.
+   s can be a buffer of size n>=0 chars, or NULL.  If n=0 or s=NULL, nothing is written. 
+   The return value is the number of chars including the terminating null.  If the
+   buffer is not big enough, the result is truncated but still null-terminated */
 attribute_hidden // but used in windlgs
 size_t wcstoutf8(char *s, const wchar_t *wc, size_t n)
 {
-    ssize_t m, res=0;
+    size_t m, res=0;
     char *t;
     const wchar_t *p;
-    if(s) {
-	for(p = wc, t = s; ; p++) {
-	    m  = (ssize_t) Rwcrtomb(t, *p);
-	    if(m <= 0) break;
-	    res += m;
-	    if(res >= n) break;
+    if (!n) return 0;
+    for(p = wc, t = s; ; p++) {
+    	if (IS_SURROGATE_PAIR(*p, *(p+1))) {
+    	    Rwchar_t cvalue =  ((*p & 0x3FF) << 10) + (*(p+1) & 0x3FF) + 0x010000;
+	    m = Rwcrtomb32(t, cvalue, n - res);
+	    p++;
+    	} else 
+    	    m = Rwcrtomb32(t, (Rwchar_t)(*p), n - res);
+    	if (!m) break;
+	res += m;
+	if (t)
 	    t += m;
-	}
-    } else {
-	for(p = wc; ; p++) {
-	    m  = (ssize_t) Rwcrtomb(NULL, *p);
-	    if(m <= 0) break;
-	    res += m;
-	}
     }
-    return (size_t) res;
+    return res + 1;
 }
 
-
 /* A version that reports failure as an error */
 size_t Mbrtowc(wchar_t *wc, const char *s, size_t n, mbstate_t *ps)
 {
