@@ -1,7 +1,8 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000, 2001 The R Development Core Team
+ *  Copyright (C) 2000-2006 The R Core Team
+ *  Copyright (C) 2004	    The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +15,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA.
+ *  along with this program; if not, a copy is available at
+ *  https://www.R-project.org/Licenses/
  *
  *  DESCRIPTION
  *
@@ -31,12 +32,19 @@ double pgeom(double x, double p, int lower_tail, int log_p)
     if (ISNAN(x) || ISNAN(p))
 	return x + p;
 #endif
-    x = floor(x+1e-7);
-    if(p < 0 || p > 1) ML_ERR_return_NAN;
+    if(p <= 0 || p > 1) ML_ERR_return_NAN;
 
-    if (x < 0. || p == 0.) return R_DT_0;
+    if (x < 0.) return R_DT_0;
     if (!R_FINITE(x)) return R_DT_1;
-    if(log_p && !lower_tail)
-	return log(1 - p) * (x + 1);
-    return R_DT_Cval(pow(1 - p, x + 1));
+    x = floor(x +1e-7);
+
+    if(p == 1.) { /* we cannot assume IEEE */
+	x = lower_tail ? 1: 0;
+	return log_p ? log(x) : x;
+    }
+    x = log1p(-p) * (x + 1);
+    if (log_p)
+	return R_DT_Clog(x);
+    else
+	return lower_tail ? -expm1(x) : exp(x);
 }
