@@ -1,7 +1,7 @@
 /*
  *  Mathlib : A C Library of Special Functions
  *  Copyright (C) 1998 Ross Ihaka
- *  Copyright (C) 2000-2002 The R Core Team
+ *  Copyright (C) 2000-2015 The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,10 +36,7 @@ double pweibull(double x, double shape, double scale, int lower_tail, int log_p)
     if (x <= 0)
 	return R_DT_0;
     x = -pow(x / scale, shape);
-    if (lower_tail)
-	return (log_p
-		/* log(1 - exp(x))  for x < 0 : */
-		? R_Log1_Exp(x) : -expm1(x));
-    /* else:  !lower_tail */
-    return R_D_exp(x);
+    return lower_tail
+	? (log_p ? R_Log1_Exp(x) : -expm1(x))
+	: R_D_exp(x);
 }
