@@ -906,6 +906,23 @@ BAR <- new("bar")
 stopifnot(identical(BAR[1L], FALSE))
 stopifnot(identical(BAR[1L, , flag=TRUE], TRUE))
 
+## avoid infinite recursion on Ops,structure methods
+setClass("MyInteger",
+         representation("integer")
+         )
+i <- new("MyInteger", 1L)
+m <- matrix(rnorm(300), 30,10)
+stopifnot(identical(i*m, m))
+
+## when rematching, do not drop arg with NULL default
+setGeneric("genericExtraArg",
+           function(x, y, extra) standardGeneric("genericExtraArg"),
+           signature="x")
+
+setMethod("genericExtraArg", "ANY", function(x, y=NULL) y)
+
+stopifnot(identical(genericExtraArg("foo", 1L), 1L))
+
 ## callNextMethod() was broken for ... dispatch
 f <- function(...) length(list(...))
 setGeneric("f")
@@ -913,6 +930,47 @@ setMethod("f", "character", function(...){ callNextMethod() })
 stopifnot(identical(f(1, 2, 3), 3L))
 stopifnot(identical(f("a", "b", "c"), 3L))
 
+## ... dispatch was evaluating missing arguments in the generic frame
+f <- function(x, ..., a = b) {
+    b <- "a"
+    a
+}
+setGeneric("f", signature = "...")
+stopifnot(identical(f(a=1), 1))
+stopifnot(identical(f(), "a"))
+
 ## ensure forwarding works correctly for dots dispatch
 f2 <- function(...) f(...)
-stopifnot(identical(f2(a=1L), 1L))
+stopifnot(identical(f2(a=1), 1))
+
+
+## R's internal C  R_check_class_and_super()  was not good enough
+if(require("Matrix")) withAutoprint({
+    setClass("Z", representation(zz = "list"))
+    setClass("C", contains = c("Z", "dgCMatrix"))
+    setClass("C2", contains = "C")
+    setClass("C3", contains = "C2")
+    m <- matrix(c(0,0,2:0), 3,5, dimnames = list(NULL,NULL))
+    (mC <- as(m, "dgCMatrix"))
+    (cc <- as(mC, "C"))
+     c2 <- as(mC, "C2")
+     c3 <- as(mC, "C3")
+    stopifnot(
+        identical(capture.output(c2),
+                  sub("C3","C2", capture.output(c3)))
+      , identical(as(cc, "matrix"), m)
+      , identical(as(c2, "matrix"), m)
+      , identical(as(c3, "matrix"), m)
+    )
+    invisible(lapply(c("Z","C","C2","C3"), removeClass))
+})
+
+
+## Automatic coerce method creation: 
+setClass("A", slots = c(foo = "numeric"))
+setClass("Ap", contains = "A", slots = c(p = "character"))
+cd <- getClassDef("Ap")
+body(cd@contains[["A"]]@coerce)[[2]] ## >>   value <- new("A")
+## was ... <-  new(structure("A", package = ".GlobalEnv"))
+## for a few days in R-devel (Nov.2017)
+
